@@ -1,7 +1,9 @@
 /* -*- mode: C; mode: fold; -*- */
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 
 #include "config.h"
@@ -10,9 +12,11 @@
 
 #include <stdio.h>
 #include <setjmp.h>
-#if defined (msdos) || defined (__os2_16__)
+#if defined (__MSDOS__) || defined (__os2_16__)
 # include <process.h>
-# include <dos.h>
+# ifndef __MINGW32__
+#  include <dos.h>
+# endif
 #endif
 #include <limits.h>
 #include <string.h>
@@ -63,7 +67,7 @@ Buffer *CBuf;
 Line *CLine;
 Buffer_Local_Type Buffer_Local = /*{{{*/
 {
-   8, 0
+   8
 };
 
 /*}}}*/
@@ -415,27 +419,26 @@ static Line *create_line_from_bunch(void) /*{{{*/
    
    /* failed so now malloc new bunch */
    
-   if (NULL != (b = (Bunch_Lines_Type *) SLMALLOC(sizeof(Bunch_Lines_Type))))
+   if (NULL == (b = (Bunch_Lines_Type *) SLmalloc (sizeof(Bunch_Lines_Type))))
+     return NULL;
+   
+   if (Bunch_Lines == NULL)
      {
-	if (Bunch_Lines == NULL)
-	  {
-	     Bunch_Lines = b;
-	     b->next = b;
-	  }
-	else
-	  {
-	     b->next = Bunch_Lines->next;
-	     Bunch_Lines->next = b;
-	     Bunch_Lines = b;
-	  }
-	
-	b->flags = MAX_LONG;
-	Next_Free_Offset = 1;
-	b->flags &= ~(unsigned long) 1;
-	Number_Freed += BUNCH_SIZE - 1;
-	return(&b->lines[0]);
+	Bunch_Lines = b;
+	b->next = b;
      }
-   return(NULL);
+   else
+     {
+	b->next = Bunch_Lines->next;
+	Bunch_Lines->next = b;
+	Bunch_Lines = b;
+     }
+   
+   b->flags = MAX_LONG;
+   Next_Free_Offset = 1;
+   b->flags &= ~(unsigned long) 1;
+   Number_Freed += BUNCH_SIZE - 1;
+   return(&b->lines[0]);
 }
 
 /*}}}*/
@@ -461,7 +464,7 @@ static void destroy_bunch_line(Line *l) /*{{{*/
      {
 	ll = b->lines;
 	if (
-#if (defined (msdos) && !defined(__WATCOMC__) && !defined(__WIN32__)) || defined (__os2_16__)
+#if defined(SIXTEEN_BIT_SYSTEM)
 	    /* stupid DOS and its memory segmentation forces me to consider
 	     segment then offset */
 	    (FP_SEG(ll) == FP_SEG(l)) &&
@@ -501,7 +504,7 @@ static void destroy_bunch_line(Line *l) /*{{{*/
 		       Next_Free_Offset = BUNCH_SIZE;
 		    }
 		  
-		  SLFREE(b);
+		  SLfree ((char *)b);
 		  
 		  if (last == b) last = NULL;
 		  b = NULL;
@@ -531,11 +534,10 @@ Line *make_line1(int size) /*{{{*/
 {
    Line *new_line;
    unsigned char *data = NULL;
-   char buff[80];
    unsigned int chunk;
    
    /* 4 byte chunks */
-#if defined(msdos) && !defined(__WIN32__)
+#if defined(SIXTEEN_BIT_SYSTEM)
    chunk = (unsigned) (size + 3) & 0xFFFCU;
 #else
    chunk = ((unsigned) (size + 3)) & 0xFFFFFFFCU;
@@ -548,14 +550,13 @@ Line *make_line1(int size) /*{{{*/
 	     data = NewLine_Buffer;
 	     chunk = 1;
 	  }
-	else data = (unsigned char *) SLMALLOC(chunk);   /* was chunk + 1 */
+	else data = (unsigned char *) SLmalloc (chunk);   /* was chunk + 1 */
      }
    
    if ((new_line == NULL) || (data == NULL))
      {
 	*Error_Buffer = 0;	       /* this is critical */
-	sprintf(buff, "Malloc Error in make_line: requested size: %d.", size);
-	msg_error(buff);
+	jed_verror ("Malloc Error in make_line: requested size: %d.", size);
 	longjmp(Jump_Buffer_Ptr->b, 1);
 	/* exit_error(buff); */
      }
@@ -621,7 +622,7 @@ void free_line(Line *line) /*{{{*/
 {
    register unsigned char *dat = line->data;
    
-   if (dat != NewLine_Buffer) SLFREE(dat);
+   if (dat != NewLine_Buffer) SLfree ((char *)dat);
    destroy_bunch_line(line);
 }
 
@@ -663,10 +664,9 @@ int delete_line() /*{{{*/
 
 unsigned char *remake_line(int size) /*{{{*/
 {
-   char buf[80];
    unsigned char *d = CLine->data;
    unsigned int mask;
-#if defined(msdos) && !defined(__WIN32__)
+#if defined(SIXTEEN_BIT_SYSTEM)
    mask = 0xFFFCu;
 #else
    mask = 0xFFFFFFFCu;
@@ -675,7 +675,7 @@ unsigned char *remake_line(int size) /*{{{*/
    
    if (d == NewLine_Buffer)
      {
-	if (NULL != (d = (unsigned char *) SLMALLOC(size))) *d = '\n';
+	if (NULL != (d = (unsigned char *) SLmalloc (size))) *d = '\n';
      }
    else
      {
@@ -684,14 +684,13 @@ unsigned char *remake_line(int size) /*{{{*/
 	if (((CLine->len + 3) & mask) == size) return d;
 #endif
 #endif
-	d = (unsigned char *) SLREALLOC(d, size);
+	d = (unsigned char *) SLrealloc ((char *) d, size);
      }
    
    if (d == NULL)
      {
 	*Error_Buffer = 0;	       /* critical error */
-	sprintf(buf, "remake_line: realloc error!, size = %d", size);
-	msg_error(buf);
+	jed_verror ("remake_line: realloc error!, size = %d", size);
 	longjmp(Jump_Buffer_Ptr->b, 1);
 	/* exit_error(buf); */
      }
@@ -705,52 +704,58 @@ unsigned char *remake_line(int size) /*{{{*/
 
 /*}}}*/
 
-void uniquely_name_buffer(char *trry) /*{{{*/
+void uniquely_name_buffer (Buffer *b, char *trry) /*{{{*/
 {
-   Buffer *b;
-   int exists, version = 0, n;
-   char neew[48];
+   Buffer *bnext;
+   int version = 0, n;
+   char neew[50];
+   char *name;
    
-   *CBuf->name = 0;
    strncpy(neew, trry, 45);  neew[45] = 0;
    n = strlen(neew);
    while (1)
      {
-	exists = 0;
-	b = CBuf->next;
-	while(b != CBuf)
+	bnext = b->next;
+	while (bnext != b)
 	  {
-	     if (!strcmp(neew, b->name)) exists = 1;
-	     b = b->next;
+	     if ((bnext->name != NULL)
+		 && (0 == strcmp(neew, bnext->name)))
+	       break;
+
+	     bnext = bnext->next;
 	  }
-	if (!exists)
+	
+	if (bnext == b)
 	  {
-	     strcpy(CBuf->name,neew);
-	     return;
+	     name = SLang_create_slstring (neew);
+	     if (name == NULL)
+	       exit_error ("Out of memory", 0);
+	     SLang_free_slstring (b->name);
+	     b->name = name;
+	     break;
 	  }
+
 	version++;
-	sprintf(neew + n, "<%d>", version);
+	sprintf (neew + n, "<%d>", version);
      }
 }
 
 /*}}}*/
 
 /* make a buffer and insert it in the list */
-Buffer *make_buffer (void) /*{{{*/
+Buffer *make_buffer (char *name, char *dir, char *file) /*{{{*/
 {
    Buffer *newB;
    
-   /* SLCALLOC used since it zeros region too */
-   newB = (Buffer *) SLCALLOC(sizeof(Buffer), 1);
-   if (newB == (Buffer *) NULL)
-     {
-	exit_error("make_buffer: malloc error", 0);
-     }
+   newB = (Buffer *) SLmalloc (sizeof(Buffer));
    
+   if (newB == NULL)
+     exit_error("Out of memory", 0);
+
    SLMEMSET ((char *) newB, 0, sizeof(Buffer));
-   
+
    newB->keymap = Global_Map;
-   newB->c_time = sys_time();
+   jed_set_buffer_ctime (newB);
    newB->local_vars.tab = Jed_Tab_Default;
 #if JED_HAS_ABBREVS
    newB->abbrev_table_handle = -1;
@@ -769,11 +774,20 @@ Buffer *make_buffer (void) /*{{{*/
 	CBuf->prev = newB;
      }
    
-#ifdef pc_system
+#ifdef IBMPC_SYSTEM
    newB->flags |= ADD_CR_ON_WRITE_FLAG;
 #endif
    newB->syntax_table = Default_Syntax_Table;
-   return(newB);
+
+   buffer_filename (newB, dir, file);
+   if ((name != NULL)
+       || (newB->name == NULL))
+     {
+	if (name == NULL) name = "*scratch*";
+	uniquely_name_buffer (newB, name);
+     }
+
+   return (newB);
 }
 
 /*}}}*/
@@ -792,14 +806,14 @@ void delete_buffer(Buffer *buf) /*{{{*/
      {
 	n = l -> next;
 	free_line(l);
-	/* SLFREE( l->data); SLFREE( l); */
+	/* SLfree( l->data); SLfree( l); */
      }
 
    m = buf->mark_array;
    while (m != NULL)
      {
 	m1 = m->next;
-	SLFREE (m);
+	SLfree ((char *)m);
 	m = m1;
      }
    
@@ -807,10 +821,10 @@ void delete_buffer(Buffer *buf) /*{{{*/
    while (m != NULL)
      {
 	m1 = m->next;
-	SLFREE (m);
+	SLfree ((char *)m);
 	m = m1;
      }
-   
+
    if (buf->user_marks != NULL) free_user_marks (buf);
    
    if (buf->undo != NULL) delete_undo_ring (buf);
@@ -821,20 +835,29 @@ void delete_buffer(Buffer *buf) /*{{{*/
    
 #if JED_HAS_SAVE_NARROW
    while (buf->save_narrow != NULL)
-     jed_free_saved_narrow ();
+     jed_free_saved_narrow (buf);
 #endif
    
 #if JED_HAS_COLOR_COLUMNS
-   if (buf->column_colors != NULL) SLFREE (buf->column_colors);
+   SLfree ((char *)buf->column_colors);
 #endif
    buf->prev->next = buf->next;
    buf->next->prev = buf->prev;
-   
+
+#if JED_HAS_TTY_MENUS
+   jed_delete_menu_bar (buf->menubar);
+#endif
+
 #if JED_HAS_SUBPROCESSES
    if (buf->subprocess) jed_kill_process (buf->subprocess - 1);
 #endif
    
-   SLFREE( buf);
+   SLang_free_slstring (buf->dir);
+   SLang_free_slstring (buf->file);
+   SLang_free_slstring (buf->name);
+   SLang_free_slstring (buf->mode_string);
+
+   SLfree ((char *)buf);
 }
 
 /*}}}*/
@@ -890,11 +913,11 @@ Line *dup_line(Line *l) /*{{{*/
    n = l->len;
 #endif
    
-   neew = (Line *) SLMALLOC(sizeof(Line));
+   neew = (Line *) SLmalloc (sizeof(Line));
    if ((neew == NULL) ||
-       (NULL == (neew->data = (unsigned char *) SLMALLOC(n))))
+       (NULL == (neew->data = (unsigned char *) SLmalloc (n))))
      {
-	exit_error("Malloc Error in dup_line.", 0);
+	exit_error("Out of memory", 0);
      }
    neew->next = l->next;
    neew->prev = l->prev;
@@ -923,11 +946,11 @@ Buffer *find_buffer(char *name) /*{{{*/
      }
    while(b != CBuf);
    
-#ifdef __os2__
+#if !JED_FILE_PRESERVE_CASE
    b = CBuf;
    do
      {
-	if (!strcmpi(b->name, name)) return(b);
+	if (!jed_case_strcmp (b->name, name)) return(b);
 	b = b->next;
      }
    while(b != CBuf);
@@ -992,6 +1015,7 @@ int switch_to_buffer(Buffer *buf) /*{{{*/
      }
    CBuf->line = CLine;
    CBuf->point = Point;
+   jed_set_umask (CBuf->umask);
    return(1);
 }
 
@@ -1034,48 +1058,63 @@ Buffer *find_file_buffer(char *file) /*{{{*/
 #else
 	m = strlen(b->dir);
 	
-# ifndef __os2__
-	if ((m == n) && (!strcmp(b->file, f))
-	    && (!strncmp(b->dir, file, (int) n)))
+	if ((m == n) 
+# if JED_FILE_PRESERVE_CASE
+	    && (0 == strcmp(b->file, f))
+	    && (0 == strncmp(b->dir, file, (unsigned int) n))
 # else
-	  if ((m == n) && (!strcmpi(b->file,f))
-	      && (!strncmpi(b->dir, file, (int) n)))
+	    && (0 == jed_case_strcmp (b->file,f))
+	    && (0 == jed_case_strncmp (b->dir, file, (unsigned int) n))
 # endif
-	    return(b);
+	    )
+	  return b;
 #endif
 	b = b->next;
      }
    while (b != CBuf);
    
-   return(NULL);
+   return NULL;
 }
 
 /*}}}*/
 
 /* take a dir and a filename, expand them then put in buffer structure */
-void buffer_filename(char *dir, char *file) /*{{{*/
+void buffer_filename(Buffer *b, char *dir, char *file) /*{{{*/
 {
-   strcpy (CBuf->dir, dir);
+   unsigned int dirlen;
    
-   if ((file == NULL) || (*file == 0))
+   if (dir == NULL)
+     dir = get_cwd ();
+
+   if (file == NULL)
      {
-	file = extract_file(CBuf->dir);
-	strcpy(CBuf->file, file);
-	*file = 0;
+	file = extract_file (dir);
+	dirlen = (unsigned int) (file - dir);
      }
-   else strcpy(CBuf->file, file);
+   else
+     dirlen = strlen (dir);
    
+   SLang_free_slstring (b->file);
+   SLang_free_slstring (b->dir);
+   
+   if ((NULL == (b->file = SLang_create_slstring (file)))
+       || (NULL == (b->dir = SLang_create_nslstring (dir, dirlen))))
+     {
+	exit_error ("Out of memory", 0);
+     }
+
 #ifdef REAL_UNIX_SYSTEM
-   get_inode_info (CBuf->dir, &CBuf->device, &CBuf->inode);
+   get_inode_info (b->dir, &b->device, &b->inode);
 #endif
-   uniquely_name_buffer(CBuf->file);
+
+   if (*file) uniquely_name_buffer (b, file);
 }
 
 /*}}}*/
 
 
 /* This kills all undo information! */
-int erase_buffer() /*{{{*/
+int erase_buffer (void) /*{{{*/
 {
    Line *beg, *tthis;
    
@@ -1084,13 +1123,13 @@ int erase_buffer() /*{{{*/
    bob();
    CLine = CLine->next; LineNum++;
    Suspend_Screen_Update = 1;
-   while(CLine != NULL)
+   while (CLine != NULL)
      {
 	tthis = CLine;
 	beg->next = tthis->next;
 	tthis->prev = beg;
 	
-	update_marks(LDELETE, 1);
+	jed_update_marks(LDELETE, 1);
 	CLine = tthis->next;
 	Max_LineNum--;
 	free_line(tthis);
@@ -1098,7 +1137,7 @@ int erase_buffer() /*{{{*/
    
    CLine = CBuf->beg; LineNum = 1;
    Point = 0;
-   update_marks(CDELETE, CLine->len);
+   jed_update_marks(CDELETE, CLine->len);
    CLine->len = 0;
    /*   CLine->next = NULL; */
    if (CBuf->undo != NULL) delete_undo_ring(CBuf);
@@ -1110,19 +1149,15 @@ int erase_buffer() /*{{{*/
 
 /*}}}*/
 
-void mark_buffer_modified(int *flag) /*{{{*/
+void mark_buffer_modified (int modif) /*{{{*/
 {
-   unsigned long now;
-   
-   if (*flag)
+   if (modif)
      {
-	if (CBuf->flags & BUFFER_TRASHED) return;
-	now = sys_time();
-	CBuf->m_time = now;
-	CBuf->flags |= BUFFER_TRASHED;
-	return;
+	if (CBuf->flags & BUFFER_MODIFIED) return;
+	CBuf->m_time = sys_time();
+	CBuf->flags |= BUFFER_MODIFIED;
      }
-   else CBuf->flags &= ~BUFFER_TRASHED;
+   else CBuf->flags &= ~BUFFER_MODIFIED;
 }
 
 /*}}}*/
@@ -1131,7 +1166,7 @@ void check_line() /*{{{*/
 {
    if ((CLine->len == 1) && (*CLine->data == '\n') && (CLine->data != NewLine_Buffer))
      {
-	SLFREE(CLine->data);
+	SLfree ((char *)CLine->data);
 	CLine->data = NewLine_Buffer;
 #ifdef KEEP_SPACE_INFO
 	CLine->space = 1;
@@ -1140,3 +1175,79 @@ void check_line() /*{{{*/
 }
 
 /*}}}*/
+
+
+static SLang_Name_Type **find_buffer_hook (Buffer *b, char *s)
+{
+   if (0 == strcmp("par_sep", s)) return &b->par_sep;
+   if (0 == strcmp("indent_hook", s)) return &b->indent_hook;
+   if (0 == strcmp("wrap_hook", s)) return &b->wrap_hook;
+   if (0 == strcmp("newline_indent_hook", s)) return &b->newline_indent_hook;
+   if (0 == strcmp("bob_eob_error_hook", s)) return &b->bob_eob_error_hook;
+#ifdef HAS_MOUSE
+   if (0 == strcmp("mouse_down", s)) return &b->mouse_down_hook;
+   if (0 == strcmp("mouse_up", s)) return &b->mouse_up_hook;
+   if (0 == strcmp("mouse_drag", s)) return &b->mouse_drag_hook;
+# if JED_HAS_MULTICLICK
+   if (0 == strcmp("mouse_2click", s)) return &b->mouse_2click_hook;
+   if (0 == strcmp("mouse_3click", s)) return &b->mouse_3click_hook;
+# endif
+#endif
+   if (0 == strcmp("update_hook", s)) return &b->update_hook;
+
+   return NULL;
+}
+
+
+int jed_unset_buffer_hook (Buffer *b, char *name)
+{
+   if (b == NULL) 
+     return -1;
+   
+   if ((name != NULL) && (*name != 0))
+     {
+	SLang_Name_Type **nt = find_buffer_hook (b, name);
+	if (nt == NULL)
+	  return -1;
+	
+	*nt = NULL;
+	return 0;
+     }
+
+   /* Otherwise, unset all of them */
+
+   b->par_sep = NULL;
+   b->indent_hook = NULL;
+   b->wrap_hook = NULL;
+   b->newline_indent_hook = NULL;
+   b->bob_eob_error_hook = NULL;
+   b->update_hook = NULL;
+
+#ifdef HAS_MOUSE
+   b->mouse_down_hook = NULL;
+   b->mouse_up_hook = NULL;
+   b->mouse_drag_hook = NULL;
+# if JED_HAS_MULTICLICK
+   b->mouse_2click_hook = NULL;
+   b->mouse_3click_hook = NULL;
+# endif
+#endif
+   
+   return 0;
+}
+
+int jed_set_buffer_hook (Buffer *b, char *name, SLang_Name_Type *nt)
+{
+   SLang_Name_Type **ntp;
+
+   if (b == NULL)
+     return -1;
+
+   ntp = find_buffer_hook (b, name);
+   if (ntp == NULL)
+     return -1;
+   
+   *ntp = nt;
+   return 0;
+}
+
