@@ -1,7 +1,9 @@
 /* -*- mode: c; mode: fold -*- */
-/*
- *  Copyright (c) 1992, 1996 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include "config.h"
 #include "jed-feat.h"
@@ -9,6 +11,11 @@
 /*{{{ #include files */
 
 #include <stdio.h>
+
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
+
 #include <slang.h>
 
 #include "jdmacros.h"
@@ -24,7 +31,6 @@
 #include "line.h"
 #include "paste.h"
 #include "display.h"
-#include "vterm.h"
 #include "sysdep.h"
 #include "text.h"
 #include "file.h"
@@ -42,6 +48,9 @@
 
 /*}}}*/
 
+/* fast insert/delete not implemented */
+#define USE_FAST_INSDEL 0
+
 /*{{{ Global variables */
 
 void (*X_Suspend_Hook)(void);
@@ -63,6 +72,20 @@ static char *End_Of_Buffer_Error = "End Of Buffer.";
 /*}}}*/
 /*{{{ Static functions */
 
+#if USE_FAST_INSDEL
+
+static void fast_ins(char ch) /*{{{*/
+{
+}
+
+/*}}}*/
+
+static void fast_del() /*{{{*/
+{
+}
+
+/*}}}*/
+
 /* return 1 if line extends beyond the screen */
 static int long_line(void)
 {
@@ -98,6 +121,8 @@ static int tabs_present(void)
 
 static int fast_not_ok (unsigned char ch)
 {
+   return 1;
+
    return ((Repeat_Factor != NULL) || JWindow->trashed
 	   || (ch < 32) || (ch > 126) || Input_Buffer_Len
 	   || Suspend_Screen_Update
@@ -111,6 +136,7 @@ static int fast_not_ok (unsigned char ch)
 	   || tabs_present()
 	   || (Read_This_Character != NULL));
 }
+#endif				       /* USE_FAST_INSDEL */
 
 #if JED_HAS_LINE_ATTRIBUTES
 static int check_line_attr_no_modify (Line *l)
@@ -231,20 +257,25 @@ static int next_visible_chars (int n)
 static void check_last_key_function (FVOID_STAR f)
 {
    /* For syntax highlighting */
-   if ((Last_Key_Function != f)
+   if (Last_Key_Function != f)
+     touch_window ();
+#if 0
        && ((Last_Key_Function == (FVOID_STAR) ins_char_cmd)
 	   || (Last_Key_Function == (FVOID_STAR) eol_cmd)
 	   || (Last_Key_Function == (FVOID_STAR) delete_char_cmd)
 	   || (Last_Key_Function == (FVOID_STAR) backward_delete_char_cmd)
 	   || (Last_Key_Function == (FVOID_STAR) backward_delete_char_untabify)))
+     
      register_change(0);
+#endif
 }
 
    
 static void next_line_prev_line_helper (int *gcp, int *ret, int dr, FVOID_STAR f)
 {
    int gc;
-   
+
+   (void) dr;
    check_line();
    gc = calculate_column();
    if (Cursor_Motion <= 0) Goal_Column = gc;
@@ -254,13 +285,15 @@ static void next_line_prev_line_helper (int *gcp, int *ret, int dr, FVOID_STAR f
    Cursor_Motion = 2;
    
    check_last_key_function (f);
-   
+   *ret = 1;
+#if 0
    *ret = (JWindow->trashed 
 	   || (CLine == JScreen[JWindow->top + dr - 1].line)
 #if JED_HAS_LINE_ATTRIBUTES
-	   || (CLine->flags & JED_LINE_READONLY)
+	   || (CLine->flags & JED_LINE_IS_READONLY)
 #endif
 	   );
+#endif
 }
 
 static void eob_bob_error (int f)
@@ -291,8 +324,10 @@ static void eob_bob_error (int f)
 int delete_char_cmd (void)
 {
    int upd, n;
+#if USE_FAST_INSDEL
    char ch;
-   
+#endif
+
    CHECK_READ_ONLY
 #if 0
      ;
@@ -309,14 +344,15 @@ int delete_char_cmd (void)
 	return(0);
      }
    
+#if USE_FAST_INSDEL
    ch = *(CLine->data + Point);
-   
    if (fast_not_ok ((unsigned char) ch) || long_line () || *tt_Term_Cannot_Insert)
      {
+#endif
 	if ((Point == 0) && eolp()
 #if JED_HAS_LINE_ATTRIBUTES
 	    && ((CLine->prev != NULL)
-		&& (0 == (CLine->prev->flags & JED_LINE_READONLY)))
+		&& (0 == (CLine->prev->flags & JED_LINE_IS_READONLY)))
 #endif
 	    ) n = 1;
 	else n = 0;
@@ -325,22 +361,24 @@ int delete_char_cmd (void)
 	del();
 	forwchars(&n);
 	upd = 1;
+#if USE_FAST_INSDEL
      }
    else
      {
-	upd = ((CBuf->flags & BUFFER_TRASHED) == 0);
+	upd = ((CBuf->flags & BUFFER_MODIFIED) == 0);
 	fast_del();
 	tt_delete_char ();
      }
-   
+#endif
    return(upd);
 }
 
 int backward_delete_char_cmd()
 {
+#if USE_FAST_INSDEL
    char ch;
    int ret;
-   
+#endif
    CHECK_READ_ONLY
 #if 0
      ;
@@ -367,22 +405,25 @@ int backward_delete_char_cmd()
 	return delete_char_cmd();
      }
    
+#if USE_FAST_INSDEL
    ch = *(CLine->data + (Point - 1));
-   
    if (fast_not_ok ((unsigned char) ch) || long_line () || *tt_Term_Cannot_Insert)
      {
+#endif
 	backwchars(&Number_One);
 	del();
-	return(1);
+	return 1;
+#if USE_FAST_INSDEL
      }
-   
-   ret = ((CBuf->flags & BUFFER_TRASHED) == 0);
+
+   ret = ((CBuf->flags & BUFFER_MODIFIED) == 0);
    backwchars(&Number_One);
    tt_putchar('\b');
    Screen_Col--;
    fast_del();
    tt_delete_char();
    return(ret);
+#endif
 }
 
 int backward_delete_char_untabify()
@@ -472,10 +513,11 @@ int newline_cmd (void)
 int ins_char_cmd (void)
 {
    char ch;
-   
    int wrap = Jed_Wrap_Column;
+   int do_blink;
+#if USE_FAST_INSDEL
    int ret, ll;
-   
+#endif
    CHECK_READ_ONLY
 #if 0
      ;
@@ -505,8 +547,23 @@ int ins_char_cmd (void)
    if (((ch == ' ') || (Point >= wrap)) && (CBuf->modes & WRAP_MODE)
        && (calculate_column() > wrap)) /* JWindow->width */
      {
+	unsigned int this_line_num = LineNum;
 	ins(ch);
 	wrap_line(0);			/* do not format--- just wrap */
+	
+	/* There is a bug involving wrapping a very long line containing
+	 * no whitespace and then we try to insert a character.  This work
+	 * arounds the bug.
+	 */
+	if ((this_line_num == LineNum)
+	    && (ch == ' ')
+	    && (calculate_column () > wrap))
+	  {
+	     int one = 1;
+	     if (0 == forwchars (&one))
+	       newline ();
+	  }
+
 	if (Indented_Text_Mode) indent_line ();
 	if (CBuf->wrap_hook != NULL)
 	  SLexecute_function(CBuf->wrap_hook);
@@ -514,19 +571,23 @@ int ins_char_cmd (void)
 	return(1);
      }
    
+   do_blink = ((((CBuf->syntax_table != NULL)
+		 && ((CBuf->syntax_table->char_syntax[(unsigned char) ch] & CLOSE_DELIM_SYNTAX)))
+		|| ((ch == ')') || (ch == '}') || (ch == ']')))
+	       && !input_pending(&Number_Zero));
+#if USE_FAST_INSDEL
    if ((ch == ' ') || fast_not_ok ((unsigned char) ch) || (CBuf->flags & OVERWRITE_MODE)
-       || (*tt_Term_Cannot_Insert && !eolp())
-       )
+       || (*tt_Term_Cannot_Insert && !eolp()))
      {
+#endif
 	ins(ch);
-	if ((CBuf->syntax_table != NULL)
-	    && (CBuf->syntax_table->char_syntax[(unsigned char) ch] & CLOSE_DELIM_SYNTAX)
-	    && !input_pending(&Number_Zero)) blink_match (); /* (ch); */
-	return(1);
+	if (do_blink) blink_match ();
+	return 1;
+#if USE_FAST_INSDEL
      }
    
    
-   ret = ((CBuf->flags & BUFFER_TRASHED) == 0) || (User_Prefers_Line_Numbers > 1);
+   ret = ((CBuf->flags & BUFFER_MODIFIED) == 0) || (User_Prefers_Line_Numbers > 1);
    
    if (long_line ()) ll = 1; else ll = 0;
    ret = ret || ll;
@@ -544,10 +605,11 @@ int ins_char_cmd (void)
      }
    else tt_putchar(ch);
    Screen_Col++;
-   if ((CBuf->syntax_table != NULL)
-       && (CBuf->syntax_table->char_syntax[(unsigned char) ch] & CLOSE_DELIM_SYNTAX)
-       && !input_pending(&Number_Zero)) blink_match (); /* (ch); */
+   
+   if (do_blink) blink_match ();
+
    return(ret);
+#endif
 }
 
 /*}}}*/
@@ -690,7 +752,7 @@ int previous_char_cmd (void)
    Cursor_Motion = 1;
    pnt = Point - 1;
    
-   check_last_key_function ((FVOID_STAR) previous_char_cmd);
+   /* check_last_key_function ((FVOID_STAR) previous_char_cmd); */
    
    if (1 != prev_visible_chars (1))
      {
@@ -706,7 +768,7 @@ int next_char_cmd ()
 {
    Cursor_Motion = 1;
    
-   check_last_key_function ((FVOID_STAR) next_char_cmd);
+   /* check_last_key_function ((FVOID_STAR) next_char_cmd); */
    
    if (1 != next_visible_chars (1))
      {
@@ -726,7 +788,7 @@ int eol_cmd (void)
    eol();
    if ((0 == (CBuf->flags & READ_ONLY))
 #if JED_HAS_LINE_ATTRIBUTES
-       && (0 == (CLine->flags & JED_LINE_READONLY))
+       && (0 == (CLine->flags & JED_LINE_IS_READONLY))
 #endif
        )
      trim_whitespace();
@@ -813,7 +875,7 @@ int pagedown_cmd()
    
    if (JWindow->trashed)
      {
-	update (NULL, 0, 0);
+	update (NULL, 0, 0, 1);
 	if (JWindow->trashed) return next_visible_lines (n);
      }
    
@@ -868,7 +930,7 @@ int pageup_cmd (void)
    
    if (JWindow->trashed)
      {
-	update (NULL, 0, 0);
+	update (NULL, 0, 0, 1);
 	if (JWindow->trashed) return prev_visible_lines (n);
      }
    
@@ -1264,9 +1326,10 @@ int sys_spawn_cmd (void)
 	return (0);
      }
    
-   SLang_run_hooks("suspend_hook", NULL, NULL);
-   if (SLang_Error) return(0);
-   reset_display();
+   if (-1 == SLang_run_hooks("suspend_hook", 0))
+     return 0;
+
+   jed_reset_display();
    reset_tty();
    sys_suspend();
    if (-1 == init_tty())
@@ -1275,9 +1338,8 @@ int sys_spawn_cmd (void)
      }
    
    flush_input ();
-   init_display(1);
-   /* cls(); */
-   SLang_run_hooks("resume_hook", NULL, NULL);
+   jed_init_display ();
+   (void) SLang_run_hooks("resume_hook", 0);
    check_buffers();
    return(1);
 }
@@ -1295,7 +1357,7 @@ int quit_jed(void)
    do
      {
 	if ((b->flags & AUTO_SAVE_JUST_SAVE)
-	    && (b->flags & BUFFER_TRASHED)
+	    && (b->flags & BUFFER_MODIFIED)
 	    && (*b->file))
 	  {
 	     while (b->narrow != NULL) widen_buffer(b);
@@ -1304,14 +1366,14 @@ int quit_jed(void)
 	
 #if defined(__WIN32__) && JED_HAS_SUBPROCESSES
  	if (b->subprocess)
- 	  jed_kill_process(b->subprocess - 1);
+ 	  jed_kill_process (b->subprocess - 1);
 #endif
 	
 	b = b->next;
      }
    while (b != CBuf);
    
-   reset_display();
+   jed_reset_display();
    reset_tty();
 #ifdef VMS
    vms_cancel_exithandler();
@@ -1322,71 +1384,50 @@ int quit_jed(void)
 #ifdef MALLOC_DEBUG
    SLmalloc_dump_statistics ();
 #endif
+   /* SLstring_dump_stats (); */
    exit(0);
    return(1);
 }
 
 
 /*}}}*/
+
+
 /*{{{ save_some_buffers */
 /* I should try it like emacs--- if prefix argument, then save all without user
  intervention */
-int save_some_buffers(void)
+int save_some_buffers (void)
 {
    Buffer *b, *tmp;
    int ans = 0;
-   char msg[256];
    int err;
    
    b = CBuf;
    do
      {
-	if ((b->flags & BUFFER_TRASHED)
+	if ((b->flags & BUFFER_MODIFIED)
 	    && (*b->file))
 	  {
-	     if (b->flags & AUTO_SAVE_JUST_SAVE) ans = 'y';
-	     else
+	     if (b->flags & AUTO_SAVE_JUST_SAVE) ans = 1;
+	     else ans = jed_vget_y_n ("Buffer %s not saved. Save it",
+				      b->name);
+	     
+	     if (ans == -1)
+	       /* warning--- bug here if user edits file at
+		startup and forgets to save it then aborts. */
+	       return -1;
+	     
+	     if (ans == 1)
 	       {
-		  sprintf(msg,"Buffer %s not saved. Save it? (y/n)", b->name);
-		  flush_message(msg);
-		  ans = my_getkey();
-	       }
-
-	     while (1)
-	       {
-		  if ((ans == 'y') || (ans == 'Y'))
-		    {
-		       tmp = CBuf;
-		       switch_to_buffer(b);
-		       while (b->narrow != NULL) widen_buffer(b);
-		       err = write_file_with_backup(b->dir, b->file);
-		       switch_to_buffer(tmp);
-  		       if (err < 0) return(-1);
-		       b->flags &= ~BUFFER_TRASHED;
-		       b->flags |= AUTO_SAVE_BUFFER;
-		       b->hits = 0;
-		       break;
-		    }
-		  else if ((ans == 'n') || (ans == 'N'))
-		    {
-		       /* disabling this */
-		       /* auto_save_buffer(b); */
-		       break;
-		    }
-		  else if (SLKeyBoard_Quit)
-		    {
-		       /* warning--- bug here if user edits file at
-			startup and forgets to save it then aborts. */
-		       return(-1);
-		    }
-		  else
-		    {
-		       beep();
-		       sprintf(msg,"Buffer %s not saved. Save it", b->name);
-		       ans = get_yes_no(msg);
-		       if (ans == 1) ans = 'y'; else if (!ans) ans = 'n';
-		       else return(-1);
-		    }
+		  tmp = CBuf;
+		  switch_to_buffer(b);
+		  while (b->narrow != NULL) widen_buffer(b);
+		  err = write_file_with_backup(b->dir, b->file);
+		  switch_to_buffer(tmp);
+		  if (err < 0) return -1;
+		  b->flags &= ~BUFFER_MODIFIED;
+		  /* b->flags |= AUTO_SAVE_BUFFER; */
+		  b->hits = 0;
 	       }
 	  }
 	
@@ -1395,7 +1436,7 @@ int save_some_buffers(void)
    while (b != CBuf);
 
    clear_message ();
-   return(1);
+   return 1;
 }
 
 /*}}}*/
@@ -1404,20 +1445,27 @@ int exit_jed (void)
 {
    static int in_exit_jed = 0;
    
-   if (!in_exit_jed)
+   if (in_exit_jed == 0)
      {
 	in_exit_jed = 1;
-	if (SLang_run_hooks("exit_hook", NULL, NULL))
+	if (-1 == SLang_run_hooks("exit_hook", 0))
 	  {
 	     in_exit_jed = 0;
-	     return(1);
+	     return 1;
 	  }
      }
    
    in_exit_jed = 0;
-   if (SLang_Error) return(0);
+   if (SLang_Error) return 0;
+
+#if JED_HAS_SUBPROCESSES
+   if (Num_Subprocesses
+       && (1 != jed_get_y_n ("Subprocesses exist.  Exit anyway")))
+     return 1;
+#endif
+   
    if (save_some_buffers() > 0) quit_jed();
-   return(1);
+   return 1;
 }
 
 /*}}}*/
