@@ -1,13 +1,35 @@
 /* -*- mode: C; mode: fold; -*- */
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include "config.h"
 #include "jed-feat.h"
 
 /*{{{ system include files */
 
+#ifdef __WIN32__
+/* This needs to go first before stdio is included. */
+# include <windows.h>
+# if !defined(__MINGW32__) && !defined(__CYGWIN32__)
+#  define sleep Sleep
+# endif
+# include <fcntl.h>
+# include <io.h>
+# include <sys/stat.h>
+#endif
+
+#ifdef __IBMC__
+#if SLANG_VERSION < 10308
+# define sleep(x) sys_pause(1000 * (x))
+#else
+/* sleep added to slang 10308 */
+extern unsigned int sleep (unsigned int);
+#endif
+#endif
+
 #include <stdio.h>
 #include <slang.h>
 
@@ -30,10 +52,14 @@
 # include <sys/file.h>
 #endif
 
-#ifdef REAL_UNIX_SYSTEM
+#ifdef HAVE_UTIME
 # include <utime.h>
 #endif
 
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+
 #ifdef HAVE_SYS_FCNTL_H
 # include <sys/fcntl.h>
 #endif
@@ -49,14 +75,11 @@
  int WriteEAs (char *name, PHOLDFEA pHoldFEA);
 #endif
 
-#ifdef msdos
+#ifdef __MSDOS__
 #include <fcntl.h>
 #include <io.h>
 #include <sys/stat.h>
 #endif
-#ifdef __GO32__
-#include <fcntl.h>
-#endif
 
 #if defined(__DECC) && defined(VMS)
 # include <unixio.h>
@@ -65,11 +88,6 @@
 
 #include <errno.h>
 
-#ifdef __WIN32__
-# include <windows.h>
-# define sleep Sleep
-#endif
-
 /* Was anything missed? */
 #ifndef O_RDONLY
 #ifdef VMS
@@ -93,7 +111,7 @@
 
 /*}}}*/
 
-#if defined (msdos) || defined (__os2_16__)
+#if defined (SIXTEEN_BIT_SYSTEM)
 #define MAX_LINE_LEN 1024
 #else
 #define MAX_LINE_LEN 64 * 1024
@@ -220,7 +238,7 @@ static int sys_open(char *file, int acces) /*{{{*/
    
    flags |= mode;
    
-#if defined (msdos) || defined (__os2__)
+#if defined (IBMPC_SYSTEM)
    fp = open(file, flags, S_IREAD | S_IWRITE); 
 #else
    fp = open(file, flags, 0666); 
@@ -245,24 +263,49 @@ char *file_type(char *file) /*{{{*/
 }
 
 /*}}}*/
-void set_file_modes () /*{{{*/
+
+
+void jed_set_buffer_ctime (Buffer *b)
+{
+   char file[JED_MAX_PATH_LEN + 1];
+
+   if ((b->file == NULL)
+       || (b->file[0] == 0))
+     {
+	b->c_time = 0;
+	return;
+     }
+   
+   strcpy (file, b->dir);
+   strcat (file, b->file);
+
+   if (0 == sys_file_mod_time (file))
+     b->c_time = 0;
+   else
+     b->c_time = sys_time ();
+}
+
+void set_file_modes (void) /*{{{*/
 {
    char *type;
 
    if (CBuf == NULL) return;
-   CBuf->c_time = sys_time();
-   if (CBuf->file[0])
+
+   jed_set_buffer_ctime (CBuf);
+
+   if ((CBuf->file != NULL)
+       && (CBuf->file[0] != 0))
      {
-#if defined(msdos) && !defined(W_OK)
-#define W_OK 2
-#define F_OK 0
+#if (defined(__MSDOS__) || defined(__WIN32__)) && !defined(W_OK)
+# define W_OK 2
+# define F_OK 0
 #endif
 
 #ifdef W_OK
 #ifdef __GO32__
 # define access i386_access
 #endif
-	char name[512];
+	char name [JED_MAX_PATH_LEN + 256];
 #ifndef VMS
 	  {
 	     /* COmment:  I need a more generic call to contruct a directory
@@ -272,7 +315,7 @@ void set_file_modes () /*{{{*/
 	     strcpy (name, CBuf->dir);
 	     n = strlen (name);
 	     if (n >
-#ifdef pcsystem
+#ifdef IBMPC_SYSTEM
 		 3		       /* allow C:/file */
 #else
 		 1		       /* allow /file */
@@ -293,11 +336,15 @@ void set_file_modes () /*{{{*/
 	CBuf->hits = 0;
 	type = file_type(CBuf->file);
      }
-   else type = (char *) NULL;
+   else type = NULL;
 
-   if (type == (char *) NULL) CBuf->modes = NO_MODE;
-   else if (SLang_run_hooks("mode_hook", type, NULL));
-   else CBuf->modes = NO_MODE;
+   CBuf->modes = NO_MODE;
+#if JED_HAS_LINE_ATTRIBUTES
+   CBuf->min_unparsed_line_num = 1;
+   CBuf->max_unparsed_line_num = Max_LineNum + CBuf->nup;
+#endif
+   if (type != NULL)
+     SLang_run_hooks("mode_hook", 1, type);
 }
 
 /*}}}*/
@@ -379,7 +426,7 @@ static char *Output_Bufferp_max;
 /* definitely perform the write.  Return number of chars written */
 static int jed_write1(int fd, char *b, unsigned int n) /*{{{*/
 {
-#if !defined(msdos) || defined(__WATCOMC__) || defined(__WIN32__)
+#if !defined(IBMPC_USE_ASM)
    int len;
    unsigned int total = 0;
 #ifdef VMS
@@ -647,7 +694,7 @@ static int jed_close (int fp) /*{{{*/
    while (-1 == close(fp))
      {
 #ifdef EINTR
-#ifndef pc_system
+#ifndef IBMPC_SYSTEM
 	if (errno == EINTR) 
 	  {
 	     errno = 0;
@@ -668,14 +715,12 @@ int write_region (char *file) /*{{{*/
 {
    int fp;
    int n;
-   char msg[JED_MAX_PATH_LEN];
 
    if (!check_region(&Number_Zero)) return(-1);
    if ((fp = sys_open(file, 1)) < 0)
      {
-	sprintf(msg, "Unable to open %s for writing.", file);
-	msg_error(msg);
-	return(-1);
+	jed_verror ("Unable to open %s for writing.", file);
+	return -1;
      }
    n = write_region_to_fp(fp);
    if (-1 == jed_close (fp)) n = -1;
@@ -769,32 +814,31 @@ static int make_autosave_filename(char *save, char *dir, char *file) /*{{{*/
    if (*file == 0) return(0);
       
    
-   if (SLang_run_hooks("make_autosave_filename", dir, file) && !SLang_Error)
+   if (1 == SLang_run_hooks ("make_autosave_filename", 2, dir, file))
      {
-	if (SLpop_string(&s)) return(0);
+	if (SLang_pop_slstring(&s)) return(0);
 	strncpy(save, s, JED_MAX_PATH_LEN - 2);
 	save[JED_MAX_PATH_LEN - 1] = 0;
-	SLFREE(s);
+	SLang_free_slstring (s);
      }
    else
      {
-#ifdef __unix__
-	sprintf(save, "%s#%s#", dir, file);
-#else
-#if defined (msdos) || defined (__os2__)
+#if defined(IBMPC_SYSTEM)
 	sprintf(save, "%s#%s", dir, file);
 #else
+# ifdef VMS
 	sprintf(save, "%s_$%s;1", dir, file);
-#endif
+# else
+	sprintf(save, "%s#%s#", dir, file);
+# endif
 #endif
      }
-   return(1);
+   return 1;
 }
 
 /*}}}*/
 
 #ifndef VMS
-# ifdef REAL_UNIX_SYSTEM
 int jed_copy_file (char *from, char *to) /*{{{*/
 {
    int mode;
@@ -804,7 +848,9 @@ int jed_copy_file (char *from, char *to) /*{{{*/
    unsigned int readlen;
    int ret;
    struct stat st;
+#ifdef HAVE_UTIME
    struct utimbuf ut;
+#endif
 
    if (1 != sys_chmod (from, 0, &mode, &uid, &gid))
      return -1;		       /* from does not exist as regular file */
@@ -848,45 +894,73 @@ int jed_copy_file (char *from, char *to) /*{{{*/
      }
    
    (void) sys_chmod (to, 1, &mode, &uid, &gid);
+
+#ifdef HAVE_UTIME
    /* Set file modification times */
    ut.actime = st.st_atime;
    ut.modtime = st.st_mtime;
    (void) utime (to, &ut);
-   
+#endif
+
    return ret;
 }
 /*}}}*/
-# endif
-#endif
+#endif				       /* NOT VMS */
 
 #ifndef VMS   
 static int perform_backup (char *from, char *to, int try_force_rename) /*{{{*/
 {
    int ret = -1;
+   int use_copy = Jed_Backup_By_Copying;
+#ifdef REAL_UNIX_SYSTEM
+   /* If the file has hardlinks, then backup by copying. */
+   struct stat st;
    
+   if (0 == stat (from, &st))
+     {
+	if (st.st_nlink > 1)
+	  use_copy = 1;
+     }
+#endif
+
    if (try_force_rename
-       || (Jed_Backup_By_Copying == 0))
+       || (use_copy == 0))
      {
 	(void) unlink(to);
+#ifdef __WIN32__
+	/* Rename is broken on win32 */
+	ret = jed_win32_rename (from, to);
+#else
 	ret = rename (from, to);
+#endif
      }
-#ifdef REAL_UNIX_SYSTEM
-   /* Needs porting to OS/2 and DOS */
+
    if (ret == -1)
      ret = jed_copy_file (from, to);
-#endif
 
    return ret;
 }
 /*}}}*/
 #endif
 
+unsigned long sys_file_mod_time(char *file) /*{{{*/
+{
+   struct stat buf;
+   
+   if (stat(file, &buf) < 0) return 0;
+
+   return (unsigned long) buf.st_mtime;
+}
+
+/*}}}*/
+
 int write_file_with_backup(char *dir, char *file) /*{{{*/
 {
    char neew[JED_MAX_PATH_LEN]; char autosave_file[JED_MAX_PATH_LEN];
    int n;
    int mode, do_mode;
    short uid, gid;
+   char *mode_string;
 #ifndef VMS
    char *old = NULL;
    int backup_went_ok;
@@ -898,7 +972,7 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
    if (*file == 0) return(-1);
 
    sprintf(neew, "%s%s", dir, file);
-   
+
    *autosave_file = 0;
    if (CBuf->flags & AUTO_SAVE_BUFFER)
       (void) make_autosave_filename(autosave_file, dir, file);
@@ -909,7 +983,14 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
     * there is nothing to backup.  If do_mode == 1, the file is a regular
     * file which we do want to backup.
     */
-   
+   if (NULL == (mode_string = CBuf->mode_string))
+     mode_string = "";
+   if (-1 == SLang_run_hooks ("save_buffer_hook", 2, neew, mode_string))
+     {
+	auto_save_buffer (CBuf);
+	return -1;
+     }
+
 #ifndef VMS
    
 # ifdef __os2__
@@ -919,16 +1000,16 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
    backup_went_ok = 0;
    if (do_mode
        && ((CBuf->flags & NO_BACKUP_FLAG) == 0)
-       && SLang_run_hooks("make_backup_filename", dir, file))
+       && (1 == SLang_run_hooks("make_backup_filename", 2, dir, file)))
      {
-	if ((0 == SLpop_string(&old))
+	if ((0 == SLang_pop_slstring(&old))
 	    && (*old != 0))
 	  {
 	     backup_went_ok = (0 == perform_backup (neew, old, 0));
 	  }
      }
 #endif				       /* NOT VMS */
-   
+
    if (-1 != (n = write_file(neew)))
      {
 	if (*autosave_file)
@@ -938,7 +1019,7 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
 	     
 	if (do_mode) /* must be an existing file, so preserve mode */
 	  {
-#ifdef msdos
+#if defined(__MSDOS__)
 	     /* Want the archive bit set since this is a new version */
 	     mode |= 1 << 5;
 #endif
@@ -952,10 +1033,11 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
 	
 	/* Since we wrote the buffer to the file, it is not modified. */
 	if (
-#ifdef __os2__
-	    !strcmpi(file, CBuf->file) && !strcmp(dir, CBuf->dir)
+#if JED_FILE_PRESERVE_CASE
+	    (0 == strcmp(file, CBuf->file)) && (0 == strcmp(dir, CBuf->dir))
 #else
-	    !strcmp(file, CBuf->file) && !strcmp(dir, CBuf->dir)
+	    (0 == jed_case_strcmp (file, CBuf->file))
+	     && (0 == jed_case_strcmp (dir, CBuf->dir))
 #endif
 	    )
 	  CBuf->flags &= ~FILE_MODIFIED;
@@ -966,7 +1048,7 @@ int write_file_with_backup(char *dir, char *file) /*{{{*/
    /* error -- put it back */
    else if (backup_went_ok) perform_backup (old, neew, 1); 
    
-   if (old != NULL) SLFREE(old);
+   if (old != NULL) SLang_free_slstring (old);
 #endif
    return(n);
 }
@@ -981,7 +1063,7 @@ void auto_save_buffer(Buffer *b) /*{{{*/
    
    if (b == NULL) return;
    b->hits = 0;
-   if ((0 == (b->flags & BUFFER_TRASHED))
+   if ((0 == (b->flags & BUFFER_MODIFIED))
        || (0 == (b->flags & (AUTO_SAVE_BUFFER | AUTO_SAVE_JUST_SAVE))))
      return;
 
@@ -1002,7 +1084,7 @@ void auto_save_buffer(Buffer *b) /*{{{*/
 	     flush_message("autosaving..."); 
 	     sys_delete_file(tmp);
 	     write_file(tmp);
-#ifndef pc_system
+#ifndef IBMPC_SYSTEM
 	     (void) chmod (tmp, 0600);
 #endif
 	     message("autosaving...done");
@@ -1010,7 +1092,7 @@ void auto_save_buffer(Buffer *b) /*{{{*/
       }
    else if (write_file_with_backup(b->dir, b->file) >= 0)
      {
-	b->flags &= ~BUFFER_TRASHED;
+	b->flags &= ~BUFFER_MODIFIED;
      }
 
    switch_to_buffer(old_buf);
@@ -1019,7 +1101,7 @@ void auto_save_buffer(Buffer *b) /*{{{*/
 
 /*}}}*/
 
-void auto_save_all() /*{{{*/
+void auto_save_all (void) /*{{{*/
 {
     Buffer *b;
 
@@ -1027,7 +1109,10 @@ void auto_save_all() /*{{{*/
    do
      {
 	jed_widen_whole_buffer (b);
-	if ((*b->file != 0) && (b->hits != 0)) auto_save_buffer(b);
+	if (CBuf == NULL) return;
+	if (*b->file != 0) auto_save_buffer(b);
+	if (CBuf == NULL) return;
+
 	b = b->next;
      }
    while (b != CBuf);
@@ -1073,6 +1158,21 @@ static int is_link(char *f, char *f1) /*{{{*/
    if (-1 == (int) (l = readlink(f, f1, l))) return 0;
    if (is_dir) f1[l++] = '/';
    f1[l] = 0;
+   
+   /* If the link is an absolute pathname (starts with /) then the 
+    * ../ prefix should not go there.
+    */
+   if (is_dir && (*f1 == '/'))
+     {
+	char ch;
+	
+	f = f1 - 3;
+	while (0 != (ch = *f1++))
+	  *f++ = ch;
+	
+	*f = 0;
+     }
+
    return(1);
 }
 
@@ -1091,7 +1191,7 @@ char *expand_link(char *f) /*{{{*/
    char *d = expand_filename(f);
  
   
-   if (is_link(d, lnk))
+   while (is_link(d, lnk))
      {
 	char *w, *w1, ch;
 	
@@ -1128,17 +1228,15 @@ char *expand_link(char *f) /*{{{*/
 
 void visit_file(char *dir, char *file) /*{{{*/
 {
-#ifndef __os2__
-   if (strcmp(file, CBuf->file) || strcmp(dir, CBuf->dir))
+   if (
+#if JED_FILE_PRESERVE_CASE
+       strcmp(file, CBuf->file) || strcmp(dir, CBuf->dir)
 #else
-   if (strcmpi(file, CBuf->file) || strcmp(dir, CBuf->dir))
+       jed_case_strcmp (file, CBuf->file) || jed_case_strcmp (dir, CBuf->dir)
 #endif
+       )
      {
-	if (NULL == find_buffer(file)) strcpy(CBuf->name, file); 
-	else uniquely_name_buffer(file);
-   
-	strcpy(CBuf->dir, dir);
-	strcpy(CBuf->file, file);
+	buffer_filename (CBuf, dir, file);
      }
    /* We have already taken care of this in write buffer function.
     */
@@ -1155,7 +1253,7 @@ char *dir_file_merge(char *dir, char *file) /*{{{*/
  * considered to be whole file.
  */
 {
-   char name[512];
+   char name[2 * JED_MAX_PATH_LEN];
 
    strcpy (name, dir);
    if ((file != NULL) && *file)
@@ -1163,6 +1261,7 @@ char *dir_file_merge(char *dir, char *file) /*{{{*/
 	fixup_dir(name);
 	strcat(name, file);
      }
+
    return expand_filename(name);
 }
 
@@ -1187,14 +1286,39 @@ int file_status(char *file) /*{{{*/
 }
 
 /*}}}*/
-int file_changed_on_disk(char *file) /*{{{*/
+int file_changed_on_disk (Buffer *buf, char *file) /*{{{*/
 {
    unsigned long t;
-   Buffer *buf;
-   if (NULL == (buf = find_file_buffer(file))) return(0);
-   if (buf->flags & FILE_MODIFIED) return 1;
+
    t = sys_file_mod_time(file);
-   return(t > buf->c_time);
+   if (t == 0)
+     {
+	/* File does not exist.  If the buffer has not been modified since
+	 * the last save, then the file has changed on the disk.
+	 */
+#if 1
+	return 0;
+#else
+	if (buf->c_time == 0)
+	  return 0;
+
+	return (0 == (buf->flags & BUFFER_MODIFIED));
+#endif
+     }
+
+   if (t > buf->c_time)
+     {
+	if (buf->c_time == 0)	       /* new buffer, writing to exiting file */
+	  return 0;
+
+	return 1;
+     }
+   
+   /* FIXME: The file may still have been changed via a rename operation.  Check
+    * the inodes.
+    */
+   
+   return 0;
 }
 
 /*}}}*/
@@ -1236,7 +1360,7 @@ void check_buffer(Buffer *b) /*{{{*/
 #endif
 
    if ((*b->file != 0)
-       && file_changed_on_disk(dir_file_merge(b->dir, b->file)))
+       && file_changed_on_disk(b, dir_file_merge(b->dir, b->file)))
      {
 	b->flags |= FILE_MODIFIED;
      }
@@ -1291,12 +1415,12 @@ int read_file_pointer(int fp) /*{{{*/
 	CLine->len = num;
 	if (SLang_Error) break;
      }
-   if (vp->buf != NULL) SLFREE(vp->buf);
+   if (vp->buf != NULL) SLfree(vp->buf);
    
    if (vp->cr_flag) CBuf->flags |= ADD_CR_ON_WRITE_FLAG;
    else CBuf->flags &= ~ADD_CR_ON_WRITE_FLAG;
    
-   SLFREE(vp);
+   SLfree ((char *)vp);
    return n;
 }
 
@@ -1306,7 +1430,7 @@ int read_file_pointer(int fp) /*{{{*/
 int get_inode_info (char *dir, int *device, int *inode) /*{{{*/
 {
    struct stat st;
-   char work[512];
+   char work [JED_MAX_PATH_LEN + 512];
    char *f;
    
    f = extract_file (dir);
@@ -1327,5 +1451,30 @@ int get_inode_info (char *dir, int *device, int *inode) /*{{{*/
    return 0;
 }
 /*}}}*/
+
 #endif
-   
+
+void jed_set_umask (int mask)
+{
+#ifdef REAL_UNIX_SYSTEM
+   static int default_umask;
+#endif
+
+   if (mask == 0) 
+     {
+#ifdef REAL_UNIX_SYSTEM
+	if (default_umask != 0)
+	  (void) umask (default_umask);
+#endif
+     }
+   else
+     {
+#ifdef REAL_UNIX_SYSTEM
+	int u;
+	
+	u = umask (mask & 0777);
+	if (default_umask == 0) default_umask = u;
+#endif
+     }
+}
+
