@@ -1,10 +1,12 @@
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include <stdio.h>
 #ifdef VMS
-#include <file.h>
+# include <fcntl.h>
 #endif
 
 #include "buffer.h"
@@ -27,7 +29,7 @@ extern void visit_file(char *, char *);
 extern void fixup_dir(char *);
 extern char *dir_file_merge(char *, char *);
 extern int file_status(char *);
-extern int file_changed_on_disk(char *);
+extern int file_changed_on_disk (Buffer *, char *);
 extern int file_time_cmp(char *, char *);
 extern char *file_type(char *);
 extern void check_buffer(Buffer *);
@@ -38,8 +40,12 @@ extern char *expand_link(char *);
 #endif
 #ifdef REAL_UNIX_SYSTEM
 extern int get_inode_info (char *, int *, int *);
-extern int jed_copy_file (char *, char *);
 extern int Jed_Backup_By_Copying;
 #endif
+#ifndef VMS
+extern int jed_copy_file (char *, char *);
+#endif
+extern void jed_set_umask (int);
+extern void jed_set_buffer_ctime (Buffer *);
 
 
