@@ -1,13 +1,18 @@
-/*
- *  Copyright (c) 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 extern int (*X_Read_Hook) (void);	 
 extern int (*X_Input_Pending_Hook) (void);
-extern void (*X_Get_Term_Size_Hook) (int *, int *);
 extern void (*X_Suspend_Hook)(void);
 extern int (*X_Argc_Argv_Hook)(int, char **);
+#if 0
 extern int (*X_Init_SLang_Hook)(void);
+#endif
+extern int jed_add_init_slang_hook (int (*)(void));
+
 extern int (*X_Init_Term_Hook) (void);
 extern void (*X_Reset_Term_Hook) (void);
 extern void (*X_Update_Open_Hook)(void);      /* hooks called when starting */
