@@ -1,4 +1,10 @@
 /* -*- mode: C; mode: fold; -*- */
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
+ */
 #include "config.h"
 #include "jed-feat.h"
 
@@ -13,6 +19,16 @@
 #include <errno.h>
 #include <signal.h>
 
+#ifdef __os2__
+# include <io.h>
+# include <process.h>
+#endif
+
+#if defined(__BORLANDC__) && (__BORLANDC__>>8)==5 /* [JMS:MISC] - needed for open/close in BC v5 */
+# include <io.h>
+# include <dos.h>
+#endif
+
 #include <slang.h>
 
 #include "jdmacros.h"
@@ -26,6 +42,9 @@
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
 #endif
+#ifdef HAVE_SYS_FCNTL_H
+# include <sys/fcntl.h>
+#endif
 #ifndef WEXITSTATUS
 # define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
 #endif
@@ -33,6 +52,11 @@
 # define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
 #endif
 
+#ifdef REALLY_HAVE_TERMIOS_H
+# ifndef __os2__
+#  define USE_PTY
+# endif
+#endif
 
 #include "buffer.h"
 #include "ins.h"
@@ -41,11 +65,17 @@
 #include "jprocess.h"
 #include "paste.h"
 #include "sig.h"
+#include "cmds.h"
 
 /*}}}*/
 
 int Num_Subprocesses;
 int Max_Subprocess_FD;
+
+/* This also servers as a lookup table for actual system pids to the 
+ * pseudo-pids used here.  See, e.g., jed_get_child_status for usage in this
+ * fashion.
+ */
 int Subprocess_Read_fds [MAX_PROCESSES][2];   /* 0 is actual fd, 1 is our rep */
 
 volatile int Child_Status_Changed_Flag;/* if this is non-zero, editor
@@ -66,6 +96,7 @@ typedef struct /*{{{*/
    
    int status_changed;		       /* non-zero if status changed. */
    int rd, wd;			       /* read/write descriptors */
+   int is_pty;
    int pid;			       /* real process pid */
    int output_type;
 #define PROCESS_USE_BUFFER	1
@@ -74,7 +105,7 @@ typedef struct /*{{{*/
 #define PROCESS_AT_POINT	8
    Buffer *buffer;		       /* buffer associated with process */
    SLang_Name_Type *slang_fun;	       /* function to pass output to */
-   SLuser_Object_Type *umark;	       /* marks point of last output */
+   SLang_MMT_Type *umark;	       /* marks point of last output */
    
    SLang_Name_Type *status_change_fun; /* call this if process status changes 
 					* The function should be declared like
@@ -83,19 +114,90 @@ typedef struct /*{{{*/
 					* the flags field in this struct and 
 					* the pid is NOT the pid of this struct
 					*/
-} Process_Type;
-
+} 
 /*}}}*/
-
+Process_Type;
 
 static Process_Type Processes[MAX_PROCESSES];
 
+static int signal_safe_close (int fd)
+{
+   while (-1 == close (fd))
+     {
+#ifdef EINTR
+	if (errno == EINTR)
+	  continue;
+#endif
+	return -1;
+     }
+   return 0;
+}
+
+static int signal_safe_open (char *file, int mode)
+{
+   int fd;
+
+   while (-1 == (fd = open (file, mode)))
+     {
+#ifdef EINTR
+	if (errno == EINTR)
+	  continue;
+#endif
+#ifdef EAGAIN
+	if (errno == EAGAIN)
+	  {
+	     sleep (1);
+	     continue;
+	  }
+#endif
+	return -1;
+     }
+   return fd;
+}
+
+static int signal_safe_fcntl (int fd, int cmd, int arg)
+{
+   int ret;
+
+   while (-1 == (ret = fcntl (fd, cmd, arg)))
+     {
+#ifdef EINTR
+	if (errno == EINTR)
+	  continue;
+#endif
+#ifdef EAGAIN
+	if (errno == EAGAIN)
+	  {
+	     sleep (1);
+	     continue;
+	  }
+#endif
+	return -1;
+     }
+   return ret;
+}
+
+static int signal_safe_dup2 (int fd1, int fd2)
+{
+   while (-1 == dup2 (fd1, fd2))
+     {
+#ifdef EINTR
+	if (errno == EINTR)
+	  continue;
+#endif
+	return -1;
+     }
+   return 0;
+}
+
+
 static Process_Type *get_process (int fd) /*{{{*/
 {
    Process_Type *p;
    
    if ((fd >= 0) && (fd < MAX_PROCESSES)
-       && (p = &Processes[fd], p->flags != 0)) return p;
+       && (p = &Processes[fd], p->flags != 0))
+     return p;
    
    msg_error ("process does not exist.");
    return NULL;
@@ -137,19 +239,18 @@ static void close_rd_and_wd (Process_Type *p) /*{{{*/
 {   
    if (p->rd != -1) 
      {
-	close (p->rd);
+	signal_safe_close (p->rd);
 	p->rd = -1;
      }
    if (p->wd != -1)
      {
-	close (p->wd); 
+	if (p->is_pty == 0) signal_safe_close (p->wd); 
 	p->wd = -1;
      }
 }
 
 /*}}}*/
 
-
 /* This routine is called to clean up after the process has exited.  
  * After getting the exit status, we call a slang hook and if the
  * process is dead, adjust the process arrays to delete the process.
@@ -171,29 +272,31 @@ static void get_process_status (Process_Type *p) /*{{{*/
    
    if (p->buffer != NULL) p->buffer->subprocess = 0;
    slfd = (int) (p - Processes);
+
+   if (p->umark != NULL) SLang_free_mmt (p->umark);
+   memset ((char *)p, 0, sizeof (Process_Type));
    
-   fd = Subprocess_Read_fds [slfd][0];
-   
-   p->flags = 0;
-   if (p->umark != NULL) jed_free_user_object_mark (p->umark);
    /* Adjust the array of read descriptors */
+
    i = 0;
+   
    while (i < Num_Subprocesses)
      {
-	if (Subprocess_Read_fds[i][0] == fd)
-	  {
-	     while (i < Num_Subprocesses - 1)
-	       {
-		  Subprocess_Read_fds[i][0] = Subprocess_Read_fds[i + 1][0];
-		  Subprocess_Read_fds[i][1] = Subprocess_Read_fds[i + 1][1];
-		  i++;
-	       }
-	     break;
-	  }
+	if (Subprocess_Read_fds[i][0] == slfd)
+	  break;
+	  
+	i++;
+     }
+   fd = Subprocess_Read_fds [i][0];
+
+   Num_Subprocesses--;
+   while (i < Num_Subprocesses)
+     {
+	Subprocess_Read_fds[i][0] = Subprocess_Read_fds[i + 1][0];
+	Subprocess_Read_fds[i][1] = Subprocess_Read_fds[i + 1][1];
 	i++;
      }
    
-   Num_Subprocesses--;
    
    if (Max_Subprocess_FD == fd)
      {
@@ -201,7 +304,8 @@ static void get_process_status (Process_Type *p) /*{{{*/
 	fd = -1;
 	while (i < Num_Subprocesses)
 	  {
-	     if (Subprocess_Read_fds[i][0] > fd) fd = Subprocess_Read_fds[i][0];
+	     if (Subprocess_Read_fds[i][0] > fd) 
+	       fd = Subprocess_Read_fds[i][0];
 	     i++;
 	  }
 	Max_Subprocess_FD = fd;
@@ -247,21 +351,23 @@ void jed_kill_process (int fd) /*{{{*/
 
 void jed_get_child_status (void) /*{{{*/
 {
-   Process_Type *p, *pmin;
+   Process_Type *p, *pmax;
    
    Child_Status_Changed_Flag--;
-   pmin = Processes; p = pmin + Num_Subprocesses;
    
    get_process_input (&Number_Zero);
-
-   while (p > pmin)
+   p = Processes;
+   pmax = p + MAX_PROCESSES;
+   
+   while (p < pmax)
      {
-	p--;
-	if (p->status_changed)
+	if (p->flags && p->status_changed)
 	  {
+	     read_process_input ((int) (p - Processes));
 	     p->status_changed--;
 	     get_process_status (p);
 	  }
+	p++;
      }
 }
 
@@ -307,7 +413,7 @@ static void child_signal_handler (int sig) /*{{{*/
 	/* What else?? */
 	
 	p = Processes;
-	pmax = p + Num_Subprocesses;
+	pmax = p + MAX_PROCESSES;
 	while (p < pmax)
 	  {
 	     if (p->pid == pid)
@@ -327,42 +433,310 @@ static void child_signal_handler (int sig) /*{{{*/
 
 /*}}}*/
 
+#ifdef USE_PTY
+# include "pty.c"
+#endif
+
+static int get_master_slave_fds (int *slave_read, int *slave_write,
+				 int *master_read, int *master_write,
+				 char *slave_tty_name, int *is_pty)
+{
+#ifdef USE_PTY
+   int master;
+   
+   if (-1 == pty_open_master_pty (&master, slave_tty_name))
+     return -1;
+
+   *master_read = *master_write = master;
+   *slave_read = *slave_write = -1;
+   
+   *is_pty = 1;
+   return 0;
+#else
+
+   int fds0[2], fds1[2];
+   
+   if (-1 == pipe (fds0))
+     return -1;
+   
+   if (-1 == pipe (fds1))
+     {
+	signal_safe_close (fds0[0]);
+	signal_safe_close (fds0[1]);
+	return -1;
+     }
+   *master_read = fds0[0];
+   *slave_write = fds0[1];
+   *master_write = fds1[1];
+   *slave_read = fds1[0];
+   *slave_tty_name = 0;
+   *is_pty = 0;
+   return 0;
+#endif
+}
+
+
+static void my_setenv (char *what, char *value)
+{
+#ifdef HAVE_SETENV
+   (void) setenv (what, value, 1);
+#else
+# ifdef HAVE_PUTENV
+   char buf[512];
+   sprintf (buf, "%s=%s", what, value);
+   (void) putenv (buf);
+# endif
+#endif
+}
+
+static void my_unsetenv (char *what)
+{
+#ifdef HAVE_UNSETENV
+   unsetenv (what);
+#endif
+}
+
+   
+#ifdef __os2__
 static int open_process (char *pgm, char **argv) /*{{{*/
 {
-   int spfd, val;
-   int pd;
-   int fds0[2], fds1[2];
+    int val;
+    int pd;
+   int slave_read, slave_write, master_read, master_write, org_fd[3];
    int pid, i;
    Process_Type *p;
-   SLuser_Object_Type *uo;
+   SLang_MMT_Type *mmt;
+   char slave_tty_name [80];
+   int max_write_tries;
+   SLSig_Fun_Type *sig_orig[NSIG];
+
+   if (Jed_Secure_Mode)
+     {
+	msg_error ("Access to shell denied.");
+	return -1;
+     }
    
    pd = 0; while ((pd < MAX_PROCESSES) && Processes[pd].flags) pd++;
    if (pd == MAX_PROCESSES) return -1;
    p = &Processes[pd];
 
    SLMEMSET ((char *) p, 0, sizeof (Process_Type));
+
+   if (NULL == (mmt = jed_make_user_object_mark ()))
+     return -1;
    
-   if (0 == pipe (fds0))
+   if (-1 == get_master_slave_fds (&slave_read, &slave_write,
+				   &master_read, &master_write,
+				   slave_tty_name, &p->is_pty))
      {
-	if (-1 == pipe (fds1))
-	  {
-	     close (fds0[0]);
-	     close (fds0[1]);
-	     return -1;
-	  }
+	SLang_free_mmt (mmt);
+	return -1;
      }
-   else return -1;
+
+   SLsignal_intr (SIGCHLD, child_signal_handler);
    
-   if (NULL == (uo = jed_make_user_object_mark (SLANG_IVARIABLE))) return -1;
+     {
+	char ch;
+	
+	/* At this point the slave tty is in raw mode.  Make sure that
+	 * the read to synchronize with the parent blocks.
+	 */
+	val = signal_safe_fcntl (slave_read, F_GETFL, 0);
+	signal_safe_fcntl (slave_read, F_SETFL, val & ~O_NONBLOCK);
+
+	signal_safe_fcntl (slave_read, F_SETFL, val);
+
+	
+	org_fd[0] = dup(0);
+	org_fd[1] = dup(1);
+	org_fd[2] = dup(2);
+
+	signal_safe_close (0);
+	signal_safe_close (1);
+	signal_safe_close (2);
+	
+	signal_safe_dup2(slave_read, 0);	/* stdin */
+	signal_safe_dup2 (slave_write, 1);	/* stdout */
+	signal_safe_dup2 (slave_write, 2);	/* stderr */
+
+	signal_safe_close (slave_read);
+	signal_safe_close (slave_write);
+	setvbuf (stdout, NULL, _IONBF, 0);
+	
+	for (i=1 ; i<NSIG ; i++)
+	  sig_orig[i] = SLsignal(i, SIG_DFL);
+	
+	pid = spawnvp (P_SESSION | P_MINIMIZE | P_BACKGROUND, pgm, argv);
+
+	for (i=1 ; i<NSIG ; i++)
+	  SLsignal(i, sig_orig[i]);
+	signal_safe_close (0);
+	signal_safe_close (1);
+	signal_safe_close (2);
+	signal_safe_dup2(org_fd[0], 0);
+	signal_safe_dup2(org_fd[1], 1);
+	signal_safe_dup2(org_fd[2], 2);
+	signal_safe_close (org_fd[0]); 
+	signal_safe_close (org_fd[1]); 
+	signal_safe_close (org_fd[2]); 
+	signal_safe_close (slave_read);
+	signal_safe_close (slave_write);
+	
+	if (pid < 0)
+	  {
+	     signal_safe_close (master_read);
+	     signal_safe_close (master_write); 
+	     p->flags = 0;
+	     SLang_free_mmt (mmt);
+	     return -1;
+	     
+	  }
+	p->pid = pid;
+     }
+
    
+
+   p->flags = PROCESS_RUNNING;
+   p->rd = master_read;
+   p->wd = master_write;
+   
+   Subprocess_Read_fds[Num_Subprocesses][0] = master_read;
+   Subprocess_Read_fds[Num_Subprocesses][1] = pd;
+   if (master_read > Max_Subprocess_FD) Max_Subprocess_FD = master_read;
+   Num_Subprocesses += 1;
+   
+   val = signal_safe_fcntl (master_read, F_GETFL, 0);
+   val |= O_NONBLOCK;
+   signal_safe_fcntl (master_read, F_SETFL, val);
+
+   CBuf->subprocess = pd + 1;
+
+   /* Processing options */
+   p->buffer = CBuf;
+   p->output_type = PROCESS_USE_BUFFER;
+   p->umark = mmt;
+   SLang_inc_mmt (mmt);	       /* tell slang we are keeping a copy */
+   
+   return pd;
+}
+/*}}}*/
+#else	/* unix and compatible OS */
+
+static volatile int SigUsr_Flag;
+static void sigusr_handler (int sig)
+{
+   (void) sig;
+
+   SigUsr_Flag = 1;
+}
+
+static sigset_t Old_Signal_Mask, Zero_Signal_Mask;
+
+static int init_child_parent_sync (void)
+{
+   sigset_t new_mask;
+
+   if (SIG_ERR == SLsignal (SIGUSR1, sigusr_handler))
+     return -1;
+   
+   /* Get a copy of the current signal mask */
+   while ((-1 == sigprocmask (SIG_BLOCK, NULL, &Zero_Signal_Mask))
+	  && (errno == EINTR))
+     ;
+   /* and remove SIGUSR1 from it. */
+   sigdelset (&Zero_Signal_Mask, SIGUSR1);
+
+   /* Add SIGUSR1 to the set of currently blocked signals */
+   sigemptyset (&new_mask);
+   sigaddset (&new_mask, SIGUSR1);
+
+   while ((-1 == sigprocmask (SIG_BLOCK, &new_mask, &Old_Signal_Mask))
+	  && (errno == EINTR))
+     ;
+   
+   SigUsr_Flag = 0;
+   return 0;
+}
+
+static void wait_for_parent (void)
+{
+   /* Suspend this process and wait for a signal.  All signals are allowed
+    * here.  
+    */
+   while (SigUsr_Flag == 0)
+     sigsuspend (&Zero_Signal_Mask);
+   
+   SigUsr_Flag = 0;
+   
+   while ((-1 == sigprocmask (SIG_SETMASK, &Old_Signal_Mask, NULL))
+	  && (errno == EINTR))
+     ;
+}
+
+static void tell_child_to_go (pid_t pid)
+{   
+   kill (pid, SIGUSR1);
+
+   SigUsr_Flag = 0;
+   while ((-1 == sigprocmask (SIG_SETMASK, &Old_Signal_Mask, NULL))
+	  && (errno == EINTR))
+     ;
+}
+
+static int set_non_blocking_io (int fd)
+{   
+   int val;
+   
+   val = signal_safe_fcntl (fd, F_GETFL, 0);
+   return signal_safe_fcntl (fd, F_SETFL, val | O_NONBLOCK);
+}
+
+static int open_process (char *pgm, char **argv) /*{{{*/
+{
+   int pd;
+   int slave_read, slave_write, master_read, master_write;
+   int pid, i;
+   Process_Type *p;
+   SLang_MMT_Type *mmt;
+   char slave_tty_name [80];
+
+   if (Jed_Secure_Mode)
+     {
+	msg_error ("Access to shell denied.");
+	return -1;
+     }
+   
+   pd = 0; while ((pd < MAX_PROCESSES) && Processes[pd].flags) pd++;
+   if (pd == MAX_PROCESSES) return -1;
+   p = &Processes[pd];
+
+   SLMEMSET ((char *) p, 0, sizeof (Process_Type));
+
+   if (NULL == (mmt = jed_make_user_object_mark ()))
+     return -1;
+   
+   if (-1 == get_master_slave_fds (&slave_read, &slave_write,
+				   &master_read, &master_write,
+				   slave_tty_name, &p->is_pty))
+     {
+	SLang_free_mmt (mmt);
+	return -1;
+     }
+
    SLsignal_intr (SIGCHLD, child_signal_handler);
 
-   if ((pid = fork ()) < 0)
+   if ((-1 == init_child_parent_sync ())
+       || ((pid = fork ()) < 0))
      {
-	close (fds1[0]); close (fds1[1]);
-	close (fds0[0]); close (fds0[1]);
+	signal_safe_close (master_read);
+	if (p->is_pty == 0)
+	  {
+	     signal_safe_close (slave_read);
+	     signal_safe_close (master_write); 
+	     signal_safe_close (slave_write);
+	  }
 	p->flags = 0;
-	jed_free_user_object_mark (uo);
+	SLang_free_mmt (mmt);
 	return -1;
      }
    p->pid = pid;
@@ -373,69 +747,99 @@ static int open_process (char *pgm, char **argv) /*{{{*/
     * them unless they catch the signal.  This call means that the INTR signal
     * will not be sent to any child processes sent by this fork.
     */
-   setpgid(pid, pid);
+   if (p->is_pty == 0) 
+     setpgid(pid, pid);
 
-   
    if (pid == 0)
      {
-	char ch;
 	/* child code */
-	/* Wait here for the parent to initialize its structures */
-	/* This will block */
-	read (fds0[0], &ch, 1);  
+	wait_for_parent ();
 	
-	for (i = 0; i < 32; i++) SLsignal_intr (i, SIG_DFL);
+	if (p->is_pty == 0)
+	  signal_safe_close (master_write);	       /* close write end of 0 */
+	signal_safe_close (master_read);	       /* close read end of 1 */
+
+	for (i = 0; i < 32; i++) SLsignal (i, SIG_DFL);
 	
-	close (fds0[1]);	       /* close write end of 0 */
-	close (fds1[0]);	       /* close read end of 1 */
-	
-	if ((dup2(fds0[0], 0) < 0)     /* stdin */
-	    || (dup2 (fds1[1], 1) < 0) /* stdout */
-	    || (dup2 (fds1[1], 2) < 0)) /* stderr */
+#ifdef USE_PTY	
+	/* Call set setsid so that the child will become the session leader.
+	 * This has the side effect that we will loose the controlling
+	 * terminal.  For this reason, the pty slave is opened after setsid
+	 * and then for good luck, the controlling terminal is set 
+	 * via the TIOCSCTTY ioctl.
+	 */
+	if (p->is_pty)
 	  {
-	     fprintf (stderr, "Dups failed!\n");
-	     exit (-1);
+	     if (-1 == setsid ())
+	       fprintf (stderr, "child: setsid failed.\n");
+	
+	     if (-1 == pty_open_slave_pty (slave_tty_name, &slave_read))
+	       {
+		  fprintf (stderr, "child: failed to open slave.");
+		  _exit (1);
+	       }
+	     slave_write = slave_read;
+
+	     /* (void) pty_setup_slave_term (slave_read, 0); */
 	  }
-	
-	
+#endif	/* USE_PTY */
+
+#ifdef USE_PTY
+	/* Put tty back into cbreak mode. */
+	if (p->is_pty) (void) pty_setup_slave_term (slave_read, 0);
+#endif
+	if ((signal_safe_dup2(slave_read, 0) < 0)     /* stdin */
+	    || (signal_safe_dup2 (slave_write, 1) < 0) /* stdout */
+	    || (signal_safe_dup2 (slave_write, 2) < 0)) /* stderr */
+	  {
+	     fprintf (stderr, "dup2 failed. errno = %d\n", errno);
+	     _exit (1);
+	  }
+
+	my_setenv ("TERM", "unknown");
+	my_unsetenv ("TERMCAP");
+
 	if (execvp (pgm, argv) < 0)
 	  {
-	     fprintf (stderr, "exec failed!\n");
-	     exit (-1);
+	     fprintf (stderr, "execvp of %s failed!\r\n", pgm);
+	     _exit (1);
 	  }
      }
    
    /* parent */
-   close (fds0[0]);		       /* close read of child read */
-   close (fds1[1]);		       /* close write of child write */
+   if (p->is_pty == 0) 
+     {
+	signal_safe_close (slave_read);
+	signal_safe_close (slave_write);
+     }
 
    p->flags = PROCESS_RUNNING;
-   spfd = p->rd = fds1[0];
-   p->wd = fds0[1];
+   p->rd = master_read;
+   p->wd = master_write;
    
-   Subprocess_Read_fds[Num_Subprocesses][0] = spfd;
+   Subprocess_Read_fds[Num_Subprocesses][0] = master_read;
    Subprocess_Read_fds[Num_Subprocesses][1] = pd;
-   if (spfd > Max_Subprocess_FD) Max_Subprocess_FD = spfd;
+   if (master_read > Max_Subprocess_FD) Max_Subprocess_FD = master_read;
    Num_Subprocesses += 1;
    
-   val = fcntl (spfd, F_GETFL, 0);
-   val |= O_NONBLOCK;
-   fcntl (spfd, F_SETFL, val);
-
    CBuf->subprocess = pd + 1;
 
    /* Processing options */
    p->buffer = CBuf;
    p->output_type = PROCESS_USE_BUFFER;
-   p->umark = uo;
+   p->umark = mmt;
+   SLang_inc_mmt (mmt);	       /* tell slang we are keeping a copy */
    
+   set_non_blocking_io (master_read);
+   set_non_blocking_io (master_write);
+
    /* Tell child it is ok to go. */
-   write (p->wd, "&", 1);
-   
+   tell_child_to_go (pid);
    return pd;
 }
 
 /*}}}*/
+#endif				       /* __os2__ */
 
 
 /* This function is only called when we are reading characters from the 
@@ -481,8 +885,8 @@ void read_process_input (int fd) /*{{{*/
 	else if (otype == PROCESS_USE_SLANG)
 	  {
 	     buf[n] = 0;
-	     SLang_push_string ((char *) buf);
 	     SLang_push_integer ((int) (p - Processes));
+	     SLang_push_string ((char *) buf);
 	     SLexecute_function (p->slang_fun);    /* function to pass output to */
 	  }
      }
@@ -495,17 +899,44 @@ void read_process_input (int fd) /*{{{*/
 	if (b != CBuf) switch_to_buffer (b);
 	pbuf->locked--;
      }
-   if (total) touch_screen ();
+   if (total)
+     {
+#ifdef __os2__
+	/* Why does OS/2 need this?? -- JED */
+	JWindow->trashed = 1;
+	update ((Line *)NULL, 0, 1, 0);
+#endif
+	touch_screen ();
+     }
 }
 
+
 /*}}}*/
 
+static int write_to_process (int fd, char *buf, unsigned int len)
+{
+   if (len == 0) 
+     return 0;
+   
+   /* FIXME!! check for write errors!! */
+   while (-1 == write (fd, buf, len))
+     {
+	if (errno != EINTR)
+	  break;
+     }
+   
+   return 0;
+}
+
 int jed_send_process (int *fd, char *str) /*{{{*/
 {
+   unsigned int len;
    Process_Type *p = get_process (*fd);
    if ((p == NULL) || (p->wd == -1)) return -1;
-   write (p->wd, str, strlen(str));
-   return 0;
+   
+   len = strlen (str);
+   
+   return write_to_process (p->wd, str, len);
 }
 
 /*}}}*/
@@ -513,12 +944,23 @@ int jed_send_process (int *fd, char *str) /*{{{*/
 void jed_send_process_eof (int *fd) /*{{{*/
 {
    Process_Type *p = get_process (*fd);
-   if (p == NULL) return;
    
-   if (p->wd != -1) close (p->wd);
-   p->wd = -1;
+   if (p == NULL) 
+     return;
+   
+   if (p->wd == -1)
+     return;
+   
+   if (p->is_pty)
+     write_to_process (p->wd, "\004", 1);
+   else
+     {
+	signal_safe_close (p->wd);
+	p->wd = -1;
+     }
 }
 
+
 /*}}}*/
 
 void jed_set_process (int *pd, char *what, char *s) /*{{{*/
@@ -554,7 +996,7 @@ void jed_get_process_mark (int *fd) /*{{{*/
    Process_Type *p;
    if (NULL == (p = get_process (*fd))) return;
    
-   SLang_push_user_object (p->umark);
+   SLang_push_mmt (p->umark);
 }
 
 /*}}}*/
@@ -583,7 +1025,7 @@ int jed_open_process (int *np) /*{{{*/
    argv[n] = NULL;
    while (n--)
      {
-	if (SLpop_string (&argv[n]))
+	if (SLang_pop_slstring (&argv[n]))
 	  {
 	     n++;
 	     goto free_return;
@@ -601,7 +1043,7 @@ int jed_open_process (int *np) /*{{{*/
    
    while (n <= *np)
      {
-	SLFREE (argv[n]);
+	SLang_free_slstring (argv[n]);
 	n++;
      }
    
@@ -685,17 +1127,17 @@ FILE *jed_popen(char *cmd, char *type) /*{{{*/
      {				       /* child */
 	if (*type == 'r') 
 	  {
-	     close(pfd[0]);
-	     if (pfd[1] != 1) dup2(pfd[1], 1);
-	     close(pfd[1]);
+	     signal_safe_close(pfd[0]);
+	     if (pfd[1] != 1) signal_safe_dup2(pfd[1], 1);
+	     signal_safe_close(pfd[1]);
 	  }
 	else
 	  {
-	     close(pfd[1]);
+	     signal_safe_close(pfd[1]);
 	     if (pfd[0] != 0)
 	       {
-		  dup2(pfd[0], STDIN_FILENO);
-		  close(pfd[0]);
+		  signal_safe_dup2(pfd[0], STDIN_FILENO);
+		  signal_safe_close(pfd[0]);
 	       }
 	  }
 	
@@ -704,7 +1146,7 @@ FILE *jed_popen(char *cmd, char *type) /*{{{*/
 	 */
 	for (i = 0; i < OPEN_MAX; i++)
 	  {
-	     if (Popen_Child_Pids[i] > 0) close(i);
+	     if (Popen_Child_Pids[i] > 0) signal_safe_close(i);
 	  }
 
 	execl("/bin/sh", "sh", "-c", cmd, (char *) 0);
@@ -714,13 +1156,13 @@ FILE *jed_popen(char *cmd, char *type) /*{{{*/
    /* parent */
    if (*type == 'r') 
      {
-	close(pfd[1]);
+	signal_safe_close(pfd[1]);
 	if (NULL == (fp = fdopen(pfd[0], type)))
 	  return(NULL);
      } 
    else 
      {
-	close(pfd[0]);
+	signal_safe_close(pfd[0]);
 	if (NULL == (fp = fdopen(pfd[1], type)))
 	  return(NULL);
      }
