@@ -1,9 +1,19 @@
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
+ */
 #define MAX_PROCESSES 10
 
 #ifdef __WIN32__
-#include <windows.h>
-#include <stdio.h>
-extern CRITICAL_SECTION Critical_Section;
+# ifdef VOID
+#  undef VOID
+# endif
+# include <windows.h>
+# include <stdio.h>
+/* # define WIN95BUG	1 */
+/* extern CRITICAL_SECTION Critical_Section; */
 extern HANDLE Input_Events[];
 #else
 extern int Subprocess_Read_fds [MAX_PROCESSES][2];
