@@ -1,7 +1,9 @@
 /* -*- mode: C; mode: fold; -*- */
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include "config.h"
 #include "jed-feat.h"
@@ -31,6 +33,13 @@
 #include "ins.h"
 #include "hooks.h"
 #include "replace.h"
+#if JED_HAS_TTY_MENUS
+# include "menu.h"
+#endif
+
+#ifdef IBMPC_SYSTEM
+# include "doskeys.h"
+#endif
 
 /*}}}*/
 
@@ -51,15 +60,15 @@ int *Repeat_Factor;                    /* repeats a key sequence if non null */
 
 char *Read_This_Character = NULL;      /* alternate keyboard buffer */
 
-char Jed_Key_Buffer[13];
-char Key_Buffer[13];
+char Jed_Key_Buffer[SLANG_MAX_KEYMAP_KEY_SEQ + 1];
+char Key_Buffer[SLANG_MAX_KEYMAP_KEY_SEQ + 1];
 char *Key_Bufferp = Key_Buffer;
 static int Last_Key_Buffer_Len;
 
 int Jed_Max_Hits = 300;
 
 
-int beep (void) /*{{{*/
+int jed_beep (void) /*{{{*/
 {
    tt_beep ();
    flush_output ();
@@ -68,17 +77,64 @@ int beep (void) /*{{{*/
 
 /*}}}*/
 
+static int redraw (void)
+{
+   jed_redraw_screen (0);
+   return 0;
+}
+
+#ifndef IBMPC_SYSTEM
+static int xterm_mouse_cmd (void)
+{
+   int b, id;
+   static int last_down = JMOUSE_BUTTON_1;
+   JMouse_Type m;
+
+   b = my_getkey ();
+   m.x = (unsigned char) my_getkey () - 32;
+   m.y = (unsigned char) my_getkey () - 32;
+   
+   b -= 32;
+
+   if ((b & 3) == 3)
+     {
+	m.type = JMOUSE_UP;
+	m.button = last_down;
+	m.state = last_down;
+     }
+   else
+     {
+	m.type = JMOUSE_DOWN;
+	if ((b & 3) == 0)
+	  m.button = JMOUSE_BUTTON_1;
+	else if (b & 1)
+	  m.button = JMOUSE_BUTTON_2;
+	else if (b & 2)
+	  m.button = JMOUSE_BUTTON_3;
+	m.state = 0;
+     }
+   last_down = m.button;
+
+   if (-1 == (id = jed_mouse_add_event (&m)))
+     return -1;
+   
+   ungetkey (&id);
+   jed_mouse_cmd ();
+   return 1;
+}
+#endif
+
 SLKeymap_Function_Type Jed_Functions[] = /*{{{*/
 {
      {"backward_delete_char", backward_delete_char_cmd},
      {"backward_delete_char_untabify", backward_delete_char_untabify},
-     {"beep", beep},
+     {"beep", jed_beep},
      {"begin_macro", begin_keyboard_macro},
      {"beg_of_buffer", bob},
      {"beg_of_line", bol},
      {"center_line", center_line},
      {"copy_region", copy_to_pastebuffer},
-#if (defined(msdos) || defined(__GO32__)) && !defined(__WIN32__)
+#if defined(__MSDOS__) && !defined(__WIN32__)
      {"coreleft", show_memory},
 #endif
      {"delete_char_cmd", delete_char_cmd},
@@ -112,7 +168,6 @@ SLKeymap_Function_Type Jed_Functions[] = /*{{{*/
      {"narrow", narrow_to_lines},
      {"narrow_paragraph", narrow_paragraph},
      {"narrow_to_region", narrow_to_region},
-     {"newline", newline},
      {"newline", newline_cmd},
      {"newline_and_indent", newline_and_indent},
      {"next_char_cmd", next_char_cmd},
@@ -145,6 +200,9 @@ SLKeymap_Function_Type Jed_Functions[] = /*{{{*/
      {"widen_region", widen_region},
      {"write_buffer", write_buffer},
      {"yank", yank},
+#if JED_HAS_TTY_MENUS
+     {"select_menubar", jed_select_menu_bar},
+#endif
      {(char *) NULL, NULL}
 };
 
@@ -158,13 +216,6 @@ int kbd_quit(void) /*{{{*/
    int sle = SLang_Error;
    SLKeyBoard_Quit = 1;
    
-   /* --- never serious, testing only
-   if (Repeat_Factor != NULL) 
-     {
-	msg_error("Bang!");
-	longjmp(Jump_Buffer_Ptr->b, 1);
-     } */
-   
    msg_error("Quit!");
    if (Ignore_User_Abort) SLang_Error = sle;
 #ifdef UNDO_HAS_REDO
@@ -215,7 +266,41 @@ void init_keymaps(void) /*{{{*/
    SLkm_define_key ("^[9", (FVOID_STAR) digit_arg, Global_Map);
    SLkm_define_key ("^[0", (FVOID_STAR) digit_arg, Global_Map);
    
-#ifndef pc_system
+#ifdef IBMPC_SYSTEM
+   SLkm_define_key (PC_DEL, (FVOID_STAR) delete_char_cmd, Global_Map);
+   SLkm_define_key (PC_DEL1, (FVOID_STAR) delete_char_cmd, Global_Map);
+   SLkm_define_key (PC_NULL, (FVOID_STAR) set_mark_cmd, Global_Map);
+   SLkm_define_key (PC_LT, (FVOID_STAR) previous_char_cmd, Global_Map);
+   SLkm_define_key (PC_LT1, (FVOID_STAR) previous_char_cmd, Global_Map);
+   SLkm_define_key (PC_UP, (FVOID_STAR) previous_line_cmd, Global_Map);
+   SLkm_define_key (PC_UP1, (FVOID_STAR) previous_line_cmd, Global_Map);
+   SLkm_define_key (PC_DN, (FVOID_STAR) next_line_cmd, Global_Map);
+   SLkm_define_key (PC_DN1, (FVOID_STAR) next_line_cmd, Global_Map);
+   SLkm_define_key (PC_RT, (FVOID_STAR) next_char_cmd, Global_Map);
+   SLkm_define_key (PC_RT1, (FVOID_STAR) next_char_cmd, Global_Map);
+   SLkm_define_key (PC_PGUP, (FVOID_STAR) pageup_cmd, Global_Map);
+   SLkm_define_key (PC_PGUP1, (FVOID_STAR) pageup_cmd, Global_Map);
+   SLkm_define_key (PC_PGDN, (FVOID_STAR) pagedown_cmd, Global_Map);
+   SLkm_define_key (PC_PGDN1, (FVOID_STAR) pagedown_cmd, Global_Map);
+   SLkm_define_key (PC_HOME, (FVOID_STAR) bol, Global_Map);
+   SLkm_define_key (PC_HOME1, (FVOID_STAR) bol, Global_Map);
+   SLkm_define_key (PC_END, (FVOID_STAR) eol_cmd, Global_Map);
+   SLkm_define_key (PC_END1, (FVOID_STAR) eol_cmd, Global_Map);
+   
+   /* Now special keypad stuff */
+   SLkm_define_key (PC_ENTER, (FVOID_STAR) newline_cmd, Global_Map);
+   
+   /* wordperfect type stuff */
+   SLkm_define_key (PC_F1, (FVOID_STAR) kbd_quit, Global_Map);
+   /* SLkm_define_key (PC_F2, (FVOID_STAR) search_forward_cmd, Global_Map);
+    SLkm_define_key (PC_SHIFT_F2, (FVOID_STAR) search_backward_cmd, Global_Map); */
+   SLkm_define_key (PC_F4, (FVOID_STAR) indent_line, Global_Map);
+   SLkm_define_key (PC_ALT_F5, (FVOID_STAR) set_mark_cmd, Global_Map);
+   SLkm_define_key (PC_SHIFT_F4, (FVOID_STAR) narrow_paragraph, Global_Map);
+   SLkm_define_key (PC_SHIFT_F6, (FVOID_STAR) center_line, Global_Map);
+   SLkm_define_key (PC_F7, (FVOID_STAR) exit_jed, Global_Map);
+#else
+   
    /* give vtxxx arrow keys */
    SLkm_define_key ("^[[D", (FVOID_STAR) previous_char_cmd, Global_Map);
    SLkm_define_key ("^[OD", (FVOID_STAR) previous_char_cmd, Global_Map);
@@ -231,23 +316,22 @@ void init_keymaps(void) /*{{{*/
    SLkm_define_key ("^K^[[D", (FVOID_STAR) scroll_right, Global_Map);
    SLkm_define_key ("^K^[[A", (FVOID_STAR) bob, Global_Map);
    SLkm_define_key ("^K^[[B", (FVOID_STAR)eob, Global_Map);
-   
+#if HAS_MOUSE
+   SLkm_define_key ("\033[M", (FVOID_STAR)xterm_mouse_cmd, Global_Map);
+#endif
 #ifdef sun
    SLkm_define_key ("\033[216z", (FVOID_STAR) pageup_cmd, Global_Map);
    SLkm_define_key ("\033[222z", (FVOID_STAR) pagedown_cmd, Global_Map);
    SLkm_define_key ("\033[214z", (FVOID_STAR) bol, Global_Map);
    SLkm_define_key ("\033[220z", (FVOID_STAR) eol_cmd, Global_Map);
 #endif
-   
-#else
-#include "doskeys.c"
-#endif
+#endif   
    
    SLkm_define_key ("'", (FVOID_STAR) text_smart_quote, Global_Map);
    SLkm_define_key ("\"", (FVOID_STAR) text_smart_quote, Global_Map);
    SLkm_define_key ("^_", (FVOID_STAR) undo, Global_Map);
    SLkm_define_key ("^?", (FVOID_STAR) backward_delete_char_untabify, Global_Map);
-#ifndef pc_system
+#ifndef IBMPC_SYSTEM
    SLkm_define_key ("^H", (FVOID_STAR) backward_delete_char_untabify, Global_Map);
 #endif
    SLkm_define_key ("^B", (FVOID_STAR) bol, Global_Map);
@@ -429,8 +513,14 @@ int do_key (void) /*{{{*/
 	Key_Bufferp = Key_Buffer;
 	*Key_Bufferp = 0;
      }
-   
-   key = SLang_do_key (CBuf->keymap, jed_getkey);
+
+#if JED_HAS_TTY_MENUS
+   if (Jed_Menus_Active)
+     key = jed_menu_do_key ();
+   else
+#endif
+     key = SLang_do_key (CBuf->keymap, jed_getkey);
+
    update_jed_keybuffer ();
    
    if ((key != NULL) && (key->f.f != NULL))
@@ -453,7 +543,7 @@ int do_key (void) /*{{{*/
      }
    else if (!Executing_Keyboard_Macro && !SLKeyBoard_Quit)
      {
-	beep ();
+	jed_beep ();
 	flush_input ();
      }
    if (SLKeyBoard_Quit) kbd_quit();
@@ -464,23 +554,31 @@ int do_key (void) /*{{{*/
 
 void do_jed(void) /*{{{*/
 {
-   char *name;
+   char *mode;
    Buffer *tthis = CBuf;
-   
-   /* Mark Undo boundary now because tthis might not be valid later */
-   mark_undo_boundary(tthis);
-   
+
+   /* Mark Undo boundary now because this might not be valid later */
+   mark_undo_boundary (tthis);
+   mode = CBuf->mode_string;
+
    Repeat_Factor = NULL;
    Replace_Preserve_Case = 0;
    if (do_key()) JWindow->trashed = 1;
    
     /* internal editing commands may have selected a different buffer
-       so put it back. */
+     * so put it back to the one associated with the window.
+     */
    if (CBuf != JWindow->buffer)
      {
+	char *name;
 	if (buffer_exists(JWindow->buffer)) name = JWindow->buffer->name; else name = "*scratch*";
 	switch_to_buffer_cmd(name);
      }
+
+#if JED_HAS_TTY_MENUS
+   if ((tthis != CBuf) || (CBuf->mode_string != mode))
+     jed_notify_menu_buffer_changed ();
+#endif
 }
 
 /*}}}*/
@@ -490,9 +588,14 @@ void jed(void) /*{{{*/
    /* This routine is called from main.  So before actually strting, check
       one more hook to just to make sure  all things are go. */
    char *startup = ".()jed_startup_hook";
+
+#if JED_HAS_TTY_MENUS
+   jed_notify_menu_buffer_changed ();
+#endif
+   
    if (2 == SLang_is_defined(startup + 3))
      {
-	SLang_run_hooks(startup + 3, NULL, NULL);
+	SLang_run_hooks(startup + 3, 0);
 	/* remove its definition from memory */
 	SLang_load_string (startup);
      }
@@ -510,14 +613,19 @@ void jed(void) /*{{{*/
 	switch_to_buffer(JWindow->buffer);
 	window_buffer(CBuf);
      }
+
+#if JED_HAS_TTY_MENUS
+   jed_notify_menu_buffer_changed ();
+#endif
+
    JWindow->trashed = 1;
-   update((Line *) NULL, 0, 0);
+   update((Line *) NULL, 0, 0, 1);
    Read_This_Character = NULL;
    while(1)
      {
 	Suspend_Screen_Update = 0;
 	do_jed();
-	if (!SLKeyBoard_Quit && (CBuf->flags & BUFFER_TRASHED)
+	if (!SLKeyBoard_Quit && (CBuf->flags & BUFFER_MODIFIED)
 	    && (!Cursor_Motion)) CBuf->hits += 1;
 	if (CBuf->hits > Jed_Max_Hits)
 	  {
@@ -530,7 +638,7 @@ void jed(void) /*{{{*/
 	    || Suspend_Screen_Update ||
 	    (Mini_Ghost) || (*Message_Buffer) || (*Error_Buffer))
 	  {
-	     update((Line *) NULL, 0, 0);
+	     update((Line *) NULL, 0, 0, 1);
 	  }
      }
 }
@@ -542,7 +650,7 @@ int digit_arg(void) /*{{{*/
    static int repeat;
    char buf[20];
    int key;
-   int i;
+   unsigned int i;
    
    i = 0;
    buf[i++] = (char) SLang_Last_Key_Char;
@@ -555,13 +663,14 @@ int digit_arg(void) /*{{{*/
    Key_Bufferp = Key_Buffer + Last_Key_Buffer_Len;
    
    SLang_Key_TimeOut_Flag = 1;
-   while(1)
+   while (i < sizeof(buf)-1)
      {
 	buf[i] = 0;
 	key = jed_getkey();
 	if ((key < '0') || (key > '9')) break;
 	buf[i++] = (char) key;
      }
+   buf[i] = 0;
    repeat = atoi(buf);
    Repeat_Factor = &repeat;
    if (Executing_Keyboard_Macro) Macro_Buffer_Ptr--;
@@ -601,8 +710,9 @@ int which_key (char *f) /*{{{*/
 	    (((type == SLKEY_F_INTERPRET) && (!strcmp((char *) f, key_root->f.s)))
 	     || ((type == SLKEY_F_INTRINSIC) && (fp == key_root->f.f))))
 	  {
-	     *buf = 2;
-	     *(buf + 1) = 256 - 1 - i;
+	     buf[0] = 2;
+	     buf[1] = 256 - 1 - i;
+	     buf[2] = 0;
 	     SLang_push_string(SLang_make_keystring(buf));
 	     num++;
 	  }
@@ -695,17 +805,20 @@ static void dump_this_binding (SLang_Key_Type *key) /*{{{*/
      }
    else fun = key->f.s;
    
+   if (fun == NULL) fun = "** Unknown **";
    insert_string (fun);
    newline();
 }
 
 /*}}}*/
 
-void  dump_bindings(char *map) /*{{{*/
+void dump_bindings(char *map) /*{{{*/
 {
    int i;
    SLang_Key_Type *next, *key_root;
    SLKeyMap_List_Type *kml;
+
+   CHECK_READ_ONLY_VOID
    
    if (NULL == (kml = SLang_find_keymap(map)))
      {
@@ -744,7 +857,12 @@ char *find_key(int *ret) /*{{{*/
    key = SLang_do_key (CBuf->keymap, jed_getkey);
    update_jed_keybuffer ();
    
-   if (key == NULL) return(NULL);
+   if (key == NULL)
+     {
+	flush_input ();
+	return NULL;
+     }
+
    if (key->type == SLKEY_F_INTRINSIC)
      {
 	*ret = 1;
@@ -760,11 +878,13 @@ char *find_key(int *ret) /*{{{*/
 void use_keymap(char *name) /*{{{*/
 {
    SLKeyMap_List_Type *map;
-   if (NULL == (map = SLang_find_keymap(name)))
+   
+   if ((name == NULL) || (*name == 0)
+       || (NULL == (map = SLang_find_keymap(name))))
      {
 	msg_error("Unknown keymap.");
      }
-   else (CBuf->keymap = map);
+   else CBuf->keymap = map;
 }
 
 /*}}}*/
@@ -780,7 +900,7 @@ void set_abort_char(int *c) /*{{{*/
 {
    char str[2];
    int i;
-#ifdef pc_system
+#ifdef __MSDOS_16BIT__
    char ch, *s, *scan = "@#$%^&*()_+?IQWERTYUIOP[]!!ASDFGHJKL!!!!\\ZXCVBNM";
    
    ch = 64 + (char) *c;
@@ -846,7 +966,9 @@ int next_function_list(char *buf) /*{{{*/
      {
 	name = *Slang_Flist_Context;
 	if (name == NULL) return(0);
+#if 0
 	name++;  /* skip past hash mark */
+#endif
 	Slang_Flist_Context++;
 	if (!Flist_Context_Len || !strncmp(buf, name, Flist_Context_Len))
 	  {
@@ -874,13 +996,16 @@ void add_to_completion(char *name) /*{{{*/
    char **p, *n;
    static char **last = Slang_Functions;
    
-   n = SLang_find_name(name);
-   if (n == NULL)
+   if (0 == SLang_is_defined (name))
      {
-	msg_error("Undefined Symbol.");
+	jed_verror ("add_completion: %s undefined", name);
 	return;
      }
-   
+
+   n = SLang_create_slstring (name);
+   if (n == NULL)
+     return;
+
    p = last;  /* Slang_Functions; */
    
    while (p < Slang_Functions + MAX_USER_FLIST)
@@ -888,12 +1013,14 @@ void add_to_completion(char *name) /*{{{*/
 	if (*p == NULL)
 	  {
 	     *p = n;
-	     last = ++p;
+	     last = p + 1;
 	     return;
 	  }
 	p++;
      }
-   msg_error("Completion Quota Exceeded.");
+
+   SLang_free_slstring (n);
+   /* msg_error("Completion Quota Exceeded."); */
 }
 
 /*}}}*/
