@@ -1,7 +1,9 @@
 /* -*- mode: C; mode: fold; -*- */
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include "config.h"
 #include "jed-feat.h"
@@ -35,10 +37,12 @@
 
 #include <string.h>
 
-#ifdef pc_system
+#ifdef IBMPC_SYSTEM
 # include <fcntl.h>
 #endif
 
+#include <stdarg.h>
+
 #include "buffer.h"
 #include "keymap.h"
 #include "file.h"
@@ -56,6 +60,11 @@
 #include "text.h"
 #include "hooks.h"
 #include "undo.h"
+#include "menu.h"
+
+#ifdef __WIN32__
+# include "win32.h"
+#endif
 
 #define JED_PROMPT "S-Lang>"
 
@@ -76,11 +85,7 @@ extern char *get_cwd(void);
 int Ignore_Beep = 0;
 int MiniBuffer_Active = 0;
 
-#ifdef pc_system
-char Jed_Library[JED_MAX_PATH_LEN];
-#else
-char Jed_Library[1024];
-#endif
+char *Jed_Library;
 
 int (*complete_open)(char *);
 int (*complete_next)(char *);
@@ -91,44 +96,6 @@ static char *Expect_File_Error = "Expecting filename.";
 static char *Keymap_Error = "Unknown_Keymap";
 
 
-/* read doc file */
-int get_doc_string(char *f, char *file) /*{{{*/
-{
-   char *buf, *b, *f1;
-   VFILE *fp;
-   int ret = 0;
-   unsigned int n;
-   char nline = 1;		       /* ascii value of line term (^A) */
-   
-   if (*f == 0) return(0);
-   
-   if ((fp = vopen(file, 1024, VFILE_TEXT)) == NULL) return(0);
-   
-   while ((buf = vgets(fp, &n)) != NULL)
-     {
-	f1 = f; b = buf;
-	while (*f1 && (*f1 == *b)) f1++, b++;
-	if (*f1) continue;
-	
-	b = buf;
-	b[n - 1] = 0;
-	
-	while (*b)
-	  {
-	     if (*b == nline) *b = '\n';
-	     b++;
-	  }
-	
-	SLang_push_string(buf + 1);   /* lose the prefix */
-	ret = 1;
-	break;
-     }
-   vclose(fp);
-   return(ret);
-}
-
-/*}}}*/
-
 static int next_bufflist(char *buf) /*{{{*/
 {
    Buffer *tthis;
@@ -137,15 +104,18 @@ static int next_bufflist(char *buf) /*{{{*/
 	tthis = Buf_Context;
 	if (tthis == MiniBuffer) return(0);
 	Buf_Context = Buf_Context->next;
-#ifdef __os2__
- 	if (!Buf_List_Len || !strncmpi(buf, tthis->name, Buf_List_Len))
+
+	if ((Buf_List_Len == 0)
+#if JED_FILE_PRESERVE_CASE
+	    || (0 == strncmp(buf, tthis->name, Buf_List_Len))
 #else
-	  if (!Buf_List_Len || !strncmp(buf, tthis->name, Buf_List_Len))
+	    || (0 == jed_case_strncmp (buf, tthis->name, Buf_List_Len))
 #endif
+	    )
 	  {
 	     if (*tthis->name == ' ') continue;   /* internal buffers */
-	     strcpy(buf, tthis->name);
-	     return(1);
+	     strcpy (buf, tthis->name);
+	     return 1;
 	  }
      }
 }
@@ -211,12 +181,7 @@ static Buffer *find_scratch_buffer (void) /*{{{*/
    
    scratch = find_buffer (sname);
    if (NULL == scratch)
-     {
-	scratch = make_buffer();
-	strcpy(scratch->name, sname);
-	strcpy(scratch->dir, CBuf->dir);
-	*scratch->file = 0;
-     }
+     scratch = make_buffer (sname, CBuf->dir, NULL);
    return scratch;
 }
 
@@ -258,7 +223,13 @@ int kill_buffer_cmd(char *name) /*{{{*/
 	msg_error("Buffer does not exist.");
 	return(0);
      }
-   
+
+   /* Do not allow the minibuffer to be deleted.  Otherwise, the whole 
+    * minibuffer creation/selection needs to be cleaned up.
+    */
+   if (kill_buf == The_MiniBuffer)
+     return 0;
+
    this_buf = CBuf;
    switch_to_buffer(kill_buf);
    
@@ -272,13 +243,14 @@ int kill_buffer_cmd(char *name) /*{{{*/
 #endif
    
    kill = 1;
-   if ((*name != ' ') && (kill_buf->flags & BUFFER_TRASHED))
+   if ((*name != ' ') && (kill_buf->flags & BUFFER_MODIFIED))
      {
-	char msgbuf[JED_MAX_PATH_LEN];
-	sprintf (msgbuf,"Buffer %s modified. Kill, Save First, Abort: [KSA]", name);
-	flush_message(msgbuf);
+	jed_vmessage (1, "Buffer %s modified. (K)ill, (S)ave first, (A)bort:",
+		      name);
+
 	/* This does not go through keyboard macro routines
-	   on purpose! */
+	 * on purpose! 
+	 */
 	switch (my_getkey())
 	  {
 	   case 'k': case 'K': kill = 1; break;
@@ -312,33 +284,28 @@ int kill_buffer_cmd(char *name) /*{{{*/
 	switch_to_buffer(this_buf);
 	return 1;
      }
-   
-   
-   /* find a buffer to replace this one with if it is in a window */
-   if (buffer_visible(kill_buf))
+
+   buf = find_non_visible_buffer (kill_buf);
+
+   /* search through windows looking for the buffer and replace it */
+   w = JWindow;
+   do
      {
-	buf = find_non_visible_buffer (kill_buf);
-	
-	/* search through windows looking for the buffer and replace it */
-	w = JWindow;
-	do
+	if (kill_buf == JWindow->buffer)
 	  {
-	     if (kill_buf == JWindow->buffer)
-	       {
-		  touch_window_hard (JWindow, 0);
-		  window_buffer(buf);  /* I should choose a different one
-					every time */
-	       }
-	     JWindow = JWindow->next;
+	     touch_window_hard (JWindow, 0);
+	     window_buffer(buf); 
+	     buf = find_non_visible_buffer (kill_buf);
 	  }
-	while (w != JWindow);
+	JWindow = JWindow->next;
      }
-   else buf = scratch;
+   while (w != JWindow);
+
    if (kill_buf == Last_Buffer) Last_Buffer = NULL;
    if (kill_buf == this_buf) this_buf = buf;
    switch_to_buffer(this_buf);
    delete_buffer(kill_buf);
-   return(1);
+   return 1;
 }
 
 /*}}}*/
@@ -349,7 +316,6 @@ int write_buffer_cmd(char *filestr) /*{{{*/
    char *ff;
 #endif
    char dir[JED_MAX_PATH_LEN], *f, file[JED_MAX_PATH_LEN];
-   char msgstr[JED_MAX_PATH_LEN + 80];
    int n;
    
 #ifdef REAL_UNIX_SYSTEM
@@ -371,9 +337,19 @@ int write_buffer_cmd(char *filestr) /*{{{*/
    SLMEMCPY((char *) dir, (char *) file, n);
    dir[n] = 0;
    
-   if (*f == 0) f = CBuf->file;
-   
-   n = write_file_with_backup(dir, f);
+   if (*f == 0) 
+     {
+	safe_strcpy (file, CBuf->file, sizeof (file));
+	f = file;
+     }
+
+   if (file_changed_on_disk (CBuf, dir_file_merge (dir, f)))
+     {
+	if (0 == jed_get_yes_no("File changed on disk.  Save anyway?"))
+	  return 0;
+     }
+	  
+   n = write_file_with_backup (dir, f);
    if (n >= 0)
      {
 #ifdef VMS
@@ -381,10 +357,9 @@ int write_buffer_cmd(char *filestr) /*{{{*/
 #endif
 	if (Batch != 2)
 	  {
-	     sprintf(msgstr,"Wrote %d lines to %s%s", n, dir, f);
-	     message(msgstr);
+	     jed_vmessage (0, "Wrote %d lines to %s%s", n, dir, f);
 	  }
-	CBuf->flags &= ~BUFFER_TRASHED;
+	CBuf->flags &= ~BUFFER_MODIFIED;
 	CBuf->flags |= AUTO_SAVE_BUFFER;
 	CBuf->hits = 0;
 #ifdef UNDO_HAS_REDO
@@ -395,19 +370,17 @@ int write_buffer_cmd(char *filestr) /*{{{*/
      }
    else
      {
-	sprintf(msgstr,"Error writing file %s%s", dir, f);
-	msg_error(msgstr);
-	return(0);
+	jed_verror ("Error writing file %s%s", dir, f);
+	return 0;
      }
 }
 
 /*}}}*/
 
-#if defined(msdos) && !defined(__WATCOMC__) && !defined(MSWINDOWS)
+#if defined(__BORLANDC__) && !defined(MSWINDOWS)
 int show_memory() /*{{{*/
 {
    struct farheapinfo hi;
-   char buf[132];
    char *c;
    unsigned long total = 0, core, used = 0;
    unsigned long max = 0;
@@ -427,10 +400,9 @@ int show_memory() /*{{{*/
 	  }
      }
    core = farcoreleft();
-   sprintf(buf, "used:%lu, core:%lu, free:%lu, grand:%lu, max:%lu, frag:%d (%s)",
-	   used, core, total, core + total, max, i, c);
-   message(buf);
-   return (0);
+   jed_vmessage (0, "used:%lu, core:%lu, free:%lu, grand:%lu, max:%lu, frag:%d (%s)",
+		 used, core, total, core + total, max, i, c);
+   return 0;
 }
 
 /*}}}*/
@@ -440,21 +412,18 @@ int show_memory() /*{{{*/
 #ifdef MSWINDOWS
 int show_memory (void) /*{{{*/
 {
-   char buf[132];
 #ifndef __WIN32__
    MEMMANINFO mmi;
    if (MemManInfo(&mmi))
      {
- 	sprintf(buf, "tot pages: %lu, free pages: %lu",
- 		mmi.dwTotalLinearSpace, mmi.dwFreeLinearSpace);
- 	message(buf);
+ 	jed_vmessage (0, "tot pages: %lu, free pages: %lu",
+		      mmi.dwTotalLinearSpace, mmi.dwFreeLinearSpace);
      }
 #else
    MEMORYSTATUS mst;
    GlobalMemoryStatus(&mst);
-   sprintf(buf, "avail space: %lu, tot phys space: %lu, avail phys space: %lu",
- 	   mst.dwAvailPhys + mst.dwAvailPageFile, mst.dwTotalPhys, mst.dwAvailPhys);
-   message(buf);
+   jed_vmessage (0, "avail space: %lu, tot phys space: %lu, avail phys space: %lu",
+		 mst.dwAvailPhys + mst.dwAvailPageFile, mst.dwTotalPhys, mst.dwAvailPhys);
 #endif
    
    return 0;
@@ -468,13 +437,9 @@ int show_memory (void) /*{{{*/
 #include <malloc.h>
 int show_memory() /*{{{*/
 {
-   char buf[132];
-   
-   sprintf(buf, "avail mem: %lu, tot phys pgs: ???, free lin space: %lu",
-	   _memavl (), _memmax ());
-   
-   message(buf);
-   return (0);
+   jed_vmessage (0, "avail mem: %lu, tot phys pgs: ???, free lin space: %lu",
+		 _memavl (), _memmax ());
+   return 0;
 }
 
 /*}}}*/
@@ -486,21 +451,18 @@ int show_memory() /*{{{*/
 
 int show_memory() /*{{{*/
 {
-   char buf[132];
    unsigned long mem;
    _go32_dpmi_meminfo info;
    
    _go32_dpmi_get_free_memory_information(&info);
    
-   if (info.available_physical_pages != -1)
-     mem = info.available_physical_pages * 4096;
+   if ((long)info.available_physical_pages != -1L)
+     mem = info.available_physical_pages * 4096UL;
    else mem = info.available_memory;
-   
-   sprintf(buf, "avail mem: %lu, tot phys pgs: %lu, free lin space: %lu",
-	   mem, info.total_physical_pages, info.free_linear_space);
-   
-   message(buf);
-   return (0);
+
+   jed_vmessage (0, "avail mem: %lu, tot phys pgs: %lu, free lin space: %lu",
+		 mem, info.total_physical_pages, info.free_linear_space);
+   return 0;
 }
 
 /*}}}*/
@@ -520,44 +482,114 @@ int set_buffer(char *name) /*{{{*/
     /* Last_Buffer = CBuf; */
    
    if (NULL == (buf = find_buffer(name)))
-     {
-	buf = make_buffer();
-	strcpy(buf->dir, CBuf->dir);
-	*buf->file = 0;
-	switch_to_buffer(buf);
-	uniquely_name_buffer(name);
-     }
-   else switch_to_buffer(buf);
-   return(1);
+     buf = make_buffer (name, CBuf->dir, NULL);
+   switch_to_buffer(buf);
+   
+   return 1;
 }
 
 /*}}}*/
 
-int get_yes_no(char *question) /*{{{*/
+int jed_get_mini_response (char *s)
 {
-   char buf[JED_MAX_PATH_LEN], *tmp;
+   int ch;
+   MiniBuf_Get_Response_String = s;
+   update (NULL, 0, 0, 0);
+   ch = jed_getkey ();
+   MiniBuf_Get_Response_String = NULL;
+   return ch;
+}
+
+static char *strcat_malloc (char *a, char *b)
+{
+   unsigned int len;
+   char *c;
+
+   len = strlen (a);
+
+   if (NULL == (c = SLmalloc (len + strlen (b) + 1)))
+     return NULL;
+
+   strcpy (c, a);
+   strcpy (c + len, b);
+   return c;
+}
+
+
+int jed_get_y_n (char *question)
+{
+   int ans;
+   char *yn_quest;
+
+   if (NULL == (yn_quest = strcat_malloc (question, "? (y/n)")))
+     return -1;
+
+   ans = jed_get_mini_response (yn_quest);
+   SLfree (yn_quest);
+
+   if ((ans == 'y') || (ans == 'Y'))
+     return 1;
+   if ((ans == 'n') || (ans == 'N'))
+     return 0;
+   if (SLKeyBoard_Quit) 
+     return -1;
+   jed_beep ();
+   flush_input ();
+   return jed_get_yes_no (question);
+}
+
+
+int jed_vget_y_n (char *fmt, char *arg)
+{
+   char msg [1024];
+
+   if (arg == NULL) 
+     {
+	arg = fmt;
+	fmt = "%s";
+     }
+   /* FIXME: Priority=low: This is a hack to prevent a possible overflow */
+   if (strlen (arg) + strlen (fmt) >= sizeof (msg))
+     arg = "";
+
+   sprintf (msg, fmt, arg);
+   
+   return jed_get_y_n (msg);
+}
+
+
+int jed_get_yes_no (char *question) /*{{{*/
+{
+   char *yn_quest;
+   char *tmp;
    int n;
    
-   strncpy(buf, question, 240); buf[240] = 0;
-   strcat(buf, "? (yes or no)");
-   
-   while(1)
+   if (NULL == (yn_quest = strcat_malloc (question, "? (yes or no)")))
+     return -1;
+
+   while (1)
      {
-	if (NULL == (tmp = read_from_minibuffer(buf, 0, NULL, &n))) return(-1);
-	
+	if (NULL == (tmp = read_from_minibuffer(yn_quest, 0, NULL, &n))) 
+	  {
+	     SLfree (yn_quest);
+	     return -1;
+	  }
+
 	if (!strcmp(tmp, "yes"))
 	  {
-	     SLFREE(tmp);
-	     return(1);
+	     SLfree(tmp);
+	     SLfree (yn_quest);
+	     return 1;
 	  }
 	
 	if (!strcmp(tmp, "no"))
 	  {
-	     SLFREE(tmp);
-	     return(0);
+	     SLfree(tmp);
+	     SLfree (yn_quest);
+	     return 0;
 	  }
-	msg_error("yes or no!");
-	SLFREE(tmp);
+	msg_error("Answer `yes' or `no'.");
+	SLfree(tmp);
      }
 }
 
@@ -566,7 +598,6 @@ int get_yes_no(char *question) /*{{{*/
 int find_file_cmd (char *filestr)       /* filestr is const ! */ /*{{{*/
 {
    char *f, *file, filebuf[JED_MAX_PATH_LEN];
-   char msgstr[132];
    Buffer *buf;
    int n;
    
@@ -591,13 +622,13 @@ int find_file_cmd (char *filestr)       /* filestr is const ! */ /*{{{*/
    
    if (NULL != (buf = find_file_buffer(file)))
      {
-	if (file_changed_on_disk(file))
+	if (file_changed_on_disk (buf, file))
 	  {
-	     if (get_yes_no("File changed on disk.  Read From Disk"))
+	     if (jed_get_yes_no("File changed on disk.  Read From Disk"))
 	       {
 		  if (*Error_Buffer) return(1);
 		  n = (int) buf->linenum;
-		  buf->flags  &= ~BUFFER_TRASHED;
+		  buf->flags  &= ~BUFFER_MODIFIED;
 		  kill_buffer_cmd(buf->name);
 		  find_file_cmd(file);
 		  goto_line(&n);
@@ -609,9 +640,8 @@ int find_file_cmd (char *filestr)       /* filestr is const ! */ /*{{{*/
 	return(1);
      }
    
-   buf = make_buffer();
+   buf = make_buffer (NULL, file, NULL);
    switch_to_buffer(buf);
-   buffer_filename(file, NULL);
    
    n = read_file(file);
    CLine = CBuf->beg;
@@ -630,12 +660,11 @@ int find_file_cmd (char *filestr)       /* filestr is const ! */ /*{{{*/
 	if (n == -1) message("New file.");
 	else
 	  {
-	     sprintf(msgstr,"%d lines read", n);
-	     message(msgstr);
+	     jed_vmessage (0, "%d lines read", n);
 	  }
      }
    
-   SLang_run_hooks("find_file_hook", file, NULL);
+   SLang_run_hooks("find_file_hook", 1, file);
    return(1);
 }
 
@@ -654,17 +683,6 @@ int find_file_in_window(char *file) /*{{{*/
 
 /*}}}*/
 
-void set_mode_cmd(char *mode, int *flags) /*{{{*/
-{
-   char *m = CBuf->mode_str;
-   CBuf->modes = *flags;
-   strncpy((char *) m, (char *) mode, 12);
-   m[12] = 0;
-}
-
-/*}}}*/
-
-
 
 /* create a minibuffer with window and switch to it. */
 static void create_minibuffer(void) /*{{{*/
@@ -706,7 +724,7 @@ int exit_minibuffer() /*{{{*/
 	if (Last_Completion_Buffer != NULL)
 	  {
 	     pop_to_buffer (Completion_Buffer);
-	     CBuf->flags &= ~BUFFER_TRASHED;
+	     CBuf->flags &= ~BUFFER_MODIFIED;
 	     switch_to_buffer_cmd (Last_Completion_Buffer);
 	     kill_buffer_cmd (Completion_Buffer);
 	     touch_window_hard (JWindow, 0);
@@ -730,7 +748,7 @@ int select_minibuffer() /*{{{*/
    w = JWindow;
    while (MiniBuffer != NULL)
      {
-	if (JWindow->top == *tt_Screen_Rows) return(1);
+	if (JWindow->top == Jed_Num_Screen_Rows) return(1);
 	other_window();
 	if (w == JWindow) exit_error("Internal Error:  no window!", 1);
      }
@@ -756,7 +774,7 @@ static int ledit(void) /*{{{*/
    Suspend_Screen_Update = 1;
    
    SLang_load_string(tmp);
-   SLFREE(tmp);
+   SLfree(tmp);
    
    if ((SLang_Error == -1) && SLKeyBoard_Quit)
      {
@@ -777,7 +795,9 @@ static char *read_file_from_minibuffer(char *prompt, char *def) /*{{{*/
    complete_open = sys_findfirst;
    complete_next = sys_findnext;
    
-   if (*CBuf->dir == 0) strcpy (CBuf->dir, get_cwd());
+   if (*CBuf->dir == 0)
+     buffer_filename (CBuf, NULL, CBuf->file);
+
    strcpy (buf, CBuf->dir);
    return read_from_minibuffer (prompt, def, buf, &n);
 }
@@ -839,8 +859,9 @@ void read_object_with_completion(char *prompt, char *dflt, char *stuff, int *typ
      {
 	complete_open = sys_findfirst;
 	complete_next = sys_findnext;
-	if (*CBuf->dir == 0) strcpy(CBuf->dir, get_cwd());
-	strcpy(buf, CBuf->dir);
+	if (*CBuf->dir == 0) 
+	  buffer_filename (CBuf, NULL, CBuf->file);
+	strcpy (buf, CBuf->dir);
      }
    else if (type == 'b')	       /* buffer */
      {
@@ -871,8 +892,8 @@ void read_object_with_completion(char *prompt, char *dflt, char *stuff, int *typ
 	if (type == 'f') SLang_push_string(expand_filename(tmp));
 	else SLang_push_string(tmp);
 	
-	SLFREE(tmp);
-	if (str != NULL) SLFREE (str);
+	SLfree(tmp);
+	if (str != NULL) SLfree (str);
      }
 }
 
@@ -886,7 +907,7 @@ int insert_file_cmd() /*{{{*/
      
      if (NULL == (filebuf = read_file_from_minibuffer("Insert file:", NULL))) return(0);
    file = expand_filename(filebuf);
-   SLFREE (filebuf);
+   SLfree (filebuf);
    f = extract_file(file);
    if ((*file == 0) || (*f == 0))
      {
@@ -909,7 +930,7 @@ int find_file (void) /*{{{*/
      return 0;
    
    safe_strcpy (file, tmp, sizeof (file));
-   SLFREE(tmp);
+   SLfree(tmp);
    
    f = extract_file (file);
    if (*f == 0)
@@ -931,7 +952,7 @@ int write_buffer (void) /*{{{*/
    
    if (NULL == (tmp = read_file_from_minibuffer("Write to file:", (char *) NULL))) return(0);
    write_buffer_cmd(tmp);
-   SLFREE(tmp);
+   SLfree(tmp);
    return(1);
 }
 
@@ -971,7 +992,7 @@ int get_buffer() /*{{{*/
    
    if (NULL == (tmp = read_from_minibuffer("Switch to buffer:", Last_Buffer->name, NULL, &n))) return(0);
    switch_to_buffer_cmd(tmp);
-   SLFREE(tmp);
+   SLfree(tmp);
    return(1);
 }
 
@@ -991,12 +1012,12 @@ int kill_buffer (void) /*{{{*/
 	Buffer *b = find_buffer(tmp);
 	if ((b != NULL) && (b->subprocess))
 	  {
-	     if (0 == get_yes_no("Buffer has a subprocess attached.  Delete anyway"))
+	     if (0 == jed_get_yes_no("Buffer has a subprocess attached.  Delete anyway"))
 	       return 0;
 	  }
 #endif
 	kill_buffer_cmd(tmp);
-	SLFREE(tmp);
+	SLfree(tmp);
 	return(1);
      }
    return 0;
@@ -1107,7 +1128,7 @@ char *pop_to_buffer(char *name) /*{{{*/
    /* find a window to use */
    do
      {
-	if (w->top != *tt_Screen_Rows)
+	if (w->top != Jed_Num_Screen_Rows)
 	  {
 	     if (action != NULL)
 	       {
@@ -1184,21 +1205,28 @@ int num_windows() /*{{{*/
 void flush_message(char *m) /*{{{*/
 {
    message(m);
-   if ((JScreen[0].old == NULL) || Batch) return;
+   if (Batch) return;
    do_dialog(Message_Buffer);
-   tt_goto_rc(*tt_Screen_Rows - 1, strlen(Message_Buffer));
+   SLsmg_gotorc (Jed_Num_Screen_Rows - 1, strlen(Message_Buffer));
    *Message_Buffer = 0;
    JWindow->trashed = 1;
-   flush_output ();
+   SLsmg_refresh ();
 }
 
 /*}}}*/
 
-#if defined (REAL_UNIX_SYSTEM) || defined (__os2__)
+#if defined (REAL_UNIX_SYSTEM) || defined(__WIN32__) || (defined (__os2__) && !defined(__WATCOMC__))
 
-# if defined (__BORLANDC__) || defined (_MSC_VER)
-#   define popen _popen
-#   define pclose _pclose
+# if defined (__WIN32__) /* defined (__BORLANDC__) || defined (_MSC_VER) */
+#  undef popen
+#  undef pclose
+#  define popen w32_popen
+#  define pclose w32_pclose
+# endif
+
+# ifdef	__IBMC__
+extern FILE *popen(char *, char *);
+extern int pclose(FILE *);
 # endif
 
 # if !JED_HAS_SUBPROCESSES
@@ -1225,11 +1253,11 @@ int shell_command(char *cmd) /*{{{*/
 	return -1;
      }
    
-   if (NULL != (vp = vstream(fileno(pp), 0, VFILE_BINARY)))
+   if (NULL != (vp = vstream(fileno(pp), 0, VFILE_TEXT)))
      {
 	(void) insert_file_pointer(vp);
-	if (vp->buf != NULL) SLFREE(vp->buf);
-	SLFREE(vp);
+	SLfree(vp->buf);
+	SLfree ((char *)vp);
      }
    else	msg_error("Malloc Error.");
    
@@ -1272,44 +1300,59 @@ int pipe_region(char *cmd) /*{{{*/
 /*}}}*/
 #endif
 
-/* called by SLang_load_file
+/* 
  * Try to open a .slc then a .sl
  */
-#ifdef msdos
+#ifdef SIXTEEN_BIT_SYSTEM
 #define VFILE_BUF_SIZE 1024
 #else
 #define VFILE_BUF_SIZE 4096
 #endif
 
-static VFILE *jed_open_lib_file(char *file) /*{{{*/
+
+static VFILE *jed_open_lib_file (char *file) /*{{{*/
 {
-   char *lib = Jed_Library, libfsl[JED_MAX_PATH_LEN], libfslc[JED_MAX_PATH_LEN], *libp, buff[JED_MAX_PATH_LEN],
-     *type, *libf;
-   int n, comma = ',';
-   VFILE *vp;
-#ifdef VMS
-   int vmsn;
+   char libfsl[JED_MAX_PATH_LEN], libfslc[JED_MAX_PATH_LEN];
+   char *lib, *type, *libf;
+   unsigned int n;
+   VFILE *vp = NULL;
+
+   libf = file;
+   lib = Jed_Library;
+
+   /* If file begins with ./,then read it from the current directory */
+   if ((*file == '.')
+       && ((file[1] == '/')
+#ifdef IBMPC_SYSTEM
+	   || (file[1] == '\\')
 #endif
-   
-   if (*file == '.') lib = "";
-   else if (*lib == 0)
-     {
-	exit_error("JED_ROOT environment variable needs set.", 0);
-     }
-   
+       ))
+     lib = "";
+   else if ((lib == NULL) || (*lib == 0))
+     exit_error("JED_ROOT environment variable needs set.", 0);
+
+   if ((NULL != (type = file_type(file)))
+       && (*type == 0))
+     type = NULL;
+
    n = 0;
-   type = file_type(file);
-   while (1)
+   while (0 == SLextract_list_element (lib, n, ',', libfsl, sizeof(libfsl)))
      {
-	libp = SLang_extract_list_element(lib, &n, &comma);
 	n++;
-	safe_strcpy(libfsl, libp, sizeof (libfsl));
+
 	fixup_dir(libfsl);
-	safe_strcat(libfsl, file, sizeof (libfsl));
-	strcpy(libfsl, expand_filename(libfsl));
-	if ((type == NULL) || (*type == 0))
+	safe_strcat (libfsl, file, sizeof (libfsl));
+	strcpy (libfsl, expand_filename(libfsl));
+
+	libf = libfsl;
+
+	if (NULL != (vp = vopen(libf, VFILE_BUF_SIZE, VFILE_TEXT)))
+	  break;
+
+	if (type == NULL)
 	  {
 #ifdef VMS
+	     int vmsn;
 	     /* remove trailing '.' */
 	     if (0 != (vmsn = strlen(libfsl)))
 	       {
@@ -1318,149 +1361,150 @@ static VFILE *jed_open_lib_file(char *file) /*{{{*/
 	       }
 #endif
 	     
-	     strcat(libfsl, ".sl");
-	     strcpy(libfslc, libfsl);
-	     strcat(libfslc, "c");
-	     if (file_time_cmp(libfslc, libfsl) > 0) libf = libfslc; else libf = libfsl;
+	     /* Try .sl and .slc */
+	     strcat (libfsl, ".sl");
+	     strcpy (libfslc, libfsl);
+	     strcat (libfslc, "c");
+
+	     if (file_time_cmp(libfslc, libfsl) >= 0)
+	       libf = libfslc;
+
+	     if (NULL != (vp = vopen(libf, VFILE_BUF_SIZE, VFILE_TEXT)))
+	       break;
 	  }
-	else libf = libfsl;
-	vp = vopen(libf, VFILE_BUF_SIZE, VFILE_TEXT);
-	if ((vp != NULL) || (*libp == 0)) break;
      }
    
    if (Batch != 2)
-     {
-	sprintf(buff, "loading %s", libf);
-	flush_message(buff);
-     }
+     jed_vmessage (1, "loading %s", libf);
    
    return (vp);
 }
 
 /*}}}*/
 
-static int jed_close_object(SLang_Load_Type *x) /*{{{*/
-{
-   switch (x->type)
-     {
-      case 'F':  /* File */
-	
-	vclose((VFILE *) x->handle);
-	return (1);
-	
-      default: return SL_OBJ_UNKNOWN;
-     }
-}
-
-/*}}}*/
-
 static char *jed_read_from_file(SLang_Load_Type *x) /*{{{*/
 {
    char *s;
    unsigned int n;
-   
-   if ((s = vgets((VFILE *) x->handle, &n)) != NULL)
+
+   if ((s = vgets((VFILE *) x->client_data, &n)) != NULL)
      {
 	if (s[n - 1] != '\n') s[n] = 0;
      }
    
-   return (x->buf = s);
+   return s;
 }
 
 /*}}}*/
 
-static char *jed_read_from_buffer(SLang_Load_Type *x) /*{{{*/
+int jed_load_file (char *file)
 {
-   int n;
-   Line *l = (Line *) x->handle;
-   char *buf = x->buf;
-   
-   if (l == NULL) return (NULL);
-   
-   n = l->len;
-   if (n > 255) n = 255;
-   SLMEMCPY(buf, (char *) l->data, n);
-   buf[n] = 0;
-   x->handle = (long) l->next;
-   return(buf);
-}
+   VFILE *vp;
+   SLang_Load_Type *x;
+   int ret;
 
-/*}}}*/
+   if (NULL == (x = SLallocate_load_type (file)))
+     return -1;
 
-/* returns 0 if successful */
-static int jed_open_object(SLang_Load_Type *x) /*{{{*/
-{
-   
-   switch (x->type)
+   if (NULL == (vp = jed_open_lib_file (file)))
      {
-      case 'F':  /* File */
-	
-	x->read = jed_read_from_file;
-	
-	if (0 == (x->handle = (long) jed_open_lib_file((char *) x->name)))
-	  {
-	     return (SL_OBJ_NOPEN);
-	  }
-	
-	x->buf = (long) NULL;
-	x->n = 0;
-	return (0);
-	
-      case 'B': /* Buffer */
-	x->read = jed_read_from_buffer;
-	x->handle = (long) (((Buffer *) (x->name))->beg);
-	return (0);
-	
-      default: return SL_OBJ_UNKNOWN;
+	SLang_verror (SL_OBJ_NOPEN, "Unable to load %s", file);
+	SLdeallocate_load_type (x);
+	return -1;
      }
+   
+   x->client_data = (VOID_STAR) vp;
+   x->read = jed_read_from_file;
+   ret = SLang_load_object (x);
+   SLdeallocate_load_type (x);
+   vclose (vp);
+   return ret;
 }
 
+
+typedef struct
+{
+   Line *l;
+   char buf[256];
+}
+Buffer_Client_Type;
+
+static char *jed_read_from_buffer (SLang_Load_Type *x) /*{{{*/
+{
+   Buffer_Client_Type *b;
+   char *buf;
+   Line *l;
+   unsigned int len;
+
+   b = (Buffer_Client_Type *)x->client_data;
+
+   if (NULL == (l = b->l))
+     return NULL;
+
+   len = (unsigned int) l->len;
+   if (len > 255)
+     {
+	SLang_doerror ("Line len too long");
+	return NULL;
+     }
+
+   buf = b->buf;
+   SLMEMCPY(buf, (char *) l->data, len);
+   buf [len] = 0;
+   b->l = l->next;
+
+   return buf;
+}
 /*}}}*/
 
-void load_buffer() /*{{{*/
+void load_buffer (void) /*{{{*/
 {
-   SLang_Load_Type x;
+   SLang_Load_Type *x;
+   Buffer_Client_Type client_data;
    Buffer *cbuf = CBuf;
    int flags = CBuf->flags;
-   char buf[JED_MAX_PATH_LEN];
    Line *l, *lwant;
-   
+
+   /* FIXME!!! This should be changed to use the filename if present.  
+    * Otherwise, static objects will not get put in the proper table.
+    */
+   if (NULL == (x = SLallocate_load_type (cbuf->name)))
+     return;
+
+   x->read = jed_read_from_buffer;
+   x->client_data = (VOID_STAR) &client_data;
+   client_data.l = CBuf->beg;
+
    cbuf->flags |= READ_ONLY;
+   SLang_load_object(x);
+   SLdeallocate_load_type (x);
    
-   x.name = (long) cbuf;
-   x.type = 'B';
-   x.buf = buf;
+   if (buffer_exists (cbuf))
+     cbuf->flags = flags;
+   else cbuf = NULL;
+
+   if ((SLang_Error == 0)
+       && (*Error_Buffer == 0))
+     return;
    
-   SLang_load_object(&x);
-   
-   if (SLang_Error || *Error_Buffer)
+   SLang_doerror(NULL);
+   if (cbuf == NULL)
+     return;
+
+   pop_to_buffer (cbuf->name);
+   lwant = client_data.l;
+
+   if (lwant != NULL)
      {
-	SLang_doerror(NULL);
-	pop_to_buffer(cbuf->name);
-	/* if (x.ptr != NULL)
-	   { */
-	lwant = (Line *) x.handle;
-	if (lwant != NULL)
+	bob();
+	while (1)
 	  {
-	     bob();
-	     while (1)
-	       {
-		  l = CLine->next;
-		  if ((l == NULL) || (l == lwant)) break;
-		  CLine = l; LineNum++;
-	       }
+	     l = CLine->next;
+	     if ((l == NULL) || (l == lwant)) break;
+	     CLine = l; LineNum++;
 	  }
-	else eob();
-	Point = x.ofs;
-	/* Point = (int) (x.ptr - buf); */
-	if ((Point >= 0) && (Point < CLine->len))
-	  {
-	     (void) skip_whitespace();
-	  }
-	else Point = 0;
-     /* } */
+	(void) skip_whitespace();
      }
-   cbuf->flags = flags;
 }
 
 /*}}}*/
@@ -1515,11 +1559,13 @@ int mini_complete (void) /*{{{*/
    if (NULL == (tmp = make_buffer_substring(&n))) return(1);
    
    safe_strcpy(buf, tmp, sizeof (buf));
-   SLFREE(tmp);
+   SLfree(tmp);
    
    if ((last_key_char == ' ') && ((long) Last_Key_Function == (long) mini_complete))
      {
-	if (!flag || !(flag = (*complete_next)(buf)))
+	if (flag)
+	  flag = (*complete_next)(buf);
+	if (flag == 0)
 	  {
 	     safe_strcpy(buf, prev, sizeof (buf));
 	     flag = (*complete_open)(buf);
@@ -1553,7 +1599,7 @@ int mini_complete (void) /*{{{*/
 		    }
 	       }
 	     
-	     if (SLang_Error || SLpop_string (&tmp))
+	     if (SLang_Error || SLang_pop_slstring (&tmp))
 	       {
 		  msg_error ("Error encounter during expansion.  Disabling expansion hook.");
 		  Expand_File_Hook = NULL;
@@ -1561,7 +1607,8 @@ int mini_complete (void) /*{{{*/
 	       }
 	     safe_strcpy (last, tmp, sizeof (last));
 	     strcpy (prev, last);
-	     SLFREE (tmp);
+	     SLang_free_slstring (tmp);
+	     flag = 0;		       /* So we do not call complete_next */
 	     n = -1;
 	  }
      }
@@ -1587,12 +1634,23 @@ int mini_complete (void) /*{{{*/
 	     n++;
 	     pl = last;
 	     pb = buf;
-#ifdef __os2__
+
+#if !JED_FILE_PRESERVE_CASE
  	     if (complete_open == open_bufflist)
-	       while (*pl && (tolower(*pl) == tolower(*pb))) pl++, pb++;
+	       while (*pl && (UPPER_CASE(*pl) == UPPER_CASE(*pb))) 
+		 {
+		    pl++;
+		    pb++;
+		 }
  	     else  /* next statement */
 #endif
-	       while (*pl && (*pl == *pb)) pl++, pb++;
+	       while (*pl && (*pl == *pb)) 
+	       {
+		  pl++;
+		  pb++;
+	       }
+	     
+
 	     *pl = 0;
 	     
 	     if (last_key_char == '\t')
@@ -1602,14 +1660,13 @@ int mini_complete (void) /*{{{*/
 		  newline ();
 	       }
 	  }
-	
 	while(0 != (flag = (*complete_next)(buf)));
 	
-#ifdef __os2__
+#if JED_FILE_PRESERVE_CASE
 	/* OS/2 uses case-insensitive search on buffer-names. Set the 
 	 * flag if there is an exact match, so that completion will
-	 * cycle without repeats through all the matches. */
-	
+	 * cycle without repeats through all the matches. 
+	 */
 	if (complete_open == open_bufflist) 
 	  {
 	     strcpy(buf, last);
@@ -1658,15 +1715,6 @@ int what_char() /*{{{*/
 
 /*}}}*/
 
-void update_cmd(int *force) /*{{{*/
-{
-   if (Batch) return;
-   JWindow->trashed = 1;
-   update((Line *) NULL, *force, 0);
-}
-
-/*}}}*/
-
 void call_cmd(char *str) /*{{{*/
 {
    SLKeyMap_List_Type *km;
@@ -1695,27 +1743,28 @@ void copy_region_cmd(char *name) /*{{{*/
 
 /*}}}*/
 
-#ifndef pc_system
-void screen_w80() /*{{{*/
+#ifndef IBMPC_SYSTEM
+
+void screen_w80 (void) /*{{{*/
 {
-   tt_narrow_width();
-   change_screen_size(80, *tt_Screen_Rows);
+   tt_narrow_width ();
+   jed_resize_display ();
 }
 
 /*}}}*/
-void screen_w132() /*{{{*/
+void screen_w132 (void) /*{{{*/
 {
    tt_wide_width();
-   change_screen_size(132, *tt_Screen_Rows);
+   jed_resize_display ();
 }
 
 /*}}}*/
 #endif
 
-char *make_line_string(char *string) /*{{{*/
+char *make_line_string(char *string, unsigned int buflen) /*{{{*/
 {
    unsigned char *tmp, *p1, *p2;
-   int n;
+   unsigned int n;
    
    if (CBuf->marks == NULL)
      {
@@ -1732,8 +1781,8 @@ char *make_line_string(char *string) /*{{{*/
 	  }
 	pop_mark(&Number_Zero);
      }
-   n = (int) (p2 - p1);
-   if (n > 254) n = 254;
+   n = (unsigned int) (p2 - p1);
+   if (n >= buflen) n = buflen - 1;
    SLMEMCPY(string, (char *) p1, n);
    string[n] = 0;
    return(string);
@@ -1763,9 +1812,8 @@ char *make_buffer_substring(int *np) /*{{{*/
    n -= thisp;
    n += Point;
    
-   if (NULL == (buf = (unsigned char *) SLMALLOC(n + 1)))
+   if (NULL == (buf = (unsigned char *) SLmalloc (n + 1)))
      {
-	msg_error("Malloc Error");
 	pop_spot();
 	return (NULL);
      }
@@ -1851,16 +1899,26 @@ void get_buffer_info(void) /*{{{*/
 
 void set_buffer_info(char *file, char *dir, char *name, int *flags) /*{{{*/
 {
-   safe_strcpy(CBuf->file, file, sizeof (CBuf->file));
+   char dirbuf [JED_MAX_PATH_LEN + 1];
    
-   /* I have to handle this in this way because expand_filename does not modify
-    * its argument.
-    */
-   safe_strcpy(CBuf->dir, dir, sizeof (CBuf->dir));
-   fixup_dir(CBuf->dir);
-   strcpy (CBuf->dir, expand_filename (CBuf->dir));
+   safe_strcpy (dirbuf, dir, sizeof (dirbuf) - 2);
+   fixup_dir (dirbuf);
+   dir = SLang_create_slstring (expand_filename (dirbuf));
+   file = SLang_create_slstring (file);
+   name = SLang_create_slstring (name);
    
-   safe_strcpy(CBuf->name, name, sizeof (CBuf->name));
+   if (SLang_Error)
+     {
+	/* SLang_free_slstring ignores NULL */
+	SLang_free_slstring (dir);
+	SLang_free_slstring (file);
+	SLang_free_slstring (name);
+	return;
+     }
+   
+   CBuf->file = file;
+   CBuf->name = name;
+   CBuf->dir = dir;
    CBuf->flags = *flags;
 }
 
@@ -1894,20 +1952,13 @@ int window_size_intrinsic(int *what) /*{{{*/
       case 'c': n = JWindow->column; break;
       case 't': n = JWindow->top; break;
       case 'w': n = JWindow->width; break;
-      default: SLang_Error = UNKNOWN_ERROR;
+      default: SLang_Error = SL_UNKNOWN_ERROR;
      }
    return (n);
 }
 
 /*}}}*/
 
-int what_mode(void) /*{{{*/
-{
-   SLang_push_string (CBuf->mode_str);
-   return CBuf->modes;
-}
-
-/*}}}*/
 
 /* Given a file name with wildcards return expanded list to S-Lang stack
  *  with number.  This does NOT work on unix with wild cards.  Instead the 
@@ -1982,114 +2033,6 @@ void exit_function(char *name) /*{{{*/
 #endif
 
 
-#if 0
-void make_istring(char *buf, int i) /*{{{*/
-{
-   register char *b, *b1, ch;
-   int sign, d;
-   
-   b = buf;
-   if (i >= 0)
-     {
-	sign = 1;
-     }
-   else
-     {
-	sign = -1;
-	i = -i;
-     }
-   do
-     {
-	d = i % 10;
-	i = i / 10;
-	*b++ = d + '0';
-     }
-   while (i > 0);
-   
-   if (sign == -1) *b++ = '-';
-   *b-- = 0;
-   b1 = buf;
-   /* now reverse it */
-   while (b > b1)
-     {
-	ch = *b;
-	*b-- = *b1;
-	*b1++ = ch;
-     }
-}
-
-/*}}}*/
-#include <stdarg.h>
-char *simple_sprintf(char *str, char *fmt, ...) /*{{{*/
-{
-   va_list ap;
-   register char ch, *s = str, *string;
-   char *fmtmax = fmt + strlen(fmt);
-   int dig;
-   char buf[80];
-   
-   va_start(ap, fmt);
-   
-   
-   while (fmt < fmtmax)
-     {
-	ch = *fmt++;
-	switch(ch)
-	  {
-	   case '%': ch = *fmt++;
-	     string = buf;
-	     switch(ch)
-	       {
-		case 'c': ch = (char) va_arg(ap, int);  /* drop */
-		case '%': buf[0] = ch; buf[1] = 0; break;
-		case 's':
-		  string = va_arg(ap, char *); break;
-		case 'l':
-		case 'd':
-		default:
-		  dig = va_arg(ap, int);
-		  make_istring(buf, dig);
-		  break;
-	       }
-	     while (*string) *s++ = *string++;
-	     break;
-	   case '\\':
-	     ch = *fmt++;
-	     switch(ch)
-	       {
-		case 'b': ch = '\b'; break;
-		case 'r': ch = 13; break;
-		case 'n': ch = '\n'; break;
-		case 'e': ch = 27; break;
-		case 'a': ch = 7; break;
-	       }
-	     /* drop */
-	   default:
-	     *s++ = ch;
-	  }
-     }
-   va_end(ap);
-   *s = 0;
-   return(str);
-}
-
-/*}}}*/
-#endif
-
-
-char *command_line_argv(int *nn) /*{{{*/
-{
-   int n = *nn;
-   if ((n >= Main_Argc) || (n < 0))
-     {
-	msg_error("Argc out of bounds.");
-	n = 0;
-     }
-   
-   return Main_Argv[n];
-}
-
-/*}}}*/
 
 void count_chars(void) /*{{{*/
 {
@@ -2127,7 +2070,69 @@ static void jed_clear_error(void) /*{{{*/
 
 /*}}}*/
 
-int (*X_Init_SLang_Hook)(void);
+typedef struct _Init_SLang_Hook_Type
+{
+   int (*hook)(void);
+   struct _Init_SLang_Hook_Type *next;
+}
+Init_SLang_Hook_Type;
+
+static Init_SLang_Hook_Type *Init_SLang_Hooks;
+
+int jed_add_init_slang_hook (int (*hook)(void))
+{
+   Init_SLang_Hook_Type *h;
+
+   if (hook == NULL)
+     return 0;
+   
+   h = (Init_SLang_Hook_Type *)SLmalloc (sizeof (Init_SLang_Hook_Type));
+   if (h == NULL)
+     return -1;
+   
+   h->hook = hook;
+   h->next = Init_SLang_Hooks;
+   Init_SLang_Hooks = h;
+   return 0;
+}
+
+static int run_init_slang_hooks (void)
+{
+   Init_SLang_Hook_Type *h;
+   
+   h = Init_SLang_Hooks;
+
+   while (h != NULL)
+     {
+	if (-1 == (*h->hook)())
+	  return -1;
+	h = h->next;
+     }
+
+   while (Init_SLang_Hooks != NULL)
+     {
+	h = Init_SLang_Hooks->next;
+	SLfree ((char *) Init_SLang_Hooks);
+	Init_SLang_Hooks = h;
+     }
+   
+   return 0;
+}
+
+static void slang_exit_error_handler (char *fmt, va_list ap)
+{
+   char buf [2048];
+   
+   vsprintf (buf, fmt, ap);
+   exit_error (buf, 0);
+}
+
+static void vmsg_hook (char *fmt, va_list ap)
+{
+   char buf [2048];
+   vsprintf (buf, fmt, ap);
+   message (buf);
+}
 
 void init_minibuffer() /*{{{*/
 {
@@ -2135,18 +2140,18 @@ void init_minibuffer() /*{{{*/
    
    tmp = CBuf;
    
-   The_MiniBuffer = make_buffer();
+   The_MiniBuffer = make_buffer (" <mini>", NULL, NULL);
    The_MiniBuffer->modes = 0;
-   strcpy(The_MiniBuffer->name, " <mini>");
    /* do some initializing */
    switch_to_buffer(The_MiniBuffer);
    remake_line(132);
-   The_MiniWindow = create_window(*tt_Screen_Rows, 1, 1, *tt_Screen_Cols);
+   The_MiniWindow = create_window(Jed_Num_Screen_Rows, 1, 1, Jed_Num_Screen_Cols);
    The_MiniWindow->buffer = CBuf;
    Buffer_Local.tab = 0;
    switch_to_buffer(tmp);
    SLang_Dump_Routine = jed_traceback;
-   
+   SLang_Exit_Error_Hook = slang_exit_error_handler;
+
 #ifdef __GO32__
    SLang_Interrupt = i386_check_kbd;
 #endif
@@ -2155,27 +2160,28 @@ void init_minibuffer() /*{{{*/
    SLang_Enter_Function = enter_function;
    SLang_Exit_Function = exit_function;
 #endif
-   if (!init_SLang()
-#ifdef FLOAT_TYPE
-       || !init_SLmath()
+   if ((-1 == SLang_init_slang ())
+#ifndef SIXTEEN_BIT_SYSTEM
+       || (-1 == SLang_init_slmath ())
 #endif
-#ifndef pc_system
-#ifdef __unix__
-       || !init_SLunix()
+       || (-1 == SLang_init_posix_process ())
+       || (-1 == SLang_init_posix_dir ())
+       || (-1 == SLang_init_slassoc ())
+       || (-1 == init_jed_intrinsics ())
+#if JED_HAS_TTY_MENUS
+       || (-1 == jed_init_menus ())
 #endif
-#endif
-       || !init_jed_intrinsics()
-       || !register_jed_classes ()
-       || ((X_Init_SLang_Hook != NULL) && !(*X_Init_SLang_Hook)()))
+       || (-1 == register_jed_classes ())
+       || (-1 == run_init_slang_hooks ()))
      {
 	exit_error("Unable to initialize S-Lang!", 0);
      }
    
    /* use jed rouotines instead of default slang ones */
-   SLang_Error_Routine = msg_error;
-   SLang_User_Open_Slang_Object = jed_open_object;
-   SLang_User_Close_Slang_Object = jed_close_object;
+   SLang_Error_Hook = msg_error;
+   SLang_VMessage_Hook = vmsg_hook;
    SLang_User_Clear_Error = jed_clear_error;
+   SLang_Load_File_Hook = jed_load_file;
 }
 
 /*}}}*/
