@@ -1,10 +1,21 @@
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
+ */
 #include "config.h"
 #include "jed-feat.h"
 
-#if !defined (_MSC_VER) && !defined (__EMX__)
+#if !defined (_MSC_VER) && !defined (__EMX__) && !defined(__WATCOMC__) && !defined(__IBMC__)
 # include <dir.h>
 #endif
 
+#ifdef	__IBMC__
+# include <ctype.h>
+# include <direct.h>
+#endif
+
 #define INCL_BASE
 #define INCL_NOPM
 #define INCL_VIO
@@ -18,7 +29,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <process.h>
-#include <dos.h>
+#ifndef __IBMC__
+# include <dos.h>
+#endif
 #include <errno.h>
 
 #include "sysdep.h"
@@ -26,6 +39,9 @@
 #include "buffer.h"
 #include "misc.h"
 #include "hooks.h"
+#if JED_HAS_SUBPROCESSES
+#include "jprocess.h"
+#endif
 
 int Abort_Char = 7;		       /* scan code for G (control) */
 
@@ -35,7 +51,7 @@ extern char *get_cwd (void);
 void delay (int time);
 
 void set_kbd (void);
-void os2_kbdhandler_thread (void);
+/* void os2_kbdhandler_thread (void); */
 
 #define lowercase strlwr
 
@@ -43,15 +59,14 @@ KBDINFO	initialKbdInfo;	/* keyboard info		*/
 
 static int atEnd = 0;
 
-
 #ifdef __os2_16__
 
 typedef USHORT APIRET;
 typedef HSEM OS2_SEM;
 
-#define DosRequestMutexSem(hmtx,timeout) DosSemRequest(hmtx,timeout)
-#define DosReleaseMutexSem(hmtx) DosSemClear(hmtx)
-#define DosCloseMutexSem(hmtx) DosCloseSem(hmtx)
+# define DosRequestMutexSem(hmtx,timeout) DosSemRequest(hmtx,timeout)
+# define DosReleaseMutexSem(hmtx) DosSemClear(hmtx)
+# define DosCloseMutexSem(hmtx) DosCloseSem(hmtx)
 
 #else /* !defined(__os2_16__) */
 
@@ -63,7 +78,6 @@ typedef HMTX OS2_SEM;
 
 static OS2_SEM Hmtx_Kbd ;     /* Mutex Semaphore */
 
-
 APIRET CreateSem(OS2_SEM *Hmtx)
 {
    APIRET result;
@@ -79,14 +93,11 @@ APIRET CreateSem(OS2_SEM *Hmtx)
    return result;
 }
 
-
 APIRET CloseSem (OS2_SEM Hmtx)
 {
    return( DosCloseMutexSem (Hmtx) );
 }
 
-
-
 /* define these using macros to improve efficiency */
 #define RequestSem(Hmtx) DosRequestMutexSem (Hmtx, -1);
 #define ReleaseSem(Hmtx) DosReleaseMutexSem (Hmtx);
@@ -107,11 +118,10 @@ if (!(n + Input_Buffer_Len > MAX_INPUT_BUFFER_LEN - 3)) { \
    DONE_WITH_INPUT_BUFFER \
 }
 
-
 #define THREADSTACKSIZE      32768
 static TID Os2_Kbdhandler_ThreadID = (TID) 0 ;
 
-/* in a separate thread, read keyboard and put the keys into JED's keyboard 
+/* in a separate thread, read keyboard and put the keys into JED's keyboard
  * buffer */
 
 static int enhancedKeyboard;
@@ -125,7 +135,7 @@ static void os2_thread_process_key (unsigned int c0, unsigned int c1,
    unsigned char keybuf[16];
    unsigned int scan;
    unsigned int num;
-   
+
    shift = shift & 0xF;
    scan = ((c1 << 8) | c0) & 0xFFFF;
 
@@ -133,32 +143,31 @@ static void os2_thread_process_key (unsigned int c0, unsigned int c1,
    os2_buffer_keystring (keybuf, num);
 }
 
-static void os2_kbdhandler_thread ()
+static void os2_kbdhandler_thread (void)
 {
    KBDKEYINFO keyInfo;
-   
-   
-   while (!atEnd) 
+
+   while (!atEnd)
      {
 	/* at end is a flag */
 	KbdCharIn(&keyInfo, IO_NOWAIT, 0);       /* get a character	*/
-	if (keyInfo.fbStatus & 0x040) 
+	if (keyInfo.fbStatus & 0x040)
 	  {
 	     /* found a char process it */
-	     
-	     if (keyInfo.chChar == Abort_Char) 
+
+	     if (keyInfo.chChar == Abort_Char)
 	       {
-		  SLang_Error = 2;
+		  if (Ignore_User_Abort == 0) SLang_Error = USER_BREAK;
 		  SLKeyBoard_Quit = 1;
 	       }
-	     
+
 	 /* now process them */
 	 /* read codes from buffer */
 	     os2_thread_process_key (keyInfo.chChar, keyInfo.chScan,
 				     keyInfo.fsState);
-	     
+
 	  }
-	else  
+	else
 	  {
 	     /* no char available*/
 	     set_kbd();      /* is this necessary?  */
@@ -167,7 +176,6 @@ static void os2_kbdhandler_thread ()
      }
 }
 
-
 static void os2_kbdthread (void *Args)
 {
    (void) Args;
@@ -176,13 +184,12 @@ static void os2_kbdthread (void *Args)
    _endthread();
 }
 
-
 /* The code below is in the main thread */
 
 void set_kbd()
 {
    KBDINFO kbdInfo;
-   
+
    kbdInfo = initialKbdInfo;
    kbdInfo.fsMask &= ~0x0001;		/* not echo on		*/
    kbdInfo.fsMask |= 0x0002;		/* echo off		*/
@@ -195,36 +202,35 @@ void set_kbd()
 int init_tty ()
 {
    VIOCURSORINFO cursorInfo, OldcursorInfo;
-   
+
    if (Batch) return 0;
 #ifdef HAS_MOUSE
    if (X_Open_Mouse_Hook != NULL) (*X_Open_Mouse_Hook) ();
 #endif
-   
+
   /*  set ^C off */
    signal (SIGINT, SIG_IGN);
    signal (SIGBREAK, SIG_IGN);
-   
+
    initialKbdInfo.cb = sizeof(initialKbdInfo);
    KbdGetStatus(&initialKbdInfo, 0);
    set_kbd();
    enhancedKeyboard = 1;
-   
-   
+
    CreateSem (&Hmtx_Kbd);
-   
+
   /* start a separate to read the keyboard */
 #if defined(__BORLANDC__)
    Os2_Kbdhandler_ThreadID = _beginthread (os2_kbdthread, THREADSTACKSIZE, NULL);
 #else
    Os2_Kbdhandler_ThreadID = _beginthread (os2_kbdthread, NULL,  THREADSTACKSIZE, NULL);
 #endif
-   
+
    if ((int)Os2_Kbdhandler_ThreadID == -1)
      {
 	exit_error ("init_tty: Error starting keyboard thread.", 0);
      }
-   
+
    VioGetCurType (&OldcursorInfo, 0);
    cursorInfo.yStart = 1;
    cursorInfo.cEnd = 15;
@@ -232,22 +238,23 @@ int init_tty ()
    cursorInfo.attr = 1;
    if (VioSetCurType (&cursorInfo, 0))
      VioSetCurType (&OldcursorInfo, 0);   /* reset to previous value */
-   
+
    return 0;
 }
 
-
 void reset_tty ()
 {
    if (Batch) return;
    atEnd = 1;                      /* set flag and wait until thread ends */
 #if __os2_16__
-   while (atEnd) {DosSleep (0);
-   }
+   while (atEnd)
+     {
+	DosSleep (0);
+     }
 #else
    DosWaitThread (&Os2_Kbdhandler_ThreadID, DCWW_WAIT);
 #endif
-   
+
   /* close the keyboard */
    KbdSetStatus(&initialKbdInfo, 0); /* restore original state	*/
 #ifdef HAS_MOUSE
@@ -257,41 +264,77 @@ void reset_tty ()
 }
 
 /* sleep for *tsecs tenths of a sec waiting for input */
-static int sys_input_pending(int *tsecs, int unused)
+int sys_input_pending(int *tsecs, int proc)
 {
    int count = *tsecs * 5;
-   (void) unused;
-   
+
+ 
+#if JED_HAS_SUBPROCESSES
+   if ((proc >= 0) && (Batch || Input_Buffer_Len)) return(Input_Buffer_Len);
+#else
    if (Batch || Input_Buffer_Len) return(Input_Buffer_Len);
-   
+#endif
+
    if (count)
      {
 	while(count > 0)
 	  {
 	     delay(20);		       /* 20 ms or 1/50 sec */
+#if JED_HAS_SUBPROCESSES
+	     if ((proc >= 0) && keyWaiting ()) break;
+	     if (proc)
+	       {
+		  int i = 0;
+		  while (i < Num_Subprocesses)
+		    {
+		       if (Subprocess_Read_fds[i][0])
+			 read_process_input (Subprocess_Read_fds[i][1]);
+		       i++;
+		    }
+	       }
+#else
 	     if (keyWaiting ()) break;
+#endif
 	     count--;
 	  }
 	return(count);
      }
-   else return(keyWaiting ());
-}
 
+#if JED_HAS_SUBPROCESSES
+   if (proc)
+     {
+	int i = 0;
+	while (i < Num_Subprocesses)
+	  {
+	     if (Subprocess_Read_fds[i][0])
+	       read_process_input (Subprocess_Read_fds[i][1]);
+	     i++;
+	  }
+     }
+   if (proc >= 0) return keyWaiting ();
+#else
+   return keyWaiting ();
+#endif
+}
 
 unsigned char sys_getkey ()
 {
    int weird = 300;
-   
-   
-   while (!sys_input_pending(&weird, 0))
+#if JED_HAS_SUBPROCESSES
+   int all = Num_Subprocesses;
+#else
+   int all = 0;
+#endif
+
+   while (!sys_input_pending(&weird, all))
      {
 	if (Display_Time)
 	  {
 	     JWindow->trashed = 1;
-	     update((Line *) NULL, 0, 1);
+	     update((Line *) NULL, 0, 1, 0);
 	  }
      }
-   
+
 /*   if (JMouse_Hide_Mouse_Hook != NULL) (*JMouse_Hide_Mouse_Hook) (0); */
    return my_getkey ();
 }
@@ -301,38 +344,38 @@ void sys_flush_input()
    KbdFlushBuffer(0);
 }
 
-
-
+#if 0
 int get_term_dimensions(int *w, int *h)
 {
    VIOMODEINFO vioModeInfo;
-   
+
    vioModeInfo.cb = sizeof(vioModeInfo);
    VioGetMode (&vioModeInfo, 0);
    *w = vioModeInfo.col;
    *h = vioModeInfo.row;
    return 0;
 }
+#endif
 
-#if defined(_MSC_VER) || defined (__EMX__)
+#if defined(_MSC_VER) || defined (__EMX__) || defined(__WATCOMC__)
 int sys_chmod(char *file, int what, int *mode, short *uid, short *gid)
 {
    struct stat buf;
    int m;
-   
-#ifdef _MSC_VER
+
+# ifdef _MSC_VER
    /* MSC stat() is broken on directory names ending with a slash */
    char path[_MAX_PATH];
    strcpy(path, file);
    deslash(file = path);
-#endif
-   
+# endif
+
    if (what)
      {
 	chmod(file, *mode);
 	return(0);
      }
-   
+
    if (stat(file, &buf) < 0) switch (errno)
      {
       case EACCES: return(-1); /* es = "Access denied."; break; */
@@ -340,11 +383,11 @@ int sys_chmod(char *file, int what, int *mode, short *uid, short *gid)
       case ENOTDIR: return(-2); /* es = "Invalid Path."; */
       default: return(-3); /* "stat: unknown error."; break;*/
      }
-   
+
    m = buf.st_mode;
-   
+
    *mode = m & 0777;
-   
+
    if (m & S_IFDIR) return (2);
    return(1);
 }
@@ -357,7 +400,7 @@ int sys_chmod(char *file, int what, int *mode, short *dum1, short *dum2)
    USHORT Result;
    char path[_MAX_PATH];
    (void) dum1; (void) dum2;
-   
+
  /* DosQueryPathInfo won't work with directory names ending with a slash */
    strcpy(path, file);
    deslash(file = path);
@@ -366,15 +409,15 @@ int sys_chmod(char *file, int what, int *mode, short *dum1, short *dum2)
    else if (Result == ERROR_ACCESS_DENIED) return -1;
    else if (Result == ERROR_INVALID_PATH) return -2;
    else if (Result != NO_ERROR) return -3;
-   
+
    *mode = fileInfo.attrFile;
-   
-   if (what) 
+
+   if (what)
      {
 	fileInfo.attrFile = *mode;
 	DosSetPathInfo (file, FIL_STANDARD, &fileInfo, sizeof (fileInfo), 0);
      }
-   
+
    if (*mode & 0x10) return(2);
    else return (1);
 }
@@ -382,9 +425,11 @@ int sys_chmod(char *file, int what, int *mode, short *dum1, short *dum2)
 #endif
 
 static char Found_Dir[JED_MAX_PATH_LEN];
+static char Found_File[JED_MAX_PATH_LEN];
+static int Found_FLen;
 
 #ifdef __os2_16__
-#define DosFindFirst(FileName, DirHandle, Attribute, ResultBuf, \
+# define DosFindFirst(FileName, DirHandle, Attribute, ResultBuf, \
 		     ResultBufLen, SearchCount, Reserved) \
 	DosFindFirst(FileName, DirHandle, Attribute, ResultBuf, \
 		     ResultBufLen, SearchCount, 0)
@@ -394,7 +439,7 @@ int sys_findfirst(char *theFile)
 {
    char *f, the_path[CCHMAXPATH], *file, *f1;
    char *pat;
-   
+
 #if defined(__os2_16__)
    FILEFINDBUF FindBuffer;
    USHORT FindCount;
@@ -404,18 +449,21 @@ int sys_findfirst(char *theFile)
    ULONG FindCount;
    ULONG File_Attr;
 #endif
-   
+
    HDIR FindHandle;
    File_Attr = FILE_READONLY | FILE_DIRECTORY;
-   
+
    file = expand_filename(theFile);
    f1 = f = extract_file(file);
    strcpy (Found_Dir, file);
-   
+
+   strcpy (Found_File, file);
+   Found_FLen = strlen(Found_File);
+
    Found_Dir[(int) (f - file)] = 0;
-   
+
    strcpy(the_path, file);
-   
+
    while (*f1 && (*f1 != '*')) f1++;
    if (! *f1)
      {
@@ -423,7 +471,7 @@ int sys_findfirst(char *theFile)
 	if (*f) strcat(the_path, "*"); else strcat(the_path, "*.*");
      }
    pat = the_path;
-   
+
    FindHandle = 1;
    FindCount = 1;
    if (DosFindFirst(pat, &FindHandle, File_Attr, &FindBuffer,
@@ -431,6 +479,16 @@ int sys_findfirst(char *theFile)
      return 0;
    strcpy (theFile, Found_Dir);
    strcat (theFile, FindBuffer.achName);
+
+   /* OS/2 HPFS has the same problem as Windows: the filesystem is
+    * case-insensitive so if one searches for makefi*, it will return
+    * Makefile.in between the others, so check for wrong values and reject
+    * them 
+    */
+   if ((strncmp(theFile, Found_File, Found_FLen) != 0) &&
+       (!sys_findnext(theFile)))
+     return 0;
+
    if (FindBuffer.attrFile & FILE_DIRECTORY) fixup_dir (theFile);
    return 1;
 }
@@ -444,41 +502,37 @@ int sys_findnext(char *file)
    FILEFINDBUF3 FindBuffer;
    ULONG FileCount;
 #endif
-   
-   FileCount = 1;
-   
-   if (DosFindNext(1, &FindBuffer, sizeof (FindBuffer), &FileCount))
-     return 0;
-   else 
+
+   while (1)
      {
-	strcpy (file, Found_Dir);
-	strcat (file, FindBuffer.achName);
-	if (FindBuffer.attrFile & FILE_DIRECTORY) fixup_dir (file);
-	return 1;
+	FileCount = 1;
+
+	if (DosFindNext(1, &FindBuffer, sizeof (FindBuffer), &FileCount))
+	  return 0;
+	else
+	  {
+	     strcpy (file, Found_Dir);
+	     strcat (file, FindBuffer.achName);
+
+	    /* OS/2 HPFS has the same problem as Windows: the filesystem is
+	     * case-insensitive so if one searches for makefi*, it will return
+	     * Makefile.in between the others, so check for wrong values and
+	     * reject them */
+	     if (strncmp(file, Found_File, Found_FLen) != 0)
+	      continue;
+	     if (FindBuffer.attrFile & FILE_DIRECTORY) fixup_dir (file);
+	     return 1;
+	  }
      }
 }
 
-
-
-/* Here we do a find first followed by calling routine to conver time */
-#if defined(_MSC_VER) || defined(__EMX__)
-
-unsigned long sys_file_mod_time(char *file)
-{
-   struct stat buf;
-   
-   if (stat(file, &buf) < 0) return(0);
-   return((unsigned long) buf.st_mtime);
-}
-
-#else
-
+#if 0
 unsigned long sys_file_mod_time(char *file)
 {
    FILESTATUS3 fileInfo;
    struct time t;
    struct date d;
-   
+
    if (DosQueryPathInfo (file, FIL_STANDARD, &fileInfo, sizeof (fileInfo))
        != NO_ERROR)
      return 0;
@@ -491,7 +545,6 @@ unsigned long sys_file_mod_time(char *file)
    d.da_year = fileInfo.fdateLastWrite.year;
    return dostounix(&d, &t);
 }
-
 #endif
 
 void delay (int time)
@@ -509,48 +562,48 @@ int IsHPFSFileSystem (char *directory)
 #if defined (__os2_16__)
    USHORT		cbData;
    USHORT		nDrive;
-#define DosQueryCurrentDisk DosQCurDisk
+# define DosQueryCurrentDisk DosQCurDisk
 #else
    ULONG		cbData;
    ULONG		nDrive;
    PFSQBUFFER2		pFSQ = (PFSQBUFFER2)bData;
 #endif
-   
+
+#ifndef __WATCOMC__
    if ( _osmode == DOS_MODE )
      return FALSE;
-   
+#endif
    if (isalpha (directory[0]) && (directory[1] == ':'))
      nDrive = toupper (directory[0]) - '@';
    else
      DosQueryCurrentDisk (&nDrive, &lMap);
-   
+
 /* Set up the drive name */
-   
+
    bName[0] = (char) (nDrive + '@');
    bName[1] = ':';
    bName[2] = 0;
-   
+
    cbData = sizeof (bData);
-   
+
 #if defined(__os2_16__)
    if (DosQFSAttach (bName, 0, FSAIL_QUERYNAME, bData, &cbData, 0L))
 #else
      if (DosQueryFSAttach (bName, 0, FSAIL_QUERYNAME, pFSQ, &cbData))
 #endif
        return FALSE;
-   
+
 #if defined(__os2_16__)
    FName = bData + (*((USHORT *) (bData + 2)) + 7);
 #else
    FName = pFSQ->szName + pFSQ->cbName + 1;
 #endif
-   
+
    if (strcmp(FName, "HPFS"))
      return FALSE;
    return(TRUE);
 }
 
-
 /* Code for EAs */
 
 #if defined(__os2_16__)
@@ -560,9 +613,9 @@ int IsHPFSFileSystem (char *directory)
 typedef USHORT APIRET;
 typedef struct _FEA2         /* fea2 */
 {
-#if 0			/* This field is in the 32 bit structure */
+# if 0			/* This field is in the 32 bit structure */
    ULONG oNextEntryOffset;		/* New field */
-#endif
+# endif
    BYTE fEA;
    BYTE cbName;
    USHORT cbValue;
@@ -582,10 +635,10 @@ typedef FEA2LIST *PFEA2LIST;
 
 typedef struct _GEA2			/* gea2 */
 {
-#if 0
+# if 0
 		/* New field - in the 32 bit structure */
    ULONG oNextEntryOffset;
-#endif /* OS2_32 */
+# endif /* OS2_32 */
    BYTE cbName;
    CHAR szName[ 1 ];		/* New field */
 }
@@ -612,14 +665,14 @@ EAOP2;
 
 /* typedef EAOP2 *PEAOP2; */
 
-#define DosSetPathInfo(PathName, PathInfoLevel, PathInfoBuf, \
+# define DosSetPathInfo(PathName, PathInfoLevel, PathInfoBuf, \
 		       PathInfoBufSize, PathInfoFlags) \
         DosSetPathInfo(PathName, PathInfoLevel, PathInfoBuf, \
 		       PathInfoBufSize, PathInfoFlags, 0)
-#define DosQueryPathInfo(PathName, PathInfoLevel, PathInfoBuf, \
+# define DosQueryPathInfo(PathName, PathInfoLevel, PathInfoBuf, \
 			 PathInfoBufSize) \
         DosQPathInfo(PathName, PathInfoLevel, PathInfoBuf, PathInfoBufSize, 0)
-#define DosEnumAttribute(RefType, FileRef, EntryNum, EnumBuf, \
+# define DosEnumAttribute(RefType, FileRef, EntryNum, EnumBuf, \
 			 EnumBufSize, EnumCnt, InfoLevel) \
         DosEnumAttribute(RefType, FileRef, EntryNum, EnumBuf, \
 			 EnumBufSize, EnumCnt, InfoLevel, 0)
@@ -654,7 +707,6 @@ typedef struct _HoldFEA HOLDFEA;
 #define SET_INFO_LEVEL1		1	/* Set info in SFT */
 #define SET_INFO_LEVEL2		2	/* Set FEAlist */
 
-
 /* #define BufferSize 1024
 static char FileBuffer [BufferSize]; */
 static HOLDFEA *pHoldFEA;			/* EA linked-list pointer */
@@ -666,7 +718,7 @@ void Free_FEAList( HOLDFEA *pFEA )
 {
    HOLDFEA *next;	/* Holds the next field since we free the structure
 			 before reading the current next field */
-   
+
 	/* Step through the list freeing all the fields */
    while( pFEA )
      {
@@ -678,7 +730,7 @@ void Free_FEAList( HOLDFEA *pFEA )
 	if( pFEA->aValue != NULL )
 			/* Free if non-NULL value */
 	  free(pFEA->aValue);
-	
+
 		/* Free the pFEA struct itself and move on to the next field */
 	free(pFEA);
 	pFEA = next;
@@ -691,7 +743,7 @@ void Free_FEAList( HOLDFEA *pFEA )
 ULONG getEAType( const CHAR *Value )
 {
    USHORT Type = *( USHORT * ) Value;
-   
+
    return( Type );
 }
 
@@ -745,25 +797,25 @@ void* getEADataVal (const HOLDFEA *pFEA)
 HOLDFEA *QueryEAs( const CHAR *szFileName )
 {
    HOLDFEA *pHoldFEA;		/* The start of the linked list */
-   
+
    CHAR *pAllocc = NULL;	/* Temp buffer used with DosEnumAttribute */
    CHAR *pBigAlloc = NULL;	/* Temp buffer to hold each EA as it is read in */
    USHORT cbBigAlloc = 0;
-   
+
    ULONG ulEntryNum = 1;	/* Count of current EA to read (1-relative) */
    ULONG ulEnumCnt;		/* Number of EAs for Enum to return, always 1 */
-   
+
    HOLDFEA *pLastIn = 0;	/* Points to last EA added, so new EA can link    */
    HOLDFEA *pNewFEA = NULL; /* Struct to build the new EA in                  */
-   
+
    FEA2 *pFEA;				/* Used to read from Enum's return buffer */
    GEA2LIST *pGEAList;		/* Ptr used to set up buffer for DosQueryPathInfo() */
    EAOP2 eaopGet;			/* Used to call DosQueryPathInfo() */
-   
+
    pAllocc = malloc( MAX_GEA );	/* Allocate room for a GEA list */
    pFEA = ( FEA2 * ) pAllocc;
    pHoldFEA = NULL;		/* Initialize the linked list */
-   
+
 	/* Loop through all the EA's adding them to the list */
    while( TRUE )
      {
@@ -773,14 +825,14 @@ HOLDFEA *QueryEAs( const CHAR *szFileName )
 			     ulEntryNum, pAllocc, MAX_GEA, &ulEnumCnt,
 			     ( LONG ) GET_INFO_LEVEL1 ) )
 	  break;	/* An error */
-	
+
 		/* Exit if all the EA's have been read */
 	if( ulEnumCnt != 1 )
 	  break;
-	
+
 		/* Move on to next EA */
 	ulEntryNum++;
-	
+
 		/* Try and allocate the HoldFEA structure */
 	if( ( pNewFEA = malloc( sizeof( HOLDFEA ) ) ) == NULL )
 	  {
@@ -788,16 +840,16 @@ HOLDFEA *QueryEAs( const CHAR *szFileName )
 	     Free_FEAList( pHoldFEA );
 	     return( NULL );
 	  }
-	
+
 		/* Fill in the HoldFEA structure */
 	pNewFEA->cbName = pFEA->cbName;
 	pNewFEA->cbValue= pFEA->cbValue;
 	pNewFEA->fEA = pFEA->fEA;
 	pNewFEA->next = '\0';
 /*		pNewFEA->next = NULL; */
-	
+
 		/* Allocate the two arrays */
-		if( ( pNewFEA->szName = malloc( pFEA->cbName + 1 ) ) == NULL || \
+	if( ( pNewFEA->szName = malloc( pFEA->cbName + 1 ) ) == NULL || \
 			( pNewFEA->aValue = malloc( pFEA->cbValue ) ) == NULL )
 	  {
 			/* Out of memory, clean up and exit */
@@ -805,14 +857,14 @@ HOLDFEA *QueryEAs( const CHAR *szFileName )
 	       free( pNewFEA->szName );
 	     if( pNewFEA->aValue )
 	       free( pNewFEA->aValue );
-	     
+
 	     free( pAllocc );
 	     free( pNewFEA );
-	     
+
 	     Free_FEAList( pHoldFEA );
 	     return( NULL );
 	  }
-	
+
 		/* Copy EA name across */
 	strcpy( pNewFEA->szName, pFEA->szName );
 	cbBigAlloc = sizeof( FEA2LIST ) + pNewFEA->cbName + 1 + pNewFEA->cbValue;
@@ -826,7 +878,7 @@ HOLDFEA *QueryEAs( const CHAR *szFileName )
 	     Free_FEAList( pHoldFEA );
 	     return( NULL );
 	  }
-	
+
 		/* Set up GEAlist structure */
 	pGEAList = ( GEA2LIST * ) pAllocc;
 	pGEAList->cbList = sizeof( GEA2LIST ) + pNewFEA->cbName;	/* + 1 for NULL */
@@ -835,22 +887,22 @@ HOLDFEA *QueryEAs( const CHAR *szFileName )
 #endif
 	pGEAList->list[ 0 ].cbName = pNewFEA->cbName;
 	strcpy( pGEAList->list[ 0 ].szName, pNewFEA->szName );
-	
+
 	eaopGet.fpGEA2List = ( GEA2LIST FAR * ) pAllocc;
 	eaopGet.fpFEA2List = ( FEA2LIST FAR * ) pBigAlloc;
-	
+
 	eaopGet.fpFEA2List->cbList = cbBigAlloc;
-	
+
 		/* Get the complete EA info and copy the EA value */
-		DosQueryPathInfo( szFileName, FIL_QUERYEASFROMLIST, ( PVOID ) &eaopGet, \
+	DosQueryPathInfo( szFileName, FIL_QUERYEASFROMLIST, ( PVOID ) &eaopGet, \
 						  sizeof( EAOP2 ) );
-		memcpy( pNewFEA->aValue, \
+	memcpy( pNewFEA->aValue, \
 				pBigAlloc + sizeof( FEA2LIST ) + pNewFEA->cbName, \
 				pNewFEA->cbValue );
-	
+
 		/* Release the temp. Enum buffer */
 	free( pBigAlloc );
-	
+
 		/* Add to the list */
 	if( pHoldFEA == NULL )
 	  pHoldFEA = pNewFEA;
@@ -858,14 +910,13 @@ HOLDFEA *QueryEAs( const CHAR *szFileName )
 	  pLastIn->next = pNewFEA;
 	pLastIn = pNewFEA;
      }
-   
+
 	/* Free up the GEA buffer for DosEnum() */
    free( pAllocc );
-   
+
    return pHoldFEA;
 }
 
-
 /* WriteEAs(szFileName, pHoldFEA)
 
    Write the EAs contained in the linked list pointed to by pHoldFEA
@@ -880,15 +931,15 @@ int WriteEAs( const char *szFileName, HOLDFEA *pHoldFEA )
    CHAR *aPtr = NULL;
    USHORT usMemNeeded;
    APIRET rc;
-   
+
    eaopWrite.fpGEA2List = NULL;
    while( pHFEA )                                  /* Go through each HoldFEA */
      {
 	usMemNeeded = sizeof( FEA2LIST ) + pHFEA->cbName + 1 + pHFEA->cbValue;
-	
+
 	if( ( aPtr = malloc( usMemNeeded ) ) == NULL )
 	  return FALSE;
-	
+
       /* Fill in eaop structure */
 	eaopWrite.fpFEA2List = ( FEA2LIST FAR * ) aPtr;
 	eaopWrite.fpFEA2List->cbList = usMemNeeded;
@@ -901,24 +952,24 @@ int WriteEAs( const char *szFileName, HOLDFEA *pHoldFEA )
 	eaopWrite.fpFEA2List->list[ 0 ].cbName = pHFEA->cbName;
 	eaopWrite.fpFEA2List->list[ 0 ].cbValue = pHFEA->cbValue;
 	strcpy( eaopWrite.fpFEA2List->list[ 0 ].szName, pHFEA->szName );
-      memcpy( eaopWrite.fpFEA2List->list[ 0 ].szName + pHFEA->cbName + 1, \
+	memcpy( eaopWrite.fpFEA2List->list[ 0 ].szName + pHFEA->cbName + 1, \
 	     pHFEA->aValue, pHFEA->cbValue );
-	
+
       /* Write out the EA */
 	rc = DosSetPathInfo( szFileName, FIL_QUERYEASIZE,
 			    ( PVOID ) &eaopWrite, sizeof( EAOP2 ),
 			    DSPI_WRTTHRU );
-	
+
       /* Free up the FEALIST struct */
 	free( aPtr );
-	
+
       /* If the write failed, leave now */
 	if( rc )
 	  return FALSE;
-	
+
 	pHFEA = pHFEA->next;
      }
-   
+
    return( TRUE );
 }
 
@@ -926,3 +977,81 @@ void sys_pause (int ms)
 {
    DosSleep(ms);
 }
+
+#ifdef	__IBMC__
+/* Adapted from the routines posted by Jochen Friedrich */
+
+/* Note: the pipe() function assumes that the C filedescriptors are
+ * equal to the OS/2 API filehandles... This is true for IBM VisualAge .0,
+ * but may be false for other compilers...
+ */
+int pipe(unsigned long fd[2])
+{
+   unsigned long	status;
+
+   if (DosCreatePipe(&(fd[0]), &(fd[1]), 4096) < 0)
+     return (-1);
+   if (DosQueryFHState(fd[0], &status) < 0)
+     return (-1);
+   status |= OPEN_FLAGS_NOINHERIT;
+   status &= 0x7F88;
+   if (DosSetFHState(fd[0], status) < 0)
+     return (-1);
+   if (DosQueryFHState(fd[1], &status) < 0)
+     return (-1);
+   status |= OPEN_FLAGS_NOINHERIT;
+   status &= 0x7F88;
+   if (DosSetFHState(fd[1], status) < 0)
+     return (-1);
+   _setmode(fd[0], O_BINARY);
+   _setmode(fd[1], O_BINARY);
+   return (0);
+}
+
+FILE *popen(char *cmd, char *mode)
+{
+   FILE		*our_end;
+   unsigned long	fd[2], tempfd, dupfd;
+   char		*comspec;
+
+   pipe(fd);
+
+   if (mode[0] == 'r')
+     {
+	our_end = fdopen(fd[0], mode);
+	dupfd = 1;	/* stdout */
+	tempfd = dup(dupfd);
+	dup2(fd[1], dupfd);
+	close(fd[1]);
+     }
+   else
+     {
+	our_end = fdopen(fd[1], mode);
+	dupfd = 0;	/* stdin */
+	tempfd = dup(dupfd);
+	dup2(fd[0], dupfd);
+	close(fd[0]);
+     }
+
+   comspec = getenv("COMSPEC");
+   if (comspec == NULL)
+     {
+	comspec = "C:\\OS2\\CMD.EXE";
+     }
+   spawnlp(P_NOWAIT, comspec, comspec, "/c", cmd, NULL);
+
+   dup2(tempfd, dupfd);
+   close(tempfd);
+
+   return (our_end);
+}
+
+int pclose(FILE *pip)
+{
+   int i;
+
+   fclose(pip);
+   wait(&i);
+   return (0);
+}
+#endif
