@@ -1,6 +1,8 @@
-/*
- *  Copyright (c) 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include "config.h"
 
@@ -31,7 +33,7 @@
 #ifdef PATH_MAX
 # define JED_MAX_PATH_LEN PATH_MAX
 #else
-# ifdef pc_system
+# ifdef IBMPC_SYSTEM
 #  define JED_MAX_PATH_LEN 256
 # else
 #  define JED_MAX_PATH_LEN 1024
@@ -39,7 +41,7 @@
 #endif
 
 
-#ifdef msdos
+#ifdef __MSDOS__
 # include <io.h>
 #endif
 
@@ -112,9 +114,9 @@ static void additional_help (void)
    .                  match any character except newline\n\
    \\d                match any digit\n\
    \\e                match ESC char\n\
-   *                  matches zero or more occurences of previous RE\n\
-   +                  matches one or more occurences of previous RE\n\
-   ?                  matches zero or one occurence of previous RE\n\
+   *                  matches zero or more occurences of last single char RE\n\
+   +                  matches one or more occurences of last single char RE\n\
+   ?                  matches zero or one occurence of last single char RE\n\
    ^                  matches beginning of line\n\
    $                  matches end of line\n\
    [ ... ]            matches any single character between brackets.\n\
@@ -199,7 +201,7 @@ static void parse_flags(char *f)
 	     Output_Match_Only = 1;
 	     break;
 	   case 'h':
-#ifndef pc_system
+#ifndef IBMPC_SYSTEM
 	     Highlight = 1;
 #endif
 	     break;
@@ -216,7 +218,7 @@ static void parse_flags(char *f)
 		  f++;
 	       }
 	     if (Fixed_Line_Len == 0) usage ();
-	     Fixed_Len_Buf = (unsigned char *) SLMALLOC (Fixed_Line_Len);
+	     Fixed_Len_Buf = (unsigned char *) SLmalloc (Fixed_Line_Len);
 	     if (Fixed_Len_Buf == NULL)
 	       {
 		  exit_error ("Malloc error.");
@@ -414,13 +416,17 @@ static void grep(char *file)
 	
 	match_found:
 	n_matches++;
-	
+
 	if (Count_Matches) continue;
 	if (File_Name_Only)
 	  {
+	     if (Print_Non_Matching_Lines)
+	       continue;
+
 	     puts(file);
 	     return;
 	  }
+
 	if (print_file_too)
 	  {
 	     fputs(file, stdout);
@@ -438,6 +444,16 @@ static void grep(char *file)
      }
    while (NULL != (buf = (unsigned char *) rgrep_gets(&n)));
 
+   if (Print_Non_Matching_Lines 
+       && File_Name_Only
+       && (n_matches == line))
+     {
+	puts (file);
+#ifdef __unix__
+	if (Stdout_Is_TTY) fflush (stdout);
+#endif
+     }
+
    if (n_matches && Count_Matches)
      {
 	if (print_file_too || File_Name_Only)
@@ -452,7 +468,7 @@ static void grep(char *file)
      }
 }
 
-#ifdef msdos
+#ifdef __MSDOS__
 # include <dir.h>
 #endif
 
@@ -462,7 +478,7 @@ static void grep(char *file)
 #endif
 
 
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
 typedef struct Dos_DTA_Type
 {
    unsigned char undoc[21];
@@ -502,7 +518,7 @@ typedef struct
    int dir_len;
    char *file;			       /* pointer to place in dir */
    int isdir;
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
    DOS_DTA_Type *dta;
    char pattern[16];
 #endif
@@ -513,7 +529,7 @@ typedef struct
 Sys_Dir_Type;
 
 
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
 void dos_set_dta (DOS_DTA_Type *dta)
 {
    asm mov ah, 0x1A
@@ -585,7 +601,7 @@ static int unix_is_dir (char *dir, int force_link)
 
 static Sys_Dir_Type *sys_opendir(char *dir, Sys_Dir_Type *x)
 {
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
    char slash = '\\';
    char *pat = "*.*";
    dos_set_dta (x->dta);
@@ -637,7 +653,7 @@ static Sys_Dir_Type *sys_opendir(char *dir, Sys_Dir_Type *x)
    x->dirp = dirp;
    strcpy(x->dir, dir);
 #endif /* __unix__ */
-#endif /* msdos */
+#endif /* IBMPC_USE_ASM */
    x->dir_len = strlen(x->dir);
    if (x->dir[x->dir_len - 1] != slash)
      {
@@ -651,7 +667,7 @@ static Sys_Dir_Type *sys_opendir(char *dir, Sys_Dir_Type *x)
 
 static void sys_closedir(Sys_Dir_Type *x)
 {
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
    (void) x;
 #else
 #ifdef __unix__
@@ -663,7 +679,7 @@ static void sys_closedir(Sys_Dir_Type *x)
 #endif
 }
 
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
 char *dos_dta_fixup_name (Sys_Dir_Type *x)
 {
    x->file = x->dir + x->dir_len;
@@ -677,7 +693,7 @@ char *dos_dta_fixup_name (Sys_Dir_Type *x)
 static char *sys_dir_findnext(Sys_Dir_Type *x)
 {
    char *file;
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
    asm mov ah, 0x4F
      asm int 21h
      asm jnc L1
@@ -704,7 +720,7 @@ static char *sys_dir_findnext(Sys_Dir_Type *x)
    strcpy (x->file, dp->d_name);
    x->isdir = unix_is_dir (x->dir, 0);
 #endif /* __unix__ */
-#endif /* msdos */
+#endif /* IBMPC_USE_ASM */
    /* exclude '.' and '..' */
    if (*file++ == '.')
      {
@@ -716,7 +732,7 @@ static char *sys_dir_findnext(Sys_Dir_Type *x)
 
 static char *sys_dir_findfirst(Sys_Dir_Type *x)
 {
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
    unsigned int attr = 0x1 | 0x10;     /* read only + sub directory */
    char pat[JED_MAX_PATH_LEN], *patp, *file;
    
@@ -765,7 +781,7 @@ static void grep_file(char *file, char *filename)
 	     p = filename + strlen(filename);
 	     while ((p >= filename) && (*p != '.')) p--;
 	     if ((*p != '.') ||
-#ifdef msdos
+#ifdef __MSDOS__
 		 stricmp(Match_This_Extension, p + 1)
 #else
 		 strcmp(Match_This_Extension, p + 1)
@@ -804,7 +820,7 @@ static void grep_dir(char *dir)
    static int depth;
    Sys_Dir_Type x;
    char *file;
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
    DOS_DTA_Type dta;
    x.dta = &dta;
 #endif
@@ -825,7 +841,7 @@ static void grep_dir(char *dir)
 	if (x.isdir == 0) grep_file(file, x.file);
 	else if ((Do_Recursive > 0) && (x.isdir == 1)) grep_dir(file);
 	
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
 	dos_set_dta (&dta);	       /* something might move it */
 #endif
      }
@@ -883,7 +899,7 @@ int main(int argc, char **argv)
 	     recurse_reg.pat = fixup_filename ((unsigned char *) *argv);
 	     recurse_reg.buf = recurse_buf;
 	     recurse_reg.buf_len = JED_MAX_PATH_LEN;
-#ifdef msdos
+#ifdef __MSDOS__
 	     recurse_reg.case_sensitive = 0;
 #else
 	     recurse_reg.case_sensitive = 1;
@@ -950,7 +966,7 @@ int main(int argc, char **argv)
 #ifdef __unix__
 	     int ret;
 #endif
-#ifdef msdos
+#ifdef __MSDOS__
 	       {
 		  char *p = *argv;
 		  while (*p)
@@ -961,7 +977,7 @@ int main(int argc, char **argv)
 	       }
 #endif
 	     if (
-#ifdef msdos
+#ifdef IBMPC_USE_ASM
 		 dos_is_dir (*argv)
 /*		 
  *		 && (('\\' == (*argv)[strlen(*argv) - 1])
@@ -980,7 +996,7 @@ int main(int argc, char **argv)
 	       }
 	     
 	     else
-#ifdef msdos
+#ifdef __MSDOS__
 	       {
 		  char *file = *argv;
 		  while (*file && (*file != '*')) file++;
