@@ -1,11 +1,13 @@
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #ifndef _JED_SIG_H_
 #define _JED_SIG_H_
 
-#ifndef msdos
+#ifndef __MSDOS_16BIT__
 extern void init_signals (void);
 #endif
 extern int Stdin_Is_TTY;
@@ -17,9 +19,4 @@ extern int Signal_Sys_Spawn_Flag;      /* used if something else sends stop */
 # endif
 #endif
 
-#ifdef HAS_RESIZE_PENDING
-extern volatile int Jed_Resize_Pending;
-extern void jed_resize_display (int);
-#endif
-
 #endif /* _JED_SIG_H_ */
