@@ -1,7 +1,9 @@
 /* -*- mode: C; mode: fold; -*- */
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 
 #include "config.h"
@@ -21,13 +23,7 @@
 
 #include <sys/time.h>
 
-#ifdef HAVE_TERMIOS_H
-# if !defined(HAVE_TCGETATTR) || !defined(HAVE_TCSETATTR)
-#  undef HAVE_TERMIOS_H
-# endif
-#endif
-
-#ifdef HAVE_TERMIOS_H
+#ifdef REALLY_HAVE_TERMIOS_H
 # include <termios.h>
 #endif
 
@@ -68,6 +64,10 @@
 # include <unistd.h>
 #endif
 
+#ifdef HAVE_FCNTL_H
+# include <fcntl.h>
+#endif
+
 #include <sys/file.h>
 #ifdef HAVE_SYS_FCNTL_H
 # include <sys/fcntl.h>
@@ -89,14 +89,13 @@ int Flow_Control;
 int Abort_Char = 7;		       /* scan code for G (control) */
 
 
-#ifndef HAVE_TERMIOS_H
+#ifndef REALLY_HAVE_TERMIOS_H
 typedef struct /*{{{*/
 {
    struct tchars t;
    struct ltchars lt;
    struct sgttyb s;
-};
-
+}
 /*}}}*/
 TTY_Termio_Type;
 #else
@@ -105,10 +104,14 @@ typedef struct termios TTY_Termio_Type;
 
 static TTY_Termio_Type Old_TTY;
 
-#ifdef HAVE_TERMIOS_H
+#ifdef REALLY_HAVE_TERMIOS_H
 # define GET_TERMIOS(fd, x) tcgetattr(fd, x)
+#if 0
 # define SET_TERMIOS(fd, x) tcsetattr(fd, TCSAFLUSH, x)
 #else
+# define SET_TERMIOS(fd, x) tcsetattr(fd, TCSADRAIN, x)
+#endif
+#else
 # ifdef TCGETS
 #  define GET_TERMIOS(fd, x) ioctl(fd, TCGETS, x)
 #  define SET_TERMIOS(fd, x) ioctl(fd, TCSETS, x)
@@ -125,6 +128,8 @@ static TTY_Termio_Type Old_TTY;
 # endif
 #endif
 
+#define NEED_BAUDRATES 0
+#if NEED_BAUDRATES
 static int Baud_Rates[20] = /*{{{*/
 {
    0, 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400, 4800,
@@ -132,7 +137,7 @@ static int Baud_Rates[20] = /*{{{*/
 };
 
 /*}}}*/
-
+#endif
 
 #ifdef HAS_MOUSE
 static int JMouse_Fd = -1;
@@ -168,7 +173,7 @@ static void handle_interrupt (void)
      return;
    
    while (Jed_Resize_Pending)
-     jed_resize_display (0);
+     jed_resize_display ();
 }
 
 int init_tty (void) /*{{{*/
@@ -207,7 +212,7 @@ int init_tty (void) /*{{{*/
 
    TTY_Inited = 1;
 
-#ifndef HAVE_TERMIOS_H
+#ifndef REALLY_HAVE_TERMIOS_H
    newtty.s.sg_flags &= ~(ECHO);
    newtty.s.sg_flags &= ~(CRMOD);
 /*   if (Flow_Control == 0) newtty.s.sg_flags &= ~IXON; */
@@ -220,7 +225,9 @@ int init_tty (void) /*{{{*/
    newtty.s.sg_flags |= CBREAK;		/* do I want cbreak or raw????? */
 #else
    
-   
+#if NEED_BAUDRATES
+   /* No need for this anymore since SLsmg is used */
+
    if (*tt_Baud_Rate == 0)
      {
 /* Note:  if this generates an compiler error, simply remove 
@@ -232,7 +239,8 @@ int init_tty (void) /*{{{*/
 	*tt_Baud_Rate = (*tt_Baud_Rate > 0) && (*tt_Baud_Rate < 19) ?
 	  Baud_Rates[*tt_Baud_Rate] : -1;
      }
-   
+#endif
+
    newtty.c_iflag &= ~(ECHO | INLCR | ICRNL);
 # ifdef ISTRIP
    /* allow 8 bit chars to pass through */
@@ -255,7 +263,7 @@ int init_tty (void) /*{{{*/
 # ifdef VSWTCH
    newtty.c_cc[VSWTCH] = NULL_VALUE;   /* to ignore who knows what */
 # endif
-#endif /* NOT HAVE_TERMIOS_H */
+#endif /* NOT REALLY_HAVE_TERMIOS_H */
    while (-1 == SET_TERMIOS(Read_FD, &newtty))
      {
 	if (errno != EINTR)
@@ -296,8 +304,10 @@ void reset_tty (void) /*{{{*/
    
    /* This statement ensures init_tty will not try to change output_rate 
       (when coming back from suspension) */
+#if NEED_BAUDRATES
    if (*tt_Baud_Rate == 0) *tt_Baud_Rate = -1;
-   
+#endif
+
 #ifdef HAS_MOUSE
    if (X_Close_Mouse_Hook != NULL) X_Close_Mouse_Hook ();
    JMouse_Fd = -1;
@@ -329,11 +339,11 @@ unsigned char sys_getkey (void) /*{{{*/
 	if (SLKeyBoard_Quit)
 	  break;
 	
-	/* update status line incase user is displaying time */
+	/* update status line in case user is displaying time */
 	if (Display_Time || all)
 	  {
 	     JWindow->trashed = 1;
-	     update((Line *) NULL, 0, 1);
+	     update((Line *) NULL, 0, 1, 0);
 	  }
      }
    
@@ -397,7 +407,7 @@ static fd_set Read_FD_Set;
 /* If all is zero, only check for mouse or keyboard input.  If all is -1,
  * check for only subprocess input. Otherwise, check all input.
  */
-static int sys_input_pending(int *tsecs, int all) /*{{{*/
+int sys_input_pending(int *tsecs, int all) /*{{{*/
 {
    struct timeval wait;
    long usecs, secs;
@@ -413,6 +423,7 @@ static int sys_input_pending(int *tsecs, int all) /*{{{*/
    
    top:
    
+   FD_ZERO(&Read_FD_Set);
    secs = *tsecs / 10;
    usecs = (*tsecs % 10) * 100000;
    wait.tv_sec = secs;
@@ -420,11 +431,10 @@ static int sys_input_pending(int *tsecs, int all) /*{{{*/
    
    if (all >= 0)
      {
-	if (X_Input_Pending_Hook != NULL)
-	  {
-	     if ((*X_Input_Pending_Hook) ()) return 1;
-	  }
-	
+	if ((X_Input_Pending_Hook != NULL)
+	    && (*X_Input_Pending_Hook) ())
+	  return 1;
+
 	FD_SET(Read_FD, &Read_FD_Set);
 #ifdef HAS_MOUSE
 	if ((JMouse_Fd >= 0) && (JMouse_Event_Hook != NULL))
@@ -452,6 +462,9 @@ static int sys_input_pending(int *tsecs, int all) /*{{{*/
    ret = select(maxfd + 1, &Read_FD_Set, NULL, NULL, &wait);
    
    if (ret == 0) return 0; 
+   
+   if ((ret == -1) && (errno == EINTR))
+     handle_interrupt ();
 
 #ifdef ENODEV
    if ((ret == -1) && (errno == ENODEV))
@@ -503,13 +516,16 @@ static int sys_input_pending(int *tsecs, int all) /*{{{*/
 	     if ((*X_Input_Pending_Hook) ()) return 1;
 	     
 	     /* Nothing there so try to time out again --- too bad select does
-	      * not inform of of how far we got. 
+	      * not inform of of how far we got.
 	      */
+	     goto top;
+#if 0
 	     FD_SET(Read_FD, &Read_FD_Set);
 	     (void) select(Read_FD + 1, &Read_FD_Set, NULL, NULL, &wait);
 	     /* try again, it could be more bogus Xpackets.  Event driven systems
 	      * are not always the way to go. */
 	     return (*X_Input_Pending_Hook) ();
+#endif
 	  }
 	
 	if (ret < 0) return ret;
@@ -542,7 +558,7 @@ static int sys_input_pending(int *tsecs, int all) /*{{{*/
    while (i < Num_Subprocesses)
      {
 	if (FD_ISSET(Subprocess_Read_fds[i][0], &Read_FD_Set))
-	  read_process_input (i);
+	  read_process_input (Subprocess_Read_fds[i][1]);
 	i++;
      }
 #endif
@@ -568,41 +584,6 @@ void sys_pause (int ms) /*{{{*/
 
 /*}}}*/
 
-int get_term_dimensions(int *cols, int *rows) /*{{{*/
-{
-#ifdef TIOCGWINSZ
-   struct winsize wind_struct;
-   
-   if (X_Get_Term_Size_Hook == NULL)
-     {
-	if ((ioctl(1,TIOCGWINSZ,&wind_struct) < 0)
-	    && (ioctl(2, TIOCGWINSZ, &wind_struct) < 0)
-	    && (ioctl(0, TIOCGWINSZ, &wind_struct) < 0))
-	  {
-	     *rows = *cols = 0;
-	  }
-	else
-	  {
-	     *cols = (int) wind_struct.ws_col;
-	     *rows = (int) wind_struct.ws_row;
-	  }
-     }
-   else (*X_Get_Term_Size_Hook)(cols, rows);
-   
-   if (*rows <= 0) *rows = *tt_Screen_Rows;
-   if (*cols <= 0) *cols = *tt_Screen_Cols;
-#else
-   if (X_Get_Term_Size_Hook == NULL)
-     {
-	*rows = *tt_Screen_Rows;
-	*cols = *tt_Screen_Cols;
-     }
-   else (*X_Get_Term_Size_Hook)(cols, rows);
-#endif
-   return 0;
-}
-
-/*}}}*/
 
 /* returns 0 on failure, 1 on sucess */
 int sys_delete_file(char *filename) /*{{{*/
@@ -680,15 +661,6 @@ int sys_chmod(char *file, int what, int *mode, short *uid, short *gid) /*{{{*/
 
 /*}}}*/
 
-unsigned long sys_file_mod_time(char *file) /*{{{*/
-{
-   struct stat buf;
-   
-   if (stat(file, &buf) < 0) return(0);
-   return((unsigned long) buf.st_mtime);
-}
-
-/*}}}*/
 
 /* use berkeley interface
  * #include <sys/dir.h>
@@ -751,9 +723,16 @@ int sys_findnext(char *file) /*{{{*/
 {
    struct dirent *dp;
    
+   if (Dirp == NULL)
+     return 0;
+
    while (1)
      {
-	if (NULL == (dp = readdir(Dirp))) return(0);
+	if (NULL == (dp = readdir(Dirp))) 
+	  {
+	     closedir (Dirp); Dirp = NULL;
+	     return 0;
+	  }
 #ifdef NEED_D_NAMLEN
 	dp->d_name[dp->d_namlen] = 0;
 #endif
@@ -801,7 +780,7 @@ void get_passwd_info (char *name) /*{{{*/
    
    if (*name == 0)
      {
-	name = getlogin ();
+	name = (char *) getlogin ();
 	if (name == NULL) name = getenv ("LOGNAME");
 	if (name == NULL) name = getenv ("USER");
      }
