@@ -1,7 +1,9 @@
 /* -*- mode: C; mode: fold; -*- */
-/*
- *  Copyright (c) 1992, 1995 John E. Davis  (davis@space.mit.edu)
- *  All Rights Reserved.
+/* Copyright (c) 1992, 1998 John E. Davis
+ * This file is part of JED editor library source.
+ *
+ * You may distribute this file under the terms the GNU General Public
+ * License.  See the file COPYING for more information.
  */
 #include "config.h"
 #include "jed-feat.h"
@@ -73,7 +75,7 @@ void vms_exit_handler(int not_used) /*{{{*/
    if (TTY_Inited == 0) exit(0);
 
    auto_save_all();
-   reset_display(); 
+   jed_reset_display(); 
    reset_tty(); 
    exit(1);
 }
@@ -315,11 +317,6 @@ int init_tty() /*{{{*/
 
 /*}}}*/
 
-void sys_flush(int fd) /*{{{*/
-{
-}
-
-/*}}}*/
 
 static void cancel_ast (void) /*{{{*/
 {
@@ -373,6 +370,8 @@ void reset_tty() /*{{{*/
 
    if (keypad_state) SLtt_write_string("\033=\033[?1l");
    else SLtt_write_string("\033>");
+   
+   SLtt_flush_output ();
 }
 
 /*}}}*/
@@ -402,7 +401,7 @@ unsigned char sys_getkey() /*{{{*/
 	if (Display_Time)
 	  {
 	     JWindow->trashed = 1;
-	     update((Line *) NULL, 0, 1);
+	     update((Line *) NULL, 0, 1, 0);
 	  }
      }
    c = my_getkey();
@@ -412,7 +411,7 @@ unsigned char sys_getkey() /*{{{*/
 /*}}}*/
 
 /* waits *secs tenth of seconds for input */
-static int sys_input_pending(int *secs, int unused) /*{{{*/
+int sys_input_pending(int *secs, int unused) /*{{{*/
 {
    unsigned long daytim[2];
    
@@ -475,7 +474,7 @@ void sys_pause (int ms) /*{{{*/
 }
 
 /*}}}*/
-
+#if 0
 /*  This is to get the size of the terminal  */
 int get_term_dimensions(int *cols, int *rows) /*{{{*/
 {
@@ -504,13 +503,7 @@ int get_term_dimensions(int *cols, int *rows) /*{{{*/
    itmlst[3].buf_addr = 0;
    itmlst[3].len_addr = 0;
 
-   if (X_Get_Term_Size_Hook != NULL)
-     {
-	(*X_Get_Term_Size_Hook)(cols, rows);
-	return;
-     }
-   
-   if (*rows <= 0) *rows = *tt_Screen_Rows;
+   if (*rows <= 0) *rows = Jed_Num_Screen_Rows;
    if (*cols <= 0) *cols = *tt_Screen_Cols;
 
    /* Get current terminal characteristics */
@@ -529,6 +522,7 @@ int get_term_dimensions(int *cols, int *rows) /*{{{*/
 }
 
 /*}}}*/
+#endif
 
 /* returns 0 on failure, 1 on sucess */
 int sys_delete_file(char *filename) /*{{{*/
@@ -538,12 +532,6 @@ int sys_delete_file(char *filename) /*{{{*/
 
 /*}}}*/
 
-int sys_rename(char *from, char *to) /*{{{*/
-{
-    return(-1);
-}
-
-/*}}}*/
 
 /* This routine converts unix type names to vms names */
 int locate(char ch, char *string) /*{{{*/
@@ -710,7 +698,6 @@ void sys_suspend() /*{{{*/
 {
    unsigned long parent_pid;
    unsigned long status = 0;
-   char str[80];
 
    cancel_ast ();
    parent_pid = getppid();
@@ -735,8 +722,7 @@ void sys_suspend() /*{{{*/
 	/* if we attach back, status may come back unchanged */
 	if (!(status & 1))  /* Thanks to Hunter Goatley for this suggestion */
 	  {              
-	     sprintf(str,"Unable to spawn subprocess. Error = X%X (%d)", status, status);
-	     msg_error(str);
+	     jed_verror ("Unable to spawn subprocess. Error = X%X (%d)", status, status);
 	     return;
 	  }
      }
@@ -902,7 +888,7 @@ int vms_expand_filename(char *file,char *expanded_file) /*{{{*/
     if (RMS$_NORMAL == lib$find_file(&file_desc,&result,&context,
 	           		     &default_dsc,0,0,&Number_Zero))
       {
-	  MEMCPY(expanded_file, result.dsc$a_pointer, result.dsc$w_length);
+	  SLMEMCPY(expanded_file, result.dsc$a_pointer, result.dsc$w_length);
 	  expanded_file[result.dsc$w_length] = '\0';
           return (1);
       }
@@ -931,7 +917,7 @@ int sys_findnext(char *file) /*{{{*/
    if (RMS$_NORMAL == lib$find_file(&file_desc,&result,&context,
 				    &default_dsc,0,0,&Number_Zero))
      {
-	MEMCPY(file, result.dsc$a_pointer, result.dsc$w_length);
+	SLMEMCPY(file, result.dsc$a_pointer, result.dsc$w_length);
 	file[result.dsc$w_length] = 0;
 	f = &file[result.dsc$w_length];
 	if (*VMS_Star)
@@ -1019,16 +1005,6 @@ int sys_chmod(char *file, int what, int *mode, short *uid, short *gid) /*{{{*/
 
 /*}}}*/
 
-unsigned long sys_file_mod_time(char *file) /*{{{*/
-{
-   struct stat buf;
-
-   if (stat(file, &buf) < 0) return(0);
-   return((unsigned long) buf.st_mtime);
-}
-
-/*}}}*/
-
 #if (__VMS_VER < 70000000)
 int rmdir (char *d) /*{{{*/
 {
